#!/usr/bin/env python -tt
"""
Below Replace NAME with the word in the file name mcms_NAME_base_defs.py

Base NAME definitions for the MCMS project.

Provides name structure for NAME used by the MCMS project. 

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.
		
Examples:
^^^^^^^^

Notes/Warnings: 
^^^^^^^^^^^^^^
Users need not alter this file to add new NAME for their use, 
rather they can create modify the NAME_defs.py file instead.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports
import util.pick_factory

# ----------------------------------------------------------------------------

# Python dictionary defining the long name and short name for the SLP
#   sources.
#
#   {"key" : ["Long Name/Title","Short Name"]}
#
#   Keys are short unique words that are used to refer to the wanted selection.
#   Long Names are longer phrases describing each option, these are sometimes
#       used as titles for plots and such.
#   Short Names much like the keys (often the same) a short identifier used
#       for file naming and such.
#
# Map months to seasons
selections = { "annual" : ["Annual",{1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0}],
        "3mo" : ["Three Month Seasons",{ 1 : 1, 2 : 1, 3 : 2, 4 : 2, 5 : 2, 6 : 3, 7 : 3, 8 : 3, 9 : 4,
            10 : 4, 11 : 4, 12 : 1}],
        "6mo" : ["Six Month Seasons",{ 1 : 1, 2 : 1, 3 : 1, 4 : 1, 5 : 2, 6 : 2, 7 : 2, 8 : 2, 9 : 2,
            10 : 2, 11 : 1, 12 : 1}]}          
NAME = __file__.split("_")[1]
path = os.path.abspath(os.path.dirname(__file__))
fname = "%s/%s.py" % (path,"%s_defs" % NAME)

def define_m2s():
    return util.pick_factory.define_something(fname,selections,NAME)
 
#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    verbose = True
    pick_m2s = define_m2s()
    msg = pick_m2s.make_pick("3mo")
    if verbose: print msg
    m2s = pick_m2s.selection