#!/usr/bin/env python -tt
"""
Lists of statistics to collect in mcms_check.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports

# ----------------------------------------------------------------------------
def define_mapgroup(att,tracks,hemis):
    # Standard statistics for maps
    map_group = ["cnts","slp","intensity","bearing",
            "laplacian","dissimilarity"]
    if att:
        map_group.extend(["empty","att-area","area-total","att-depth","sep",
            "sep-alt","atts-area","att-perimeter","atts-perimeter",
            "atts-depth","depth-total","perimeter-total","entangled"])
    if tracks:
        map_group.extend(["track-length","track-age","speed","fill","residence"])
    map_group.sort()

    map_group_stuff = {}
    # An example of what is needed
    #    map_group_stuff['example'] = {
    #            # Used to title plot, if empty '' then no title added to plot.
    #            'title' : "Map of Something (units)",
    #            # Determines if plot is a 'contour' or 'pcolor' (filled grids).
    #            'plot_type' : 'pcolor',
    #            # Create a single global Cylindrical Equidistant map or
    #            #   2 hemispheric polar Lambert Conformal plots.
    #            'polar' : True,
    #            # Lower latitude bounds for polar plots [nh,sh].
    #            'bounding_lat' : [15.-15.],
    #            # Center longitude of the plots (set to "" to use lons[0]).
    #            'lon_0' : "",
    #            # Smooth the data for polar contour plots only.
    #            'smooth' : True,
    #            # If plot_type is contour make the contours filled rather than just lines.
    #            'filled' : False,
    #            # Color scheme for plots
    #            #    see http://matplotlib.sourceforge.net/examples/pylab_examples/show_colormaps.html
    #            #    append _r to reverse the color order.
    #            'color_scheme' : "jet_r",
    #            # Do not plot a colorbar for 'pcolor' or filled contour plots.
    #            'nocolorbar' : False,
    #            # Add a label to the colorbar (default "").
    #            'colorbar_label' : "",
    #            # Make the colorbar from a limited number (set by discrete) values rather
    #            #   than a continuous color gradation.
    #            'discrete' : False,
    #            # List [min,max] used by pcolor plot_type for range of colors.
    #            #   if empty the min/max of the data are used.
    #            'cints' : [0.0,1.0],
    #            # List [min,max,interval] used by contour plot_type for range of contours.
    #            #   if empty the min/max of the data are used and some calculated interval
    #            #   is determined by the plotting package.
    #            'clevs' : [0,10,2],
    #            # Label contours.
    #            'clabels' : True,
    #            # Value used to mask missing data in plot (default -1).
    #            'missing' : -1,
    #            # Normalize by time steps (i.e. frequency of occurrence %).
    #            'normalize' : False,
    #            }
    # Empty Template for your use.
    #    map_group_stuff[''] = {'title' : "",'plot_type' : '','polar' : True,
    #            'bounding_lat' : [15.-15.], 'lon_0' : "",'smooth' : True,'filled' : False,
    #            'color_scheme' : "jet_r",'nocolorbar' : False,'colorbar_label' : "",
    #            'discrete' : False,'cints' : [0.0,1.0],'clevs' : [0,10,2],
    #            'clabels' : True,'missing' : -1,'normalize' : False}

    # Quick way to set defaults for everything and then use modify what needs it.
    default_title = ""
    ptypes = ['pcolor','contour','imshow']
    # Note if there is missing data don't use pcolor... use imshow for grid view.
    default_plot_type = ptypes[1]
    default_polar = True
    if default_polar and "nh" not in hemis or "sh" not in hemis:
        default_polar = False
        print "Sorry: hemis %s doesn't support polar plots" % repr(hemis)
    default_bounding_lat = []
    for h in hemis:
        if h not in ["nh","sh"]:
            default_bounding_lat.append(0.0)
        elif h is "nh":
            default_bounding_lat.append(15.0)
        else:
            default_bounding_lat.append(-15.0)
    default_lon_0 = 180.0
    default_smooth = True
    default_filled = 1
    default_color_scheme = "jet"
    default_nocolorbar = False
    default_colorbar_label = ""
    default_discrete = False
    default_cints = []
    default_clevs = []
    default_clabels = True
    default_missing = -1
    default_normalize = False
    map_defaults = [default_title,default_plot_type,default_polar,default_bounding_lat,
            default_lon_0,default_smooth,default_filled,default_color_scheme,
            default_nocolorbar,default_colorbar_label,default_discrete,default_cints,
            default_clevs,default_clabels,default_missing,default_normalize]
    map_group_base = ['title','plot_type','polar','bounding_lat','lon_0','smooth',
            'filled','color_scheme','nocolorbar','colorbar_label','discrete',
            'cints','clevs','clabels','missing','normalize']
    if len(map_defaults) != len(map_group_base):
        sys.exit("map_defaults != map_group_base")
    map_group_base = dict(zip(map_group_base,map_defaults))
    #map_group_stuff = dict.fromkeys(map_group[:],map_group_base)
    map_group_stuff = dict.fromkeys(map_group[:],1)
    for i in map_group_stuff.keys():
        # Need this elsewise map_group_base all the same object!
        map_group_stuff[i] = map_group_base.copy()

    # Tweak individual items
    map_group_stuff["cnts"]['title'] = "Occurrence of Cyclone Centers (% of Total Time Steps)"
    map_group_stuff["cnts"]['cints'] = [5.0,85.0]
    map_group_stuff["cnts"]['clevs'] = [10,90,5]
    map_group_stuff["cnts"]['normalize'] = True

    map_group_stuff["slp"]['title'] = "Central SLP (hPa)"
    map_group_stuff["slp"]['cints'] = [960.0,1015.0]
    map_group_stuff["slp"]['clevs'] = [940,1020,2]
    #map_group_stuff["slp"]['clevs'] = [960,1015,2]

    map_group_stuff["intensity"]['title'] = "Center Intensity Class (1 = Weak 2 = Moderate 3 = Strong)"
    map_group_stuff["intensity"]['cints'] = [1.4,3.0]
    map_group_stuff["intensity"]['clevs'] = [1.0,3.0,0.2]

    map_group_stuff["bearing"]['title'] = "Angle to Next Center Along Track (degrees)"
    map_group_stuff["bearing"]['cints'] = [45.0,160.0]
    map_group_stuff["bearing"]['clevs'] = [45,160,5]

    map_group_stuff["laplacian"]['title'] = 'Center Laplacian ' + r"(hPa $^\circ$Lat$^{-2}$)"
    map_group_stuff["laplacian"]['cints'] = [0.2,1.2]
    map_group_stuff["laplacian"]['clevs'] = [0,1.5,0.05]

    map_group_stuff["dissimilarity"]['title'] = "Track Dissimilarity"
    map_group_stuff["dissimilarity"]['cints'] = [0.0,0.5]
    map_group_stuff["dissimilarity"]['clevs'] = [0.05,0.6,0.05]

    if att:
        map_group_stuff["empty"]['title'] = "Occurrence of Empty Cyclones (% of Total Time Steps)"
        map_group_stuff["empty"]['cints'] = [0.0,1.]
        map_group_stuff["empty"]['clevs'] = [0,2.5,0.1]
        map_group_stuff["empty"]['normalize'] = True

        map_group_stuff["att-area"]['title'] = "Cyclone ATT Area (Radius of Equivalent Circle, km)"
        map_group_stuff["att-area"]['cints'] = [200.0,1000.0]
        map_group_stuff["att-area"]['clevs'] = [400,1200,50]

        map_group_stuff["att-perimeter"]['title'] = "Perimeter Outermost ATT Contour (km)"
        map_group_stuff["att-perimeter"]['cints'] = [4000.0,11000.0]
        map_group_stuff["att-perimeter"]['clevs'] = [2000,12000,200]

        map_group_stuff["att-depth"]['title'] = "Cyclone ATT Depth (hPa)"
        map_group_stuff["att-depth"]['cints'] = [2.0,20.0]
        map_group_stuff["att-depth"]['clevs'] = [4,36,2]

        map_group_stuff["atts-area"]['title'] = "Cyclone ATTS Area (Radius of Equivalent Circle, km)"
        map_group_stuff["atts-area"]['cints'] = [700.0,1400.0]
        map_group_stuff["atts-area"]['clevs'] = [600,2000,50]

        map_group_stuff["atts-perimeter"]['title'] = "Perimeter Outermost ATTS Contour (km)"
        map_group_stuff["atts-perimeter"]['cints'] = [7000.0,14000.0]
        map_group_stuff["atts-perimeter"]['clevs'] = [7000,14000,200]

        map_group_stuff["atts-depth"]['title'] = "Cyclone ATTS Depth (hPa)"
        map_group_stuff["atts-depth"]['cints'] = [2.0,36.0]
        map_group_stuff["atts-depth"]['clevs'] = [4,40,2]

        map_group_stuff["area-total"]['title'] = "Total Cyclone Area (Radius of Equivalent Circle, km)"
        map_group_stuff["area-total"]['cints'] = [800.0,1600.0]
        map_group_stuff["area-total"]['clevs'] = [400,2000,50]

        map_group_stuff["depth-total"]['title'] = "Cyclone Depth (hPa)"
        map_group_stuff["depth-total"]['cints'] = [2.0,36.0]
        map_group_stuff["depth-total"]['clevs'] = [4,36,2]

        map_group_stuff["perimeter-total"]['title'] = "Perimeter Outermost Contour (km)"
        map_group_stuff["perimeter-total"]['cints'] = [4000.0,14000.0]
        map_group_stuff["perimeter-total"]['clevs'] = [2000,14000,200]

        map_group_stuff["sep"]['title'] = "Distance to Nearest Center (Zonal Wavenumber)"
        map_group_stuff["sep"]['cints'] = [3.0,18.0]
        map_group_stuff["sep"]['clevs'] = [1,18,1]

        map_group_stuff["sep-alt"]['title'] = "Distance to Nearest Center Alt (Zonal Wavenumber)"
        map_group_stuff["sep-alt"]['cints'] = [2.0,15.0]
        map_group_stuff["sep-alt"]['clevs'] = [1,18,1]

        map_group_stuff["entangled"]['title'] = "Number of Entangled Centers in ATTS"
        map_group_stuff["entangled"]['cints'] = [2.0,4.0]
        map_group_stuff["entangled"]['clevs'] = [2.0,4.0,0.1]

    if tracks:
        map_group_stuff["track-age"]['title'] = "Cyclone Age (Days from Track Inception)"
        map_group_stuff["track-age"]['cints'] = [0.5,5.0]
        map_group_stuff["track-age"]['clevs'] = [0.5,6,0.5]

        map_group_stuff["track-length"]['title'] = "Average Track Length Per Time Step (Days)"
        map_group_stuff["track-length"]['cints'] = [0.5,6.0]
        map_group_stuff["track-length"]['clevs'] = [0.5,7,0.5]

        map_group_stuff["speed"]['title'] = "Track Speed (km/hr)"
        map_group_stuff["speed"]['cints'] = [0.,125.0]
        map_group_stuff["speed"]['clevs'] = [0,125,5]

        map_group_stuff["fill"]['title'] = "Track Deepening/Filling Rate (hPa/day)"
        map_group_stuff["fill"]['cints'] = [-20.0,20.0]
        map_group_stuff["fill"]['clevs'] = [-20,20,1]

        # Note: This measure has issues:
        #   Cyclone location is sampled. Thus we do not know cyclone locations
        #   between samples. Indeed, for fast moving systems we don't know
        #   for sure which gridID where occupied between samples nor for how
        #   long. The way residence is measured here we accrue everything in
        #   the cyclone location at each sample. This over estimates the
        #   true residence time the fast cyclone move. Also the gridIDs that
        #   mobile cyclones skip between samples are not accounted for at all.
        #
#        #   Also, this is a track averaged value. That is, for slow moving
#        #   systems the average reduces the impact on the total count.
        map_group_stuff["residence"]['title'] = "Residence Time (Days)"
        map_group_stuff["residence"]['cints'] = [0.0,10.0]
        map_group_stuff["residence"]['clevs'] = [0.0,10.0,0.25]

    a = set(map_group_stuff.keys())
    b = set(map_group)
    c = a.symmetric_difference(b)
    if len(c):
        c = sorted(list(c))
        msg = "\tWarning map_group_stuff and map_group differ.\n\t\tDiffering: %s"
        sys.exit(msg % (repr(c)))

    # Add some for frequency of occurrence
    map_group_stuff['att-freq'] = map_group_base.copy()
    map_group_stuff['att-freq']['title'] = "Occurrence of ATT Grids (% of Total Time Steps)"
    map_group_stuff["att-freq"]['cints'] = [0.0,40.0]
    map_group_stuff["att-freq"]['clevs'] = [0,40,2.5]
    map_group_stuff["att-freq"]['normalize'] = True

    map_group_stuff['atts-freq'] = map_group_base.copy()
    map_group_stuff['atts-freq']['title'] = "Occurrence of ATTS Grids (% of Total Time Steps)"
    map_group_stuff["atts-freq"]['cints'] = [0.0,25.0]
    map_group_stuff["atts-freq"]['clevs'] = [0,30,2.5]
    map_group_stuff["atts-freq"]['normalize'] = True

    map_group_stuff['storm-freq'] = map_group_base.copy()
    map_group_stuff['storm-freq']['title'] = "Occurrence of ATT + ATTS Grids (% of Total Time Steps)"
    map_group_stuff["storm-freq"]['cints'] = [5.0,75.0]
    map_group_stuff["storm-freq"]['clevs'] = [5,90,5]
    map_group_stuff["storm-freq"]['normalize'] = True

    return (map_group,map_group_stuff)

#---Start of main code block. This show how to use the above code
if __name__=='__main__':
    verbose = 1
    att = 1
    tracks = 0
    hemis = ['global', 'nh', 'sh']
    mgroups = define_mapgroup(att,tracks,hemis)
    if verbose:
        print mgroups