#!/usr/bin/env python -tt
"""
Below Replace NAME with the word in the file name mcms_NAME_base_defs.py

Base NAME definitions for the MCMS project.

Provides name structure for NAME used by the MCMS project. 

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.
		
Examples:
^^^^^^^^

Notes/Warnings: 
^^^^^^^^^^^^^^
Users need not alter this file to add new NAME for their use, 
rather they can create modify the NAME_defs.py file instead.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports
import util.pick_factory

# ----------------------------------------------------------------------------

# Python dictionary defining the long name and short name for the SLP
#   sources.
#
#   {"key" : ["Long Name/Title","Short Name"]}
#
#   Keys are short unique words that are used to refer to the wanted selection.
#   Long Names are longer phrases describing each option, these are sometimes
#       used as titles for plots and such.
#   Short Names much like the keys (often the same) a short identifier used
#       for file naming and such.
#

selections = {"annual" : ["Annual",["annual"]],
            "3mo" : ["Three Month Seasons",["annual","djf","mam","jja","son"]],
            "6mo" : ["Six Month Seasons",["annual","ndjfma","mjjaso"]],
            }        
NAME = __file__.split("_")[1]
path = os.path.abspath(os.path.dirname(__file__))
fname = "%s/%s.py" % (path,"%s_defs" % NAME)

def define_season():
    return util.pick_factory.define_something(fname,selections,NAME)
 
#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    verbose = True
    pick_season = define_season()
    msg = pick_season.make_pick("3mo")
    if verbose: print msg
    season = pick_season.selection
