#!/usr/bin/env python -tt
"""
Below Replace NAME with the word in the file name mcms_NAME_base_defs.py

Base NAME definitions for the MCMS project.

Provides name structure for NAME used by the MCMS project.

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^
Users need not alter this file to add new NAME for their use,
rather they can create modify the NAME_defs.py file instead.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports
import util.pick_factory

# ----------------------------------------------------------------------------

# Python dictionary defining the long name and short name for the SLP
#   sources.
#
#   {"key" : ["Long Name/Title","Short Name"]}
#
#   Keys are short unique words that are used to refer to the wanted selection.
#   Long Names are longer phrases describing each option, these are sometimes
#       used as titles for plots and such.
#   Short Names much like the keys (often the same) a short identifier used
#       for file naming and such.
#
selections = {"test": ["Test/Debug","test"],
    "nra" : ["NCEP/NCAR Reanalysis 1","nra"],
    "nra2" : ["NCEP-DOE Reanalysis 2","nra2"],
    "erai" : ["ERA-Interim Reanalysis","erai"],
    "era40" : ["ERA-40 Reanalysis","era40"],
    "merrac" : ["Modern Era Retrospective-Analysis for Research and Applications (Reduced Resolution C)","merrac"],
    "merran" : ["Modern Era Retrospective-Analysis for Research and Applications (Native Resolution N)","merran"],
    "ccsm" : ["NCAR Community Climate System Model","ccsm"],
    "gissm" : ["NASA GISS ModelE M20","gissm"],
    "ipsl" : ["Institut Pierre Simon Laplace, Paris, France","ipsl"]
    }
NAME = __file__.split("_")[1]
path = os.path.abspath(os.path.dirname(__file__))
fname = "%s/%s.py" % (path,"%s_defs" % NAME)

def define_source():
    return util.pick_factory.define_something(fname,selections,NAME)

#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    verbose = True
    pick_source = define_source()
    msg = pick_source.make_pick("test")
    if verbose: print msg
    model = pick_source.selection
