#!/usr/bin/env python -tt
"""
Lists of statistics to collect in mcms_check.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.
		
Examples:
^^^^^^^^

Notes/Warnings: 
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports

# ----------------------------------------------------------------------------
def define_statgroup(att,tracks):
    # Standard statistics for histograms   
    stat_group = ["cnts-diffs","cnts","lons","lats","gridids","slp",
            "dissimilarity","laplacian","intensity"]
    # Add if attributed data wanted
    att_group = ["natts","att-area","area-total","att-depth","sep",
            "sep-alt","empty","atts-area","att-perimeter","atts-perimeter",
            "entangled","natt","atts-depth","depth-total","perimeter-total",
            "att-minedge","att-maxedge","att-aveedge","atts-minedge",
            "atts-maxedge","atts-aveedge"]
    # Add if track data wanted
    track_group = ["new-usi","lost-usi","continued-usi","track-length",
            "track-ave","track-age","bearing","speed","fill"]
    if att:
        stat_group.extend(att_group)
    if tracks:
        stat_group.extend(track_group)
    stat_group.sort()

    # Statistics that are per timestep bases rather than per center/cyclone
    #   based
    per_step_group = ["cnts-diffs","cnts","empty",
            "new-usi","lost-usi","continued-usi","track-ave"]
    if not att:
        per_step_group = [x for x in per_step_group if x not in att_group]
    if not tracks:
        per_step_group = [x for x in per_step_group if x not in track_group]

    # Statistics from stat_group for which intensity break down is not done.
    #   In some cases (as in tracks) it's not clear what intensity to use
    #   and in others it doesn't need useful to do.
    sans_intensity = ["empty","cnts-diffs","cnts","new-usi","lost-usi",
            "continued-usi","track-ave","track-length","intensity"]
    sans_intensity.sort()

    # How data is stored and read
    as_ints = ["cnts-diffs","cnts","gridids","natts","empty","new-usi",
            "lost-usi","continued-usi",'intensity',"natt","att-area",
            "area-total","att-depth","sep","sep-alt","atts-area",
            "att-perimeter","atts-perimeter","entangled","atts-depth",
            "depth-total","perimeter-total","att-minedge","att-maxedge",
            "att-aveedge","atts-minedge","atts-maxedge","atts-aveedge"]
    as_ints.sort()
    as_float = ["lons","lats","slp","track-length","track-ave","track-age",
            "bearing","dissimilarity","laplacian","speed","fill"]
    as_float.sort()
    if list(set(as_ints) & set(as_float)):
        msg = "\tNon-Empty Intersection of as_ints and as_floats: %s"
        sys.exit(msg % repr(list(set(as_ints) & set(as_float))))
    
    # Filter out only needed
    as_ints = [x for x in as_ints if x in stat_group]
    as_float = [x for x in  as_float if x in stat_group]
    if len(stat_group) != len(as_ints)+len(as_float):
        msg = "\tMismatch between stat_group and as_ints and as_floats: %d %d"
        sys.exit(msg % (len(stat_group),len(as_ints)+len(as_float)))

    # Note: track-length is the total length of a cyclone, track-age is the
    #   amount of time since cyclogenesis a cyclone is at a give timestep
    #   (that is at this point along the track this cyclone is 24 hours old),
    #   track-ave is the average track-age of all the current cyclone at a
    #   give timestep (how old the typical cyclone at any given time). So
    #   track-age is a meaningless histogram, but interesting map and the
    #   opposite for track-ave
    # Cull track-age from stat_group, kept for maps
    if tracks:
        stat_group.remove("track-age")
        
    return (stat_group,att_group,per_step_group,sans_intensity,as_float,as_ints)
    
#---Start of main code block. This show how to use the above code
if __name__=='__main__':
    verbose = 1
    att = 1
    tracks = 0
    sgroups = define_statgroup(att,tracks)
    if verbose:
        print sgroups