import sys
import types
class ProgressBar:
    """ ascii command-line progress bar with percentage and elapsed time display
    adapted from Pylot source code (original by Vasil Vangelovski)
    modified by Corey Goldberg - 2010"""

    def __init__(self,duration,label=""):
        self.duration = duration
        self.label = label
        self.prog_bar = '\t%s[]' % self.label
        self.fill_char = '#'
        self.width = 40
        self.__update_amount(0,"")
    
    def __update_amount(self,new_amount,which_one):
        label = "\t%s %s [" % (self.label,which_one)
        percent_done = int(round((new_amount / 100.0) * 100.0))
        all_full = self.width - 2
        num_hashes = int(round((percent_done / 100.0) * all_full))
        self.prog_bar = label + self.fill_char * num_hashes + ' ' * (all_full - num_hashes) + ']'
        pct_place = ((len(self.prog_bar)+len(label))/ 2) - len(str(percent_done))
        pct_string = '%i%%' % percent_done
        self.prog_bar = self.prog_bar[0:pct_place] + \
            (pct_string + self.prog_bar[pct_place + len(pct_string):])
        
    def update_time(self, elapsed_secs, which_one=""):
        self.__update_amount((elapsed_secs / float(self.duration)) * 100.0, which_one)
        self.prog_bar += '  %d/%s' % (elapsed_secs, self.duration)
        
    def __str__(self):
        return str(self.prog_bar)

def ProgressBarAlt(it, prefix = "", size = 60):
    # This does not work with generator type expressions.
    count = len(it)

    def _show(_i):
        x = int(size*_i/count)
        sys.stdout.write("%s[%s%s] %i/%i\r" % (prefix, "#"*x, "."*(size-x), _i, count))
        sys.stdout.flush()
    
    _show(0)
    for i, item in enumerate(it):
        yield item
        _show(i+1)
    sys.stdout.write("\n")
    sys.stdout.flush()

def ProgressBarGen(it, count, prefix = "", size = 60):
    
    # Modified mbauer:
    #   Allows for generators and lists rather than just lists.
    def _show(_i):
        x = int(size*_i/count)
        sys.stdout.write("%s[%s%s] %i/%i\r" % (prefix, "#"*x, "."*(size-x), _i, count))
        sys.stdout.flush()
    
    _show(0)
    for i, item in enumerate(it):
        yield item
        _show(i+1)
    sys.stdout.write("\n")
    sys.stdout.flush()

