class DefineHistogramStuff:
    '''Class to define, setup and plot histograms and time series
    for mcms_check.py. Completely depends on matplotlib and numpy.
    
    then can just define the histo_plot_defs.txt file of their liking
    same with map!
    
    Author: Mike Bauer  <mbauer@giss.nasa.gov>
    
    Log:
        2010/12  MB - File created.
    '''
    
    def __init__(self,**kwargs):
        """This is where the default values are set with the option to
           alter then upon instantiation."""
        # Name: defs_file
        # Purpose: Full path to file that contains the details of how
        #   to plot the data.
        # Default: empty - raises error
        if kwargs.has_key('defs_file'):
            self.defs_file = kwargs['defs_file']
        else:
            msg = "Error DefineHistogramStuff: Requires defs_file"
            sys.exit(msg)
        # Name: source_title
        # Purpose: Data source name for plot titles
        # Default: empty
        if kwargs.has_key('source_title'):
            self.source_title = kwargs['source_title']
        else:
            self.source_title = ""
        # Name: source_period
        # Purpose: Time Period for plot titles
        # Default: empty
        if kwargs.has_key('source_period'):
            self.source_period = kwargs['source_period']
        else:
            self.source_period = ""
            
        self.label = ""
        if self.source_title:
            if self.source_period:
            	msg = 'Source: %s Period: %s'
                self.label = msg % (self.source_title.upper(),self.source_period)
            else:
             	msg = 'Source: %s'
                self.label = msg % (self.source_title.upper())
    
    def read_histo_plot_defs(self,):
        self.histo_stuff = {}
        tmp = {}
        msg = ("make_plot = %d; normalized = %d; title = '%s'; xlab = '%s';",
            "ylab = '%s'; bin_width = %f; bins_left_edge = %s; sum_type = %d;",
            "xlabt = '%s'; ylabt = '%s'; ytupper = %f; ytlower = %f;",
            "xlab2d = '%s'; ylab2d = '%s'; yuppern = %f; xupper = %f;",
            "xlower = %f; trim_max = %f; trim_min = %f; may_trim = %d")
        msg = " ".join(msg)
        # File I/O
        try:
            r_file = open(self.defs_file,"r")
        except IOError:
            print "Error Opening: %s" % self.defs_file
        # Read
        try:
            rline = r_file.readlines()
        except IOError:
            print "Error reading:" % self.defs_file
        # Parse
        for line in rline:
            if line:
                rparts = line.split("|")
                if len(rparts) == 1:
                    # New Variable Def
                    new_var = rparts[0].rstrip()
                    if new_var:
                        tmp[new_var] = {}
                        #self.histo_stuff[new_var] = {}
                else:
                    do_var = rparts[0].rstrip()
                    do_val = rparts[1].rstrip()
                    if do_var == "title" and not do_val:
                    	do_val = self.label
                    tmp[new_var][do_var] = do_val
                    #self.histo_stuff[new_var][do_var] = do_val
        # Convert into python string for matplotlib
        for do_var in tmp:
            do = tmp[do_var]
            self.histo_stuff[do_var] = msg % (int(do["make_plot"]),
            int(do["normalized"]),do["title"],do["xlab"],do["ylab"],
            float(do["bin_width"]),do["bins_left_edge"],int(do["sum_type"]),
            do["xlabt"],do["ylabt"],float(do["ytupper"]),float(do["ytlower"]),
            do["xlab2d"],do["ylab2d"],float(do["yuppern"]),float(do["xupper"]),
            float(do["xlower"]),float(do["trim_max"]),float(do["trim_min"]),
            int(do["may_trim"]))
        # Close
        try:
            r_file.close()
        except IOError:
            print "Error Closing: %s" % self.defs_file
    