# Standard library imports

# Third party module imports

# MCMS module imports
from extras import ProgressBarAlt
from print_columns import PrintColumns
from read_mcms import Read_MCMS

def acquire_centers_att(tf,loop_year):
    # Counters for report
    centers = []
    # Read all centers into memory
    readin = {}
    infile = tf.centers_file.replace("XXXX",str(loop_year))
    what_do = {"model" : tf.model,
                "in_file" : infile,
                "out_file" : "",
                "just_center_table" : tf.just_center_table,
                "detail_tracks" : 0,
                "as_tracks" : "",
                "start_time" : "YYYY MM DD HH SEASON",
                "end_time" : "YYYY MM DD HH SEASON",
                "places" : ["GLOBAL"],
                "include_att" : False,
                "include_atts" : False,
                "save_output" : False,
                "overwrite" : False,
                "jd_fake" : tf.jd_fake
                }
    # Pass in model definitions, if sf_file available this is simple.
    if tf.model in ["nra","nra2"]:
        # For the NCAR/NCEP Reanalysis 1 and 2 these values are provided
        # and nothing need be done.
        pass
    else:
        # Provide values
        what_do["tropical_end"] = tf.tropical_end
        what_do["tropical_start"] = tf.tropical_start
        what_do["maxID"] =tf.maxid
        what_do["land_gridids"] = tf.land_gridids

    # Parse definitions.
    readit = Read_MCMS(**what_do)
    readit.check_time()
    readit.check_place()
    readit.fetch_centers()

    # Speed things up A LOT by making a sorted list of centers
    centers = readit.center_holder.keys()
    centers.sort()
    centers.reverse()
    return (centers,readit.ids,infile,readit)
#    return (centers,readit.ids,readit.aids,readit.center_group_fmt,
#        readit.att_group_fmt,readit.atts_group_fmt,readit.empty_group_fmt,
#        readit.problematic_fmt,infile,readit)

def acquire_centers(tf,verbose=0):
    # Counters for report
    centers = []
    # Read all centers into memory
    years = [x for x in range(int(tf.super_years[0]),int(tf.super_years[-1])+1)]
    readin = {}
    what_do = {"model" : tf.model,
                "in_file" : "",
                "out_file" : "",
                "just_center_table" : True,
                "detail_tracks" : 0,
                "as_tracks" : "",
                "start_time" : "YYYY MM DD HH SEASON",
                "end_time" : "YYYY MM DD HH SEASON",
                "places" : ["GLOBAL"],
                "include_att" : False,
                "include_atts" : False,
                "save_output" : False,
                "overwrite" : False,
                "jd_fake" : tf.jd_fake
                }
    # Pass in model definitions, if sf_file available this is simple.
    if tf.model in ["nra","nra2"]:
        # For the NCAR/NCEP Reanalysis 1 and 2 these values are provided
        # and nothing need be done.
        pass
    else:
        # Provide values
        what_do["tropical_end"] = tf.tropical_end
        what_do["tropical_start"] = tf.tropical_start
        what_do["maxID"] =tf.maxid
        what_do["land_gridids"] = tf.land_gridids
    pbara = ProgressBarAlt(years,"\tReading Centers.... ",60)
    #pbara = years
    for loop_year in pbara:
        # Define some files
        header = "mcms_%s_%04d_" % (tf.model,loop_year)
        centers_file = "%s%scenters.txt" % (tf.out_path,header)
        what_do["in_file"] = centers_file
        # Parse definitions.
        readit = Read_MCMS(**what_do)
        ids = readit.ids
        readit.check_time()
        readit.check_place()
        # Read center file
        readit.fetch_centers()
        centers.extend([readit.center_holder[uci] for uci in readit.center_holder])
        readin[loop_year] = len(readit.center_holder)
    print "\tCenters Read (%d): " % sum(readin.values())
    PrintColumns([" %d (%d)" % (loop_year,readin[loop_year]) for loop_year in years],
            indent_tag="\t\t",fmt="%s",cols=4,width=30)

    # Ensure sorted and put in inverse order so t=0 at end of array
    centers.sort()
    centers.reverse()

    #print "#tmp start This is a temporary bit of code to do: What? Comment out to disable."
    ## Trim to 1 month of data for fast debugging
    ##   See similar line in find_dissimilarity_scores.py
    #centers = [x for x in centers if x[1] == 1]
    # extra trim to a single day
    #centers = [x for x in centers if x[2] == 1]
    #tf.adates = sorted(dict((x,1) for x in [x[4] for x in centers]).keys())
    #tf.tsteps = len(tf.adates)
    #print "#tmp end"

    # Extract all unique Julian dates
    if not hasattr(tf, 'adates'):
        tf.adates = sorted(dict((x,1) for x in [x[4] for x in centers]).keys())
        tf.tsteps = len(tf.adates)
    print "\t\tNumber of unique dates/times:",tf.tsteps

    return centers,ids

def acquire_centers_year(tf,loop_year):
    # Counters for report
    centers = []
    # Read centers into memory
    readin = {}
    header = "mcms_%s_%04d" % (tf.model,int(loop_year))
    in_file = "%s%s%s" % (tf.out_path,header,tf.tail)
    what_do = {"model" : tf.model,
                "in_file" : in_file,
                "out_file" : "",
                "just_center_table" : tf.just_center_table,
                "detail_tracks" : tf.detail_tracks,
                "as_tracks" : "",
                "start_time" : "YYYY MM DD HH SEASON",
                "end_time" : "YYYY MM DD HH SEASON",
                "places" : ["GLOBAL"],
                "include_att" : tf.include_att,
                "include_atts" : tf.include_atts,
                "save_output" : False,
                "overwrite" : False,
                "jd_fake" : tf.jd_fake
                }
    # Pass in model definitions, if sf_file available this is simple.
    if tf.model in ["nra","nra2"]:
        # For the NCAR/NCEP Reanalysis 1 and 2 these values are provided
        # and nothing need be done.
        pass
    else:
        # Provide values
        what_do["tropical_end"] = tf.tropical_end
        what_do["tropical_start"] = tf.tropical_start
        what_do["maxID"] =tf.maxid
        what_do["land_gridids"] = tf.land_gridids

    # Parse definitions.
    readit = Read_MCMS(**what_do)
    ids = readit.ids
    aids = readit.aids

    readit.check_time()
    readit.check_place()

    entangled = {}
    if readit.include_atts:
        # Need to read in_file to extract stormy gridids
        readit.fetch_atts()
        # Create a list of centers that are entangled with others but
        #   are not primary centers.
        for key in readit.atts_uci:
            tmp = dict.fromkeys([x for x in
                                 readit.atts_uci[key][aids[
                                     'Entangled_UCIs']].split()
                                 if x != key],1)
            entangled.update(tmp)
    else:
        readit.atts_uci = {}

    centers_previous_done = {}
    if tf.detail_tracks:
        # Store tracks in new database
        readit.detail_tracks = what_do["in_file"].replace(".txt","_tracks_dbase.txt")
        # Read center file and return sorted tracks dbase.
        readit.fetch_centers()
        # Re-set tracks so next read returns centers
        readit.detail_tracks = ""
        readit.as_tracks = ""
        merge_uci = {}
        # Tracks can extend into the next year, which if not accounted for
        # give the impression of shorter track lengths etc. at the end of
        # each year if tracks read in on a year-to-year basis as they are
        # here (centers not so affected). Solution is to read a year ahead
        # and extend any tracks starting in this year with centers from next year.
        if loop_year != tf.years[-1]:
            # Only need to read centers (not tracks)
            what_do2 = what_do.copy()
            tag = str(what_do["in_file"][-tf.cut_tail-4:-tf.cut_tail])
            what_do2["in_file"] = what_do["in_file"].replace(tag,"%04d" % loop_year+1)
            what_do2["detail_tracks"] = ""
            what_do2["as_tracks"] = ""
            what_do2['include_atts'] = 0
            what_do2["just_center_table"] = True
            readit2 = Read_MCMS(**what_do2)
            readit2.check_time()
            readit2.check_place()
            readit2.fetch_centers()
            # readit.sorted_tracks has the usi for all tracks starting loop_year
            # so screen and add any centers with these usi to the correct
            # center in readit.sorted_tracks
            addon_uci_usi = [(readit2.center_holder[x][ids['UCI']],
                readit2.center_holder[x][ids['USI']]) for x in readit2.center_holder
                if readit2.center_holder[x][ids['USI']] in readit.sorted_tracks.keys()]
            # Okay add these uci to the usi that they point too in readit.sorted_tracks
            # don't add centers to centers as will be double counted....
            merge_uci = {}
            for uci_usi in addon_uci_usi:
                uci = uci_usi[0]
                usi = uci_usi[1]
                merge_uci[uci] = readit2.center_holder[uci]
                old = readit.sorted_tracks[usi]
                old.append(uci)
                # Ensure sorted by date
                old.sort()
                readit.sorted_tracks[usi] = old
            if merge_uci and readit.include_atts:
                # Need to update entangled
                readit2.fetch_atts()
                entangled2 = {}
                for key in readit2.atts_uci:
                    # Get entangled uci
                    tmp = [x for x in readit2.atts_uci[key][aids[
                                             'Entangled_UCIs']].split()
                                         if x != key]
                    # Retain only uci found in merge_uci
                    tmp = dict.fromkeys([x for x in tmp
                                         if x in merge_uci],1)
                    entangled2.update(tmp)
                entangled.update(entangled2)
            del readit2
            # If not the 1st year ignore tracks that didn't start loop_year
            # or loop_year+1 as those are covered by another read forward
            # for the previous year.
            s = copy.deepcopy(readit.sorted_tracks)
            for usi in s:
                y = int(usi[:4])
                if y < loop_year:
                    del readit.sorted_tracks[usi]

    # Read center files
    readit.fetch_centers()
    if tf.detail_tracks and merge_uci:
        # Merge extra centers from loop_year + 1
        readit.center_holder.update(merge_uci)
        added_dates = [merge_uci[x][ids['JD']] for x in merge_uci]
        b = {}
        for i in added_dates: b[i] = 0
        added_dates = b.keys()
        added_dates.sort() # sort as dictionary unsorted
        added_steps = len(added_dates)

        if loop_year != tf.years[0]:
            # Drop tracks that start prior to loop_year as those were
            #    covered by the read forward the previous year.
            s = copy.deepcopy(readit.sorted_tracks)
            for usi in s:
                y = int(usi[:4])
                if y < loop_year:
                    del readit.sorted_tracks[usi]
            # Drop centers for tracks that start prior to loop_year as those were
            #    covered by the read forward the previous year.
            s = copy.deepcopy(readit.center_holder)
            for uci in s:
                usi = readit.center_holder[uci][ids['USI']]
                y = int(usi[:4])
                if y < loop_year:
                    del readit.center_holder[uci]
                    if readit.include_atts:
                        if uci in readit.atts_uci:
                            del readit.atts_uci[uci]
                        if uci in entangled:
                            del entangled[uci]
                    if uci[:10] in centers_previous_done:
                        old = centers_previous_done[uci[:10]]
                        old.append(uci)
                        centers_previous_done[uci[:10]] = old
                    else:
                        centers_previous_done[uci[:10]] = [uci]
            del s

    # Speed things up A LOT by making a sorted list of centers
    centers = readit.center_holder.keys()
    centers.sort()
    centers.reverse()

    #print "\tCenters Read",len(centers)
    #if tf.detail_tracks:
    #    print "\t\tTracks Read",len(readit.sorted_tracks)

    #print "#tmp start This is a temporary bit of code to do: What? Comment out to disable."
    ## Trim to 1 month of data for fast debugging
    ##   See similar line in find_dissimilarity_scores.py
    #centers = [x for x in centers if x[1] == 1]
    #print "#tmp end"
    #
    #print "#tmp start This is a temporary bit of code to do: What? Comment out to disable."
    # Trim to 1 day for each month of data for fast debugging
    #   See similar line in find_dissimilarity_scores.py
    #centers = [x for x in centers if x[6:10] == '0100']
    #print "#tmp end"

    # Extract all unique uci_starters
    alldates = []
    #alldates = [readit.center_holder[x][ids['UCI']][:10] for x in readit.center_holder]
    alldates = [x[:10] for x in centers]
    # Get unique dates via dictionary
    b = {}
    for i in alldates: b[i] = 0
    alldates = b.keys()
    alldates.sort() # sort as dictionary unsorted

    tf.adates = alldates
    tf.tsteps = len(alldates)
    #if tf.detail_tracks and merge_uci:
    #    print "\tNumber of unique dates/times:",tf.tsteps," added by track extension",added_steps
    #else:
    #    print "\tNumber of unique dates/times:",tf.tsteps
    return readit,centers,entangled,centers_previous_done
