import os
import sys
import mcms_utilities

#---Start of main code block.
if __name__ == '__main__':
    
    CreateFileHeader = mcms_utilities.CreateFileHeader
    
    stat_dir = "/Volumes/Scratch/output/nra2/stats/"
    super_run = "1979_2010"
    fmt = "%scheck_histo_stats_%s.txt"
    base_name = fmt % (stat_dir,super_run)

    # All runs slp_min (i.e., no peak_iten or all)
    slp_min_only = 0
    
    # All runs peak_iten (i.e., no slp_min or all)
    peak_iten_only = 0
    
    # All runs without slp_min or peak_iten
    all_only = 1
    
    if slp_min_only+peak_iten_only+all_only > 1:
        msg = "Warning Can't have multiple 1 values for: slp_min_only %d peak_iten_only %d all_only %d"
        sys.exit(msg % (slp_min_only,peak_iten_only,all_only))
        

    # variables one is likely to want as "single_variables"
    stat_group = ["slp","dissimilarity","laplacian","intensity","att-area",
        "area-total","att-depth","sep","sep-alt","empty","atts-area",
        "att-perimeter","atts-perimeter","entangled","natt","atts-depth",
        "depth-total","perimeter-total","att-minedge","att-maxedge",
        "att-aveedge","atts-minedge","atts-maxedge","atts-aveedge",
        "atts-compact","atts-aspect","att-compact","att-aspect","track-length",
        "track-range","track-travel","bearing","speed","fill"]
    single_variable = ""
    
    #single_variable = "att-area"
    #single_variable = "atts-area"
    #single_variable = "area-total"
    #single_variable = "att-aveedge"
    
    #single_variable = "att-depth"
    #single_variable = "atts-depth"
    single_variable = "depth-total"

    runs = []
    fmt = "Run (%d of %d):\n\tslp_min %d\n\tpeak_inten %d\n\tmid_lats %d\n\tocean %d\n\tland %d"
    # Outer filters
    for slp_min in ["","_slpmin"]:
        for peak_iten in ["","_piten"]: 
            for mid_lats in ["","_mlats"]:
                for ocean in ["","_ocean"]:
                    for land in ["","_land"]:
                        if slp_min and peak_iten:
                            continue
                        if ocean and land:
                            continue
                        # Uncomment to skip anything not whatever
                        if slp_min_only and not slp_min: continue
                        elif peak_iten_only and not peak_iten: continue 
                        elif all_only and peak_iten: continue
                        elif all_only and slp_min: continue                
                        tail = ""
                        if peak_iten: tail += "_piten"
                        if slp_min: tail += "_slpmin"
                        tail += "_"+single_variable
                        if mid_lats: tail += "_mlats"
                        if ocean: tail += "_ocean"
                        if land: tail += "_land"
                        tail += ".txt"
                        runs.append(base_name.replace(".txt",tail))
    runs = sorted(runs)
    print "\nDoing %d Runs:" % len(runs) 
    if not runs:
        sys.exit("No Files")
    
    col_report = "%scollapse_table_%s_%s.txt" % (stat_dir,super_run,single_variable)
    if all_only:
        col_report = col_report.replace(".txt","_all_only.txt")
    if peak_iten_only:
        col_report = col_report.replace(".txt","_piten_only.txt")
    if slp_min_only:
        col_report = col_report.replace(".txt","_slpmin_only.txt")                
    outfile = open(col_report, 'w')
    # Add a header to stats_report
    header = CreateFileHeader()
    header.fill("Super Summery Histogram Statistics Report","Statistical info.",
            __file__,"","","Self Documenting")
    outfile.write(header.header_string+"\n")
    results_table = {
            'Variable' : '{0:12s}',
            'Hemisphere' : '{1:12s}',
            'Season' : '{2:12s}',
            'Intensity' : '{3:12s}',
            'Other' : '{4:20s} | ',
            'Count' : '{5:15s}',
            'Mean Value' : '{6:15s}',
            'Median Value' : '{7:15s}',
            'Mode Value' : '{8:15s}',
            'ModeAlt Value' : '{9:15s}\n'}            
    report_save_info = ('Variable','Hemisphere','Season','Intensity','Other',
        'Count','Mean Value','Median Value','Mode Value','ModeAlt Value')
    report_save_fmt = "".join(['%s' % (results_table[x]) for x in report_save_info])
    report_save_info_just = [(x,x.rjust(15))[x in ['Count','Mean Value','Median Value','Mode Value','ModeAlt Value']] for x in report_save_info]
    repjust = tuple(report_save_info_just)
    outfile.write(report_save_fmt.format(*repjust))

    hemis = {}
    seasons = {}
    intens = {}
    max_mean = max_median = max_mode = max_modealt = -99999999.0
    min_min  = min_median = min_mode = min_modealt = max_mean*-1.0
    for sfile in runs:
        if not os.path.exists(sfile):
            print "\tFile %s doesn't exit" % sfile
            continue
        other = ""
        if sfile.find("piten") != -1:
            other += "Pinte"
        if sfile.find("slpmin") != -1:
            if other:
                other += "_SLPmin"
            else:
                other += "SLPmin"
        if sfile.find("mlats") != -1:
            if other:
                other += "_Mlats"
            else:
                other += "Mlats"
        if sfile.find("ocean") != -1:
            if other:
                other += "_Ocean"
            else:
                other += "Ocean"
        if sfile.find("land") != -1:        
            if other:
                other += "_Land"
            else:
                other += "Land"
        with open(sfile, 'r') as infile:
            rline_hold = []
            for line in infile:
                if line.startswith("#"): continue
                elif line.startswith("Variable:"):
                    report_line = []
                    parts = line.split(":")
                    report_line.append(parts[1].strip())
                elif line.startswith("\tSeason:"):
                    parts = line.split(":")
                    pt1 = parts[1].split("\t")
                    pt2 = parts[2].split("\t")
                    report_line.append(parts[3].strip())
                    report_line.append(pt1[0].strip())
                    report_line.append(pt2[0].strip())
                    hemis[parts[3].strip()] = 1
                    seasons[pt1[0].strip()] = 1
                    intens[pt2[0].strip()] = 1
                    # Add Other
                    report_line.append(other)
                elif line.startswith("\t\tCount"):
                    parts = line.split(":")
                    report_line.append(parts[1].strip())
                elif line.startswith("\t\tCount"):
                    parts = line.split(":")
                    report_line.append(parts[1].strip())
                elif line.startswith("\t\tMean"):
                    parts = line.split(":")
                    p = float(parts[1].strip())
                    if p >= max_mean:
                        max_mean = p
                    if p <= min_min:
                        min_mean = p
                    report_line.append(parts[1].strip())
                elif line.startswith("\t\tMedian"):
                    parts = line.split(":")
                    if p >= max_median:
                        max_median = p
                    if p <= min_median:
                        min_median = p                    
                    report_line.append(parts[1].strip())
                elif line.startswith("\t\tMode "):
                    parts = line.split(":")
                    if p >= max_mode:
                        max_mode = p
                    if p <= min_mode:
                        min_mode = p                      
                    report_line.append(parts[1].strip())                                                                                                     
                elif line.startswith("\t\tModeAlt"):
                    parts = line.split(":")
                    if p >= max_modealt:
                        max_modealt = p
                    if p <= min_modealt:
                        min_modealt = p                      
                    report_line.append(parts[1].strip())                   
                    report_line = [(x,x.rjust(15))[report_line.index(x) > 4] for x in report_line]
                    rline = tuple(report_line)
                    if rline[3] == "MOD":
                        # Hold this until STR is written.
                        rline_hold = rline
                    else:
                        outfile.write(report_save_fmt.format(*rline))
                    if rline[3] == "STR" and rline_hold:
                        outfile.write(report_save_fmt.format(*rline_hold)) 
                        rline_hold = []          
    outfile.close()

    hemis = sorted(hemis.keys())
    seasons = sorted(seasons.keys())
    intens = sorted(intens.keys())
    if len(intens) > 1:
        intens = ['ALL','STR','MOD','WEA']

    # Read what was done into memory
    outfile = open(col_report, 'r')
    lines = outfile.readlines()
    outfile.close()

    outfile = open(col_report, 'a')
    
    # Append Sort by Hemisphere, Season, Intensity
    for xkey in hemis:
        outfile.write("\n\n******Just %s******\n\n" % xkey)
        outfile.write(report_save_fmt.format(*repjust))
        for line in lines:
            # Only Write Lines from same Hemisphere
            if line.find(xkey) != -1:
                 outfile.write(line)

    # Append Sort by Season
    for xkey in seasons:
        outfile.write("\n\n******Just %s******\n\n" % xkey)
        outfile.write(report_save_fmt.format(*repjust))
        for line in lines:
            if line.find(xkey) != -1:
                 outfile.write(line)

    # Append Sort by Intensity
    for xkey in intens:
        outfile.write("\n\n******Just %s******\n\n" % xkey)
        outfile.write(report_save_fmt.format(*repjust))
        for line in lines:
            if line.find(xkey) != -1:
                 outfile.write(line)
 
    # Append Sort by Max Mean
    outfile.write("\n\n******Just %s******\n\n" % "Max Mean")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(max_mean)) != -1:
             outfile.write(line)                                               

    # Append Sort by Min Mean
    outfile.write("\n\n******Just %s******\n\n" % "Min Mean")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(min_mean)) != -1:
             outfile.write(line) 
    
    # Append Sort by Max Median
    outfile.write("\n\n******Just %s******\n\n" % "Max Median")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(max_median)) != -1:
             outfile.write(line)                                               

    # Append Sort by Min Median
    outfile.write("\n\n******Just %s******\n\n" % "Min Median")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(min_median)) != -1:
             outfile.write(line) 

    # Append Sort by Max Mode
    outfile.write("\n\n******Just %s******\n\n" % "Max Mode")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(max_mode)) != -1:
             outfile.write(line)                                               

    # Append Sort by Min Mode
    outfile.write("\n\n******Just %s******\n\n" % "Min Mode")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(min_mode)) != -1:
             outfile.write(line) 

    # Append Sort by Max ModeAlt
    outfile.write("\n\n******Just %s******\n\n" % "Max ModeAlt")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(max_modealt)) != -1:
             outfile.write(line)                                               

    # Append Sort by Min ModeAlt
    outfile.write("\n\n******Just %s******\n\n" % "Min ModeAlt")
    outfile.write(report_save_fmt.format(*repjust))
    for line in lines:
        if line.find(str(min_modealt)) != -1:
             outfile.write(line)     
    outfile.close()
    
    #print "Done"
    
