#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports

# Third party module imports

# ----------------------------------------------------------------------------
class CreateFileHeader():
    """
    Creates a file header.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 20
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.


    Notes/Warnings:
    ^^^^^^^^^^^^^^


    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/10 MB - File created.
    """

    # Format statements
    file_header = {}
    file_header['long_line'] = "-"*80
    # Title: What is this file called (could be the file name)
    file_header['title'] = "%s"
    # File Description: Describe what is in this file.
    file_header['file_description'] = "%s: %s"
    # Origin: What code created this file?
    file_header['origin'] = "%s: %s"
    # Source: What dataset does this file come from (can be 'self')?
    file_header['source'] = "%s: %s"
    # Created: When was this dataset created?
    file_header['created'] = "%s: %s"

    header_table = {'Title' : '#{0:s}\n#\n',
            'File Description' : '{1:s}\n',
            'Origin' : '{2:s}\n',
            'Source' : '{3:s}\n',
            'Created' : '{4:s}\n',
            'Format' : '{5:s}'
            }
    header_info = ['Title','File Description','Origin',
            'Source','Created','Format']

    def __init__(self,):
        header_fmt =  "".join(['%s' % (CreateFileHeader.header_table[x])
            for x in CreateFileHeader.header_info]) + "\n"
        for hf in CreateFileHeader.header_info:
            if hf == 'Title': continue
            header_fmt = header_fmt.replace(CreateFileHeader.header_table[hf],
                    "# %s: %s" % (hf,CreateFileHeader.header_table[hf]))
        header_fmt += "{6:80s}\n"
        self.header_fmt = header_fmt

    def ctime(self,):
        now = datetime.datetime.now()
        self.created = now.strftime("%Y-%m-%d %H:%M")

    def fill(self,title,description,origin,source,created,fmt):

        if title:
            self.title = title
        else:
            self.title = ""

        if description:
            self.description = WrapHeader(description,
                    len(CreateFileHeader.header_info[1]))
        else:
            self.description = ""

        if origin:
            self.origin = WrapHeader(origin,
                    len(CreateFileHeader.header_info[2]))
        else:
            self.origin = ""

        if source:
            self.source = WrapHeader(source,
                    len(CreateFileHeader.header_info[3]))
        else:
            self.source = ""

        if created:
            self.created = created
        else:
            self.ctime()

        if fmt:
            self.fmt = WrapHeader(fmt,
                    len(CreateFileHeader.header_info[5]))
        else:
            self.fmt = fmt

        self.header_string = self.header_fmt.format(self.title.center(80),
                self.description,self.origin,self.source,self.created,
                self.fmt,"# "+"-"*78)

