# Standard modules
from itertools import chain

# Third party module imports
    
# MCMS modules
from sort_rules_reducible import sort_rules_reducible

from mcms_center_finder import emsg1,emsg2,emsg2a,emsg3,emsg4,emsg5,emsg6,emsg7,emsg8,tabs
from mcms_center_finder import gfmt0,gfmt,gfmt1,gfmt2,gfmt3,gfmt4,gfmt5,gfmt6

def deal_with_irreducible(o_set,pool,kpool,dpool,kc,tc,bt=0,verbose=0):
    """Sort out centers with irreducible overlaps"""
    rname = "Irreducible"
    dfmt1 = "%spin %d, oslen %d, len(pool) %d"
    dfmt2 = "%skin %d, len(kpool) %d"
    dfmt3 = "%sdin %d, oslen-1 %d, len(dpool) %d"
    dfmt4 = "%spin %d, len(pool) %d"
    dfmt5 = "%sSkipped %d"
    dfmt6 = "%soin %d, oout %d, mlen %d"
    fmtc = "%sRetained Reducible Center: %s\n"
    max_iterations = 50
    diddone = set([])
    dothese = sorted(o_set.keys())
    for center in dothese:
        # Skip dealt with centers
        if center in diddone:
            if verbose: print dfmt5 % (tabs[bt],center)
            continue
        # Acquire the case_set for the center. This is the overlap_set
        #   of the center and all the overlap_sets of the members of
        #   this set.
        case_set = [o_set[c] for c in o_set[center]]
        members = set(chain(*case_set))
        irreducible = sum(len(o_set[center].symmetric_difference(x)) for
            x in case_set)
        if verbose:
            print gfmt2 % (tabs[bt],center,o_set[center],tabs[bt+1],case_set,
                tabs[bt+1],members,tabs[bt+1],irreducible)
        if not irreducible:
            print emsg8 % (tabs[bt],rname,"Not irreducible")
            print gfmt2 % (tabs[bt],center,o_set[center],tabs[bt+1],case_set,
                tabs[bt+1],members,tabs[bt+1],irreducible)
            return 1
        # Irreducible case_set. Apply Selection Rules iteratively.
        #
        # Recursively extend case_set to include the overlap_set of all
        #   possibly overlapping centers
        unchecked = 1
        accounted_for = o_set[center].copy()
        while unchecked:
            # List of members whose overlap_set has not been added
            #   to the case_set.
            unchecked = members.symmetric_difference(accounted_for)
            if unchecked:
                if verbose: print "%sunchecked: %s" % (tabs[bt+1],unchecked)
                # Update case_set
                case_set.extend([o_set[c] for c in unchecked])
                # Update members
                members = set(chain(*case_set))
                # Update accounted_for
                accounted_for.update(unchecked)
                if verbose:
                    #print fmta1 % (case_set,members)
                    print gfmt4 % (tabs[bt+1],"case_set",case_set,tabs[bt+2],
                        "Updated members",members)
        
        # Some members are not equivalent. Try to reduce by iteration.
        #   That is, see if one member can be retained by say having
        #   the lowest central slp. Then discard the overlap_set set
        #   for that retained center. Alter the case_set to remove both
        #   the retained and discarded centers. Retest the updated case_set
        #   to see if still irreducible then repeat the loop, if not then
        #   reduce the centers. Repeat this process until the case_set is
        #   empty or the loop_safe threshold is exceeded (prevents
        #   infinite loops).
        loop_safe = 0
        while case_set and loop_safe <= max_iterations:
            loop_safe += 1
            if verbose:
                print gfmt6 % (tabs[bt],loop_safe,tabs[bt+1],"Current case_set",
                    case_set,tabs[bt+1],"Current members",members)
            # Apply selection rules
            fnc_out = sort_rules_reducible(
                dict(zip(members,[kc[x] for x in members])),tc,verbose)
            if fnc_out == 2:
                # Drop whole case_set
                discards = members
                if verbose:
                    print fmtc % (tabs[bt],fnc_out)
                # Update pools
                pin = len(pool)
                kin = len(kpool)
                din = len(dpool)
                mlen = len(members)
                dlen = len(discards)
                pool.difference_update(members)
                dpool.update(members)
                if pin - mlen != len(pool):
                    print emsg6 % (tabs[bt],rname,"center_pool")
                    print dfmt1 % (pin,mlen,len(pool))
                    return 1
                if din + dlen != len(dpool):
                    print emsg6 % (tabs[bt],rname,"discard_pool")
                    print dfmt3 % (din,dlen,len(dpool))
                    return 1
                if verbose:
                    print gfmt1 % (tabs[bt+1],"center_pool",
                        tabs[bt+2],pin,tabs[bt+2],len(pool))
                    print gfmt1 % (tabs[bt+1],"kept_pool",
                        tabs[bt+2],kin,tabs[bt+2],len(kpool))
                    print gfmt1 % (tabs[bt+1],"discard_pool",
                        tabs[bt+2],din,tabs[bt+2],len(dpool))
                # Trim o_set, there should not be any redundant singletons here
                #   as all the removed centers were reducible.
                oin = len(o_set)
                membersin = set(chain(*[o_set[x] for x in o_set]))
                for s in members: del o_set[s]
                oout = len(o_set)
                membersout = set(chain(*[o_set[x] for x in o_set]))
                # Safety check for singletons
                singletons = set([x for x in o_set if len(o_set[x]) == 1])
                if singletons:
                    print emsg7 %  (tabs[bt],rname,"Singleton",tabs[bt],singletons)
                    return 1
                if oin - mlen != oout:
                    print emsg6 % (tabs[bt],rname,"o_set")
                    print "oin %d, oout %d, mlen %d" % (oin,oout,mlen)
                    return 1
                if len(membersin) - mlen != len(membersout):
                    print emsg6 % (tabs[bt],rname,"o_set")
                    pmat = "len(membersin) %d, len(membersout) %d, mlen %d"
                    print pmat % (len(membersin),len(membersout),mlen)
                    print "\tmembersin",membersin
                    print "\tmembersout",membersout
                    print "\tmembers",members
                    return 1
                if verbose:
                    print fmtd % ("o_set",oin,oout)
                    print fmtd % ("o_set members",len(membersin),len(membersout))
                diddone.update(members)
                case_set = []
                members = set([])
                if verbose:
                    print gfmt5 % (tabs[bt+1],"Updated case_set",case_set)
                    print gfmt5 % (tabs[bt+1],"Updated members",members)                
            elif fnc_out:
                # Update pools, here we can remove the kept center from
                #   pool, add to kpool, then discard all o_set members for
                #   the kept center (except itself). Update the case_set and
                #   evaluate if it is irreducible.
                keeps = set(fnc_out)
                drops = set([x for x in o_set[fnc_out[0]] if x not in diddone])
                discards = drops.copy()
                discards.remove(fnc_out[0])
                if verbose:
                    print gfmt5 % (tabs[bt+1],"Retained Reducible Center",
                        keeps)
                    print gfmt5 % (tabs[bt+1],"Discardable Overlap_Set",discards)
                pin = len(pool)
                kin = len(kpool)
                din = len(dpool)
                oin = len(o_set)
                oslen = len(drops)
                pool.difference_update(drops)
                kpool.update(keeps)
                dpool.update(discards)
                if pin - oslen != len(pool):
                    print emsg6 % (tabs[bt],rname,"center_pool")
                    print dfmt1 % (tabs[bt+1],pin,oslen,len(pool))
                    return 1
                if kin + 1 != len(kpool):
                    print emsg6 % (tabs[bt],rname,"kept_pool")
                    print dfmt2 % (tabs[bt+1],kin,len(kpool))
                    return 1
                if din + (oslen-1) != len(dpool):
                    print emsg6 % (tabs[bt],rname,"discard_pool")
                    print dfmt3 % (tabs[bt+1],din,oslen-1,len(dpool))
                    return 1
                if verbose:
                    print gfmt1 % (tabs[bt+1],"center_pool",
                        tabs[bt+2],pin,tabs[bt+2],len(pool))
                    print gfmt1 % (tabs[bt+1],"kept_pool",
                        tabs[bt+2],kin,tabs[bt+2],len(kpool))
                    print gfmt1 % (tabs[bt+1],"discard_pool",
                        tabs[bt+2],din,tabs[bt+2],len(dpool))
                # Record o_set[fnc_out[0]] as being checked
                diddone.update(set(drops))
                # Update case_set and members (remove o_set[fnc_out[0]])
                new_case_set = []
                for x in case_set:
                    test_x = x.copy()
                    for test_y in drops:
                        test_x.discard(test_y)
                    if len(test_x) >= 1:
                        # Update case_set for re-evaluation
                        new_case_set.append(test_x)
                case_set = new_case_set
                members = set(chain(*case_set))
                # Test each member irreducible
                irreducible = 0
                for test_x in members:
                    irreducible += sum(len(
                        set([test_x]).symmetric_difference(x))
                        for x in case_set)
                if verbose:
                    print gfmt5 % (tabs[bt+1],"Updated case_set",case_set)
                    print gfmt5 % (tabs[bt+1],"Updated members",members)
                    print gfmt5 % (tabs[bt+1],"Updated Irreducible",irreducible)
                
                if not irreducible and members:
                    singletons = [x for x in case_set if len(x) == 1]
                    if singletons:
                        smembers = set(chain(*singletons))
                        for test_x in smembers:
                            # Retain center
                            if verbose:
                                print gfmt5 % (tabs[bt+1],
                                    "Retained Reducible Center",test_x)
                            tx = set([test_x])
                            pin = len(pool)
                            kin = len(kpool)
                            pool.difference_update(tx)
                            kpool.update(tx)
                            if pin - 1 != len(pool):
                                print emsg6 % (tabs[bt],rname,"center_pool")
                                print dfmt1 % (tabs[bt+1],pin,1,len(pool))
                                return 1
                            if kin + 1 != len(kpool):
                                print emsg6 % (tabs[bt],rname,"kept_pool")
                                print dfmt2 % (tabs[bt+1],kin,len(kpool))
                                return 1
                            if verbose:
                                print gfmt1 % (tabs[bt+1],"center_pool",
                                    tabs[bt+2],pin,tabs[bt+2],len(pool))
                                print gfmt1 % (tabs[bt+1],"kept_pool",
                                    tabs[bt+2],kin,tabs[bt+2],len(kpool))
                            # Record o_set[fnc_out[0]] as being checked
                            diddone.update(tx)
                        # Update case_set and members (remove o_set[fnc_out[0]])
                        new_case_set = []
                        for x in case_set:
                            test_x = x.copy()
                            for test_y in smembers:
                                test_x.discard(test_y)
                            if len(test_x) >= 1:
                                # Update case_set for re-evaluation
                                new_case_set.append(test_x)
                        case_set = new_case_set
                        members = set(chain(*case_set))
                        if case_set:
                            print emsg8 % (tabs[bt+1],
                                "Non-Empty CaseSet Post-Singleton Reduction",
                                "")
                            return 1
                    else:
                         print "reducible multiple"
                         print singletons, smembers
                         print gfmt5 % (tabs[bt+1],"Updated case_set",case_set)
                         print gfmt5 % (tabs[bt+1],"Updated members",members)
            else:
                print "Warning Irreducible: No retained from sort_rules_reducible"
                print gfmt6 % (tabs[bt],loop_safe,tabs[bt+1],"Current case_set",
                    case_set,tabs[bt+1],"Current members",members)
                return 1
        if loop_safe >= max_iterations:
            if case_set:
                print "Warning Irreducible: Infinite Loop in case_set reduction"
                print gfmt6 % (tabs[bt],loop_safe,tabs[bt+1],"Current case_set",
                        case_set,tabs[bt+1],"Current members",members) 
                return 1
            else:
                break
    
    # Clean up o_set
    oin = len(o_set)
    mlen = len(diddone)
    membersin = set(chain(*[o_set[x] for x in o_set]))
    for s in diddone: del o_set[s]
    oout = len(o_set)
    membersout = set(chain(*[o_set[x] for x in o_set]))
    # Safety check for singletons
    singletons = set([x for x in o_set if len(o_set[x]) == 1])
    if singletons:
        print emsg7 %  (tabs[bt],rname,"Singleton",tabs[bt],singletons)
        return 1
    if oin - mlen != oout or oout:
        print emsg6 % (tabs[bt],rname,"o_set")
        print "oin %d, oout %d, mlen %d" % (oin,oout,mlen)
        return 1
    if len(membersin) - mlen != len(membersout) or membersout:
        print emsg6 % (tabs[bt],rname,"o_set")
        pmat = "len(membersin) %d, len(membersout) %d, mlen %d"
        print pmat % (len(membersin),len(membersout),mlen)
        print "\tmembersin",membersin
        print "\tmembersout",membersout
        print "\tmembers",members
        return 1
    if verbose:
        print gfmt1 % (tabs[bt],"o_set",tabs[bt+1],oin,tabs[bt+1],oout)
        print gfmt1 % (tabs[bt],"o_set members",tabs[bt+1],len(membersin),
            tabs[bt+1],len(membersout))
    
    return 0

