# Standard modules
from itertools import chain

# Third party module imports
    
# MCMS modules
from sort_rules_reducible import sort_rules_reducible
from mcms_center_finder import emsg1,emsg2,emsg2a,emsg3,emsg4,emsg5,emsg6,emsg7,emsg8,tabs
from mcms_center_finder import gfmt0,gfmt,gfmt1,gfmt2,gfmt3,gfmt4,gfmt5,gfmt6

def deal_with_reducible(o_set,pool,kpool,dpool,kc,tc,bt=0,verbose=0):
    """Sort out centers with reducible overlaps"""
    fmta = "\n\t\t\tTesting overlap_set[%7d]: %s\n\t\t\tcase_set: %s"
    fmta += "\n\t\t\tmembers: %s\n\t\t\tirreducible: %d"
    fmtb = "\t\t\t\tReducible Prune"
    fmtc = "\t\t\tRetained Reducible Center: %s\n"
    fmtc += "\t\t\tDiscarded Reducible Center(s): %s"
    fmtd = "\t\t\t\tUpdated %s:\n\t\t\t\t\tin: %d\n\t\t\t\t\tout: %d"
    rname = "Reducible"
    dfmt1 = "pin %d, members %d, len(pool)"
    dfmt2 = "kin %d, len(kpool) %d"
    dfmt3 = "din %d, dlen %d, len(dpool) %d"
    diddone = set([])
    for center in o_set.keys():
        # Skip dealt with centers
        if center in diddone: continue
        # Acquire the case_set for the center. This is the overlap_set
        #   of the center and all the overlap_sets of the members of
        #   this set.
        case_set = [o_set[c] for c in o_set[center]]
        members = set(chain(*case_set))
        irreducible = sum(len(o_set[center].symmetric_difference(x)) for
            x in case_set)
        if verbose:
            print fmta % (center,o_set[center],case_set,members,irreducible)
        if irreducible:
            if verbose: print
            continue
        # Reducible case_set. Apply Selection Rules.
        if verbose: print fmtb
        fnc_out = sort_rules_reducible(
            dict(zip(members,[kc[x] for x in members])),tc,verbose)
        if fnc_out == 2:
            # Drop whole case_set
            discards = members
            if verbose:
                print fmtc % (fnc_out,discards)
            # Update pools
            pin = len(pool)
            kin = len(kpool)
            din = len(dpool)
            mlen = len(members)
            dlen = len(discards)
            pool.difference_update(members)
            dpool.update(members)
            if pin - mlen != len(pool):
                print emsg6 % (tabs[bt],rname,"center_pool")
                print dfmt1 % (pin,mlen,len(pool))
                return 1
            if din + dlen != len(dpool):
                print emsg6 % (tabs[bt],rname,"discard_pool")
                print dfmt3 % (din,dlen,len(dpool))
                return 1
            if verbose:
                print gfmt1 % (tabs[bt+1],"center_pool",
                    tabs[bt+2],pin,tabs[bt+2],len(pool))
                print gfmt1 % (tabs[bt+1],"kept_pool",
                    tabs[bt+2],kin,tabs[bt+2],len(kpool))
                print gfmt1 % (tabs[bt+1],"discard_pool",
                    tabs[bt+2],din,tabs[bt+2],len(dpool))
            # Trim o_set, there should not be any redundant singletons here
            #   as all the removed centers were reducible.
            oin = len(o_set)
            membersin = set(chain(*[o_set[x] for x in o_set]))
            for s in members: del o_set[s]
            oout = len(o_set)
            membersout = set(chain(*[o_set[x] for x in o_set]))
            # Safety check for singletons
            singletons = set([x for x in o_set if len(o_set[x]) == 1])
            if singletons:
                print emsg7 %  (tabs[bt],rname,"Singleton",tabs[bt],singletons)
                return 1
            if oin - mlen != oout:
                print emsg6 % (tabs[bt],rname,"o_set")
                print "oin %d, oout %d, mlen %d" % (oin,oout,mlen)
                return 1
            if len(membersin) - mlen != len(membersout):
                print emsg6 % (tabs[bt],rname,"o_set")
                pmat = "len(membersin) %d, len(membersout) %d, mlen %d"
                print pmat % (len(membersin),len(membersout),mlen)
                print "\tmembersin",membersin
                print "\tmembersout",membersout
                print "\tmembers",members
                return 1
            if verbose:
                print fmtd % ("o_set",oin,oout)
                print fmtd % ("o_set members",len(membersin),len(membersout))
            diddone.update(members)            
        elif fnc_out:
            # Retain and Discard
            discards = [x for x in members if x != fnc_out[0]]
            if verbose:
                print fmtc % (fnc_out,discards)
            # Update pools
            pin = len(pool)
            kin = len(kpool)
            din = len(dpool)
            mlen = len(members)
            dlen = len(discards)
            # print "pool",sorted(list(pool))
            # print "kpool",sorted(list(kpool))
            # print "dpool",sorted(list(dpool))
            # print "members",sorted(list(members))
            # print "discards",sorted(list(discards))
            pool.difference_update(members)
            kpool.update(set(fnc_out))
            dpool.update(set(discards))
            # print "\n*pool",sorted(list(pool))
            # print "*kpool",sorted(list(kpool))
            # print "*dpool",sorted(list(dpool))
            if pin - mlen != len(pool):
                print emsg6 % (tabs[bt],rname,"center_pool")
                print dfmt1 % (pin,mlen,len(pool))
                return 1
            if kin + 1 != len(kpool):
                print emsg6 % (tabs[bt],rname,"kept_pool")
                print dfmt2 % (kin,len(kpool))
                return 1
            if din + dlen != len(dpool):
                print emsg6 % (tabs[bt],rname,"discard_pool")
                print dfmt3 % (din,dlen,len(dpool))
                return 1
            if verbose:
                print gfmt1 % (tabs[bt+1],"center_pool",
                    tabs[bt+2],pin,tabs[bt+2],len(pool))
                print gfmt1 % (tabs[bt+1],"kept_pool",
                    tabs[bt+2],kin,tabs[bt+2],len(kpool))
                print gfmt1 % (tabs[bt+1],"discard_pool",
                    tabs[bt+2],din,tabs[bt+2],len(dpool))
            
            # Trim o_set, there should not be any redundant singletons here
            #   as all the removed centers were reducible.
            oin = len(o_set)
            membersin = set(chain(*[o_set[x] for x in o_set]))
            for s in members: del o_set[s]
            oout = len(o_set)
            membersout = set(chain(*[o_set[x] for x in o_set]))
            # Safety check for singletons
            singletons = set([x for x in o_set if len(o_set[x]) == 1])
            if singletons:
                print emsg7 %  (tabs[bt],rname,"Singleton",tabs[bt],singletons)
                return 1
            if oin - mlen != oout:
                print emsg6 % (tabs[bt],rname,"o_set")
                print "oin %d, oout %d, mlen %d" % (oin,oout,mlen)
                return 1
            if len(membersin) - mlen != len(membersout):
                print emsg6 % (tabs[bt],rname,"o_set")
                pmat = "len(membersin) %d, len(membersout) %d, mlen %d"
                print pmat % (len(membersin),len(membersout),mlen)
                print "\tmembersin",membersin
                print "\tmembersout",membersout
                print "\tmembers",members
                return 1
            if verbose:
                print fmtd % ("o_set",oin,oout)
                print fmtd % ("o_set members",len(membersin),len(membersout))
            diddone.update(members)
        else:
            print "Warning Reducible: No retained from sort_rules_reducible"
            print gfmt5 % (tabs[bt+1],"Current case_set",case_set)
            print gfmt5 % (tabs[bt+1],"Current members",members)
            return 1
    return 0

