#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os

# Third party module imports

# ----------------------------------------------------------------------------
def FileExists(check_this,calling_routine,verbose=True):
    """
    Is some file available on this system?

    Input:
    ^^^^^
    check_this - String
        The name of the file for the existence/availability test.
    calling_routine - String
        The name of the routine calling FileExists

    Output:
    ^^^^^^
    result - Boolean
        True/False if program is found on the system.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> FileExists("test.dat",__name__)
    True

    >>> FileExists("test.txt",__name__)
    Sorry, the file test.nc cannot be found by __main__

    Notes/Warnings:
    ^^^^^^^^^^^^^^
    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2009/11 MB - File created.
    """
    if os.path.exists(check_this):
        return True
    else:
        msg = "Sorry, the file %s cannot be found by %s"
        if verbose: print(msg % (check_this,calling_routine),file=sys.stderr)
        return False

