from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import math

# Third party module imports
import numpy

# MCMS module imports
from extras import ProgressBarAlt
from shift_gridid import shift_gridid
from get_bin_index import get_bin_index
from gcd import gcd
from rhumb_line_nav import rhumb_line_nav
from try_to_connect import try_to_connect
from clean_dict import clean_dict

def find_dissimilarity_scores(tf,defs,centers,tdict,track_stats_inputs,track_stats):
    # Tracking Step 1: See Note 4

    if tf.save_plots == 1 and tf.save_stats:
        from plot_map import plotmap
        from save_netcdf import Save_NetCDF
        from plot_hist import plot_hist
        from rose_plot import rose_plot
    elif tf.save_plots == 2 and tf.save_stats:
        from save_netcdf import Save_NetCDF

    # Containers
    live_tracks = {}
    dead_tracks = {}
    current_centers = []
    past_centers = []
    dissimilar_score = 0.0

    # Make copy for later use
    center_orig = centers[:]

    c_halo = numpy.array([x  for y in xrange(3) for x in xrange(tf.im)],
            dtype=numpy.int32)

    # If not a hi-res model faster to make the whole range of values upfront
    #   otherwise make as needed. The method below uses more memory but is
    #   about 30% faster.
    if tf.high_res:
        tdict_exp = tdict.copy()
    else:
        tdict_exp = {}
        for gridid in xrange(tf.im*tf.jm):
            row = gridid//tf.im
            col = gridid%tf.im
            tdict_exp[gridid] = [shift_gridid(x,col,0,c_halo,tf.halfway,tf.im,tf.jm)
                    for x in tdict[row]]

    probs = []
    if tf.save_stats:
        readin_centers = track_stats[track_stats_inputs.index("readin_centers")]
        stat_groups = track_stats[track_stats_inputs.index("stat_groups")]
        bucket_freq_sum = track_stats[track_stats_inputs.index("bucket_freq_sum")]
        bucket_freq_cnt = track_stats[track_stats_inputs.index("bucket_freq_cnt")]
        probs = track_stats[track_stats_inputs.index("probs")]
        big_bins = track_stats[track_stats_inputs.index("big_bins")]
        big_bins_no_prob = track_stats[track_stats_inputs.index("big_bins_no_prob")]
        big_bins_prob = track_stats[track_stats_inputs.index("big_bins_prob")]
        CisG_bins = track_stats[track_stats_inputs.index("CisG_bins")]
        CisG_bins_no_prob = track_stats[track_stats_inputs.index("CisG_bins_no_prob")]
        CisG_bins_prob = track_stats[track_stats_inputs.index("CisG_bins_prob")]
        StheC_bins = track_stats[track_stats_inputs.index("StheC_bins")]
        StheC_bins_no_prob = track_stats[track_stats_inputs.index("StheC_bins_no_prob")]
        StheC_bins_prob = track_stats[track_stats_inputs.index("StheC_bins_prob")]
        CisB_bins = track_stats[track_stats_inputs.index("CisB_bins")]
        CisB_bins_no_prob = track_stats[track_stats_inputs.index("CisB_bins_no_prob")]
        CisB_bins_prob = track_stats[track_stats_inputs.index("CisB_bins_prob")]
        Dscore_bins = track_stats[track_stats_inputs.index("Dscore_bins")]
        Dscore_bins_no_prob = track_stats[track_stats_inputs.index("Dscore_bins_no_prob")]
        Dscore_bins_prob = track_stats[track_stats_inputs.index("Dscore_bins_prob")]
        bearing_bins = track_stats[track_stats_inputs.index("bearing_bins")]
        bearing_bins_no_prob = track_stats[track_stats_inputs.index("bearing_bins_no_prob")]
        bearing_bins_prob = track_stats[track_stats_inputs.index("bearing_bins_prob")]
        Ccount_bins = track_stats[track_stats_inputs.index("Ccount_bins")]
        Ccount_bins_no_prob = track_stats[track_stats_inputs.index("Ccount_bins_no_prob")]
        Ccount_bins_prob = track_stats[track_stats_inputs.index("Ccount_bins_prob")]
        Dscore_Used_bins = track_stats[track_stats_inputs.index("Dscore_Used_bins")]
        Dscore_Used_bins_no_prob = track_stats[track_stats_inputs.index("Dscore_Used_bins_no_prob")]
        Dscore_Used_bins_prob = track_stats[track_stats_inputs.index("Dscore_Used_bins_prob")]
        Dscore_Unused_bins = track_stats[track_stats_inputs.index("Dscore_Unused_bins")]
        Dscore_Unused_bins_no_prob = track_stats[track_stats_inputs.index("Dscore_Unused_bins_no_prob")]
        Dscore_Unused_bins_prob = track_stats[track_stats_inputs.index("Dscore_Unused_bins_prob")]

    # Timestep in julian days
    if tf.jd_fake:
        delta_jdate = 1
    else:
        delta_jdate = int(round((tf.timestep/24.0)*1000,4))

    # Pre-Bin etc.
    step = -1
    alldates = tf.adates
    date_stamps = tf.uci_stamps
    adatelast = alldates[0]
    inv_travel_distance = 1.0/tf.travel_distance
    inv_accuracy = tf.inv_accuracy*tf.sin60
    min_sep = tf.min_sep
    skip_high_lat = tf.skip_high_lat
    save_stats = tf.save_stats
    save_plots = tf.save_plots
    min_trk_cnt = tf.min_trk_cnt
    fig_format = tf.fig_format
    sin = math.sin
    d2r = math.radians
    model = tf.model
    clim_tag = tf.clim_tag
    out_path = tf.out_path
    years = tf.years
    lons = tf.lons
    lats = tf.lats
    im = tf.im
    jm = tf.jm
    psrose = tf.psrose
    # #mike
    # target_uci = '19790101060131203687'
    # mouse = ""
    # print "\t Target UCI %s" % target_uci
    #cans = []
    cntrack =0
    cotrack =0
    notrack =0
    pbara = ProgressBarAlt(alldates,"\tFinding Dissimilarity Scores.... ",60)
    #pbara = alldates
    for adate in pbara:
        step += 1
        date_stamp = date_stamps[step]
        #print "Checking adate:",adate,date_stamp

        #mike?
        #if adate == alldates[0]:
        #    print "#tmp start This is a temporary bit of code to do: What? Comment out to disable."
        ## Trim to 1 month of data for fast debugging
        ##   See similar line in acquire_centers.py
        #if int(date_stamp[4:6]) > 1: continue
        #if adate == alldates[0]:
        #    print "#tmp end"

        # Check for gapless alldates
        if adate > alldates[0]:
            if int(adate)-int(adatelast) > delta_jdate:
                err_num = 1
                smsg = "\n\tFail Check %d: A timestep(s) skipped." % (err_num)
                msg = "\t\tadate: %d adatelast: %d" % (adate,adatelast)
                tf.do_this(msg,smsg,date_stamp)
                if tf.exit_on_error: return (1,1)
        adatelast = adate

        if adate == alldates[0]:
            # Add all current_centers as track starts
            current = True
            current_centers = []
            while current:
                try:
                    center = centers.pop()
                except: # Hit end of list
                    break
                if center[4] == adate:
                    current_centers.append(center)
                else:
                    current = False
                    centers.append(center) # Overshoot, put back on
            for x in current_centers:
                live_tracks[x[14]] = [(x[14],"")]
            continue # Get another date
        else:
            # Move last set of current_centers to past_centers
            past_centers = current_centers # Set to past centers
            current = True
            current_centers = []
            while current:
                try:
                    center = centers.pop()
                except: # Hit end of list
                    break
                if center[4] == adate:
                    current_centers.append(center)
                else:
                    current = False
                    centers.append(center) # Overshoot, put back on
        deaddate = date_stamps[step]
        if not current_centers:
                err_num = 2
                smsg = "\n\tFail Check %d: No Centers Found for ." % (err_num)
                msg = "\t\tadate: %d adatelast: %d" % (adate,adatelast)
                tf.do_this(msg,smsg,date_stamp)
                if tf.exit_on_error: return (1,1)

        # #mike
        # print "\nChecking adate,date_stamp,deaddate:",adate,date_stamp,deaddate
        # print "\tCurrent Centers: ",len(current_centers)
        # print "\tPast Centers: ",len(past_centers)
        # cheese = [x for x in current_centers if x[14] == target_uci]
        # if cheese:
        #     print "\n\ttarget_uci in current_centers",cheese[0][14]

        # Tracking Step 3: See Note 6
        past_scores = {}
        for past in past_centers:
            # All current_centers w/in search radius of past.
            # Note that a center in past_centers might not
            # fall into candidates at all
            if tf.high_res:
                if past[7] not in tdict_exp:
                    row = past[7]//tf.im
                    col = past[7]%tf.im
                    tdict_exp[past[7]] = [shift_gridid(x,col,0,c_halo,
                        tf.halfway,tf.im,tf.jm) for x in tdict[row]]
            candidates = [x for x in current_centers
                         if x[7] in tdict_exp[past[7]]]
            # #mike
            # oats = [x for x in candidates if x[14] == target_uci]
            # if oats:
            #     print "\n\t\ttarget_uci a candidate for ",past[14]
            #     print "\t\t\tAll candidates", [x[14] for x in candidates]
            #cans.append(len(candidates))
            if save_stats:
                # Accrue Freq plot
                Ccount = len(candidates)
                bucket_freq_sum[5,past[7]] += Ccount
                bucket_freq_cnt[5,past[7]] += 1
                # Bin for histogram
                bin_index = get_bin_index(Ccount,Ccount_bins[1],float(Ccount_bins[4]))
                Ccount_bins[0][bin_index] += 1
                if probs:
                    # Bin for histogram
                    candidates_probs = [x for x in candidates if x[7] in probs]
                    Ccount_probs = len(candidates_probs)
                    bin_index = get_bin_index(Ccount_probs,Ccount_bins_prob[1],float(Ccount_bins_prob[4]))
                    Ccount_bins_prob[0][bin_index] += 1
                    # Bin for histogram
                    candidates_no_probs = [x for x in candidates if x[7] not in probs]
                    Ccount_no_probs = len(candidates_no_probs)
                    bin_index = get_bin_index(Ccount_no_probs,Ccount_bins_no_prob[1],float(Ccount_bins_no_prob[4]))
                    Ccount_bins_no_prob[0][bin_index] += 1
            # Selection based on minimizing the dissimilar_score
            #  between past and all candidates.
            scores = {}
            if candidates:
                clon = past[6]*0.01
                clat = 90.0 - past[5]*0.01
                for pnt in candidates:
                    if probs:
                        a_prob = 0
                        if pnt[7] in probs: a_prob = 1

                    #just_CisG = 0
                    #if pnt[7] == past[7]:
                    #    # Same grid, set only use CisG to
                    #    #   form  dissimilar_score
                    #    just_CisG = 1
                    #    #dissimilar_score = 0.0
                    #    #scores[pnt[14]] = dissimilar_score
                    #    #continue

                    plon = pnt[6]*0.01
                    plat = 90.0 - pnt[5]*0.01

                    if defs.use_gcd:
                        distx = gcd(clon,clat,plon,plat)
                    else:
                        fnc = rhumb_line_nav(plon,plat,clon,clat)
                        distx = fnc[1]
                        bearing = fnc[0]

                    # Close is Best
                    CisB = distx*inv_travel_distance

                    if CisB > 1.0:
                        # Points too far apart... skip to prevent connections.
                        continue

                    if save_stats:
                        # Acrue Freq plot
                        bucket_freq_sum[2,pnt[7]] += CisB
                        bucket_freq_cnt[2,pnt[7]] += 1
                        # Bin for histogram
                        bin_index = get_bin_index(CisB,CisB_bins[1],float(CisB_bins[4]))
                        CisB_bins[0][bin_index] += 1
                        if probs:
                            if a_prob:
                                bin_index = get_bin_index(CisB,CisB_bins_prob[1],float(CisB_bins_prob[4]))
                                CisB_bins_prob[0][bin_index] += 1
                            else:
                                bin_index = get_bin_index(CisB,CisB_bins_no_prob[1],float(CisB_bins_no_prob[4]))
                                CisB_bins_no_prob[0][bin_index] += 1

                    # Change is Gradual
                    mid_lat = 1.0/abs(sin(d2r((plat+clat)*0.5)))
                    CisG = abs(past[8]-pnt[8])*inv_accuracy**mid_lat
                    if save_stats:
                        # Accrue Freq plot
                        bucket_freq_sum[0,pnt[7]] += CisG
                        bucket_freq_cnt[0,pnt[7]] += 1
                        # Bin for histogram
                        bin_index = get_bin_index(CisG,CisG_bins[1],float(CisG_bins[4]))
                        CisG_bins[0][bin_index] += 1
                        if probs:
                            if a_prob:
                                bin_index = get_bin_index(CisG,CisG_bins_prob[1],float(CisG_bins_prob[4]))
                                CisG_bins_prob[0][bin_index] += 1
                            else:
                                bin_index = get_bin_index(CisG,CisG_bins_no_prob[1],float(CisG_bins_no_prob[4]))
                                CisG_bins_no_prob[0][bin_index] += 1

                    # Because of the limitations of bearing finding at high latitudes
                    # and because a nearly stationary center (i.e., small distx) the
                    # apparent bearing could be large but not relevant.
                    #if abs(clat) <= skip_high_lat:
                    if distx > min_sep and abs(clat) <= skip_high_lat:
                        # Bearing always from rhumb line, gcd can't be used!
                        fnc = rhumb_line_nav(
                            plon,plat,clon,clat,True)
                        bearing = fnc[0]
                        if save_stats:
                            # Acrue Freq plot
                            bucket_freq_sum[4,pnt[7]] += bearing
                            bucket_freq_cnt[4,pnt[7]] += 1
                            # Bin for histogram
                            bin_index = get_bin_index(bearing,bearing_bins[1],float(bearing_bins[4]))
                            bearing_bins[0][bin_index] += 1
                            if probs:
                                if a_prob:
                                    bin_index = get_bin_index(bearing,bearing_bins_prob[1],float(bearing_bins_prob[4]))
                                    bearing_bins_prob[0][bin_index] += 1
                                else:
                                    bin_index = get_bin_index(bearing,bearing_bins_no_prob[1],float(bearing_bins_no_prob[4]))
                                    bearing_bins_no_prob[0][bin_index] += 1

                        # Stay the Coarse
                        # 1 at 0deg or 180deg
                        # 0 at 90deg
                        # if bearing greater than 180 discourage by
                        # increasing from 1 at 360deg and 180deg to 2 at 270deg
                        if bearing <= 180.0:
                            StheC = abs(bearing-90.0)/90.0
                        else:
                            StheC = (-1.0*abs(bearing-270.0)/90.0) + 2.0
                        if save_stats:
                            # Accrue Freq plot
                            bucket_freq_sum[1,pnt[7]] += StheC
                            bucket_freq_cnt[1,pnt[7]] += 1
                            # Bin for histogram
                            bin_index = get_bin_index(StheC,StheC_bins[1],float(StheC_bins[4]))
                            StheC_bins[0][bin_index] += 1
                            if probs:
                                if a_prob:
                                    bin_index = get_bin_index(StheC,StheC_bins_prob[1],float(StheC_bins_prob[4]))
                                    StheC_bins_prob[0][bin_index] += 1
                                else:
                                    bin_index = get_bin_index(StheC,StheC_bins_no_prob[1],float(StheC_bins_no_prob[4]))
                                    StheC_bins_no_prob[0][bin_index] += 1
                    else:
                        bearing = 0.0
                        StheC = 0.0

                    # Weighting by CisB means that large coarse or pressure
                    # changes not as important if grids very close.
                    #if just_CisG:
                    #    dissimilar_score = CisG
                    #elif StheC + CisG < 0.0000001:
                    if StheC + CisG < 0.001:
                        # Special case where (StheC + CisG) == 0 but CisG != 0
                        #   so without this dissimilar_score = 0
                        dissimilar_score = CisB
                    elif CisB < 0.01:
                        # Near stationary, allow CisG to make it non-zero but
                        #   limit the value to 1.0
                        dissimilar_score = CisG
                        if dissimilar_score > 1.0: dissimilar_score = 1.0
                    else:
                        dissimilar_score = CisB*(StheC + CisG)

                    scores[pnt[14]] = dissimilar_score # key by current uci

                    # #mike
                    # rice = (1 if pnt[14] == target_uci else 0)
                    # if rice:
                    #     print "\n\t\t\t\ttarget_uci dissimilar_score,CisB,StheC,CisG",dissimilar_score,CisB,StheC,CisG

                    if save_stats:
                        # Accrue Freq plot
                        bucket_freq_sum[3,pnt[7]] += dissimilar_score
                        bucket_freq_cnt[3,pnt[7]] += 1
                        # Bin for histogram
                        bin_index = get_bin_index(dissimilar_score,Dscore_bins[1],float(Dscore_bins[4]))
                        Dscore_bins[0][bin_index] += 1
                        if probs:
                            if a_prob:
                                bin_index = get_bin_index(dissimilar_score,Dscore_bins_prob[1],float(Dscore_bins_prob[4]))
                                Dscore_bins_prob[0][bin_index] += 1
                            else:
                                bin_index = get_bin_index(dissimilar_score,Dscore_bins_no_prob[1],float(Dscore_bins_no_prob[4]))
                                Dscore_bins_no_prob[0][bin_index] += 1

            # Example of past_scores nested dictionary:
            # keyed on a prior center with dissimilar_scores for each
            # candidate among all current_centers.
            # {19960101001742823250:
            #        {19960101061741323000: 0.085061230289818587}}
            past_scores[past[14]] = scores

            if save_stats and len(scores) > 1:
                # only when multiple choices possible
                for peach in scores:
                    pnt = -1
                    for cc in candidates:
                        if cc[14] == peach:
                            pnt = cc[7]
                            break
                    if pnt < 0:
                        smsg = "\n\tFail Check: multiple choices"
                        msg = "pnt Error: %s" % (pnt)
                        tf.do_this(msg,smsg,date_stamp)
                        if tf.exit_on_error: return (1,1)
                    #if scores[peach] < 0: continue
                    bucket_freq_sum[7,pnt] += scores[peach]
                    bucket_freq_cnt[7,pnt] += 1
                    bin_index = get_bin_index(scores[peach],Dscore_Unused_bins[1],float(Dscore_Unused_bins[4]))
                    Dscore_Unused_bins[0][bin_index] += 1
                    if probs:
                        if a_prob:
                            bin_index = get_bin_index(scores[peach],Dscore_Unused_bins_prob[1],float(Dscore_Unused_bins_prob[4]))
                            Dscore_Unused_bins_prob[0][bin_index] += 1
                        else:
                            bin_index = get_bin_index(scores[peach],Dscore_Unused_bins_no_prob[1],float(Dscore_Unused_bins_no_prob[4]))
                            Dscore_Unused_bins_no_prob[0][bin_index] += 1

        #  Tracking Step 4: See Note 7
        #     Sort past_centers by SLP... assessing lowest SLPs first
        # past_centers = sorted(past_centers, key=lambda x: x[8])
        # #mike
        # milk = [x for x in past_centers if x[14] == target_uci]
        # if milk:
        #     print "\n\ttarget_uci in past centers"

        for past in past_centers:

            # See if past can be connected to a current center
            pick = try_to_connect(past[14],past_scores)

            # #mike
            # wheat = (1 if past[14] == target_uci else 0)
            # if wheat:
            #     print "\n\tTesting connection between %s and target_uci:" % pick[0],
            #     if pick[1] == 1:
            #         print " A connection was made"
            #     else:
            #         print " No connection was made"
            # rwheat = (1 if pick[0] == target_uci else 0)
            # if rwheat:
            #     print "\n\t*Testing connection between %s and target_uci:" % past[14],
            #     if pick[1] == 1:
            #         print " A connection was made"
            #         mouse = past[14]
            #     else:
            #         print " No connection was made"

            if pick[1] == 1:
                # A connection was made
                # Remove past from further consideration
                #print "\nRemoving",past[14],"len(past_scores)",len(past_scores),
                del past_scores[past[14]]
                #print len(past_scores)

                # Cull the current center from further consideration
                #print "\tculling",pick[0],sum(len(v) for v in past_scores.itervalues()),
                past_scores = clean_dict(past_scores,pick[0])
                #print sum(len(v) for v in past_scores.itervalues())

                # Update live_tracks
                new_track = True
                for usi in live_tracks:
                    # Continuation of a existing track
                    #print "\tusi",usi,live_tracks[usi][-1][0]
                    if past[14] == live_tracks[usi][-1][0]:
                        new_track = False
                        #print "\t\tContinue",usi,"len(live_tracks[usi])",len(live_tracks[usi]),
                        live_tracks[usi].append((pick[0],pick[2]))
                        #print len(live_tracks[usi]),"live_tracks[usi]",live_tracks[usi]
                        break
#                        if len(live_tracks[usi]) > might_be_long:
#                            # If this track is already more than 5 days long
#                            #   suspect that something is up
#                            if pick[2] > 1.0:
#                                # Large dissimularity... force track break.
#                                break
#                            else:
#                                live_tracks[usi].append((pick[0],pick[2]))
#                                break
#                        else:
#                            live_tracks[usi].append((pick[0],pick[2]))
#                            break
                if new_track:
                    # Create a new track
                    #print "\t\tNew",past[14],
                    live_tracks[past[14]] = [(past[14],""),(pick[0],pick[2])]
                    #print "live_tracks[past[14]]",live_tracks[past[14]]
                    #cntrack += 1
                #else:
                #    cotrack += 1

                if save_stats and scores:
                    for peach in past_scores:
                        pnt = past[7]
                        ss = past_scores[peach].values()
                        if not ss:
                            break
                        bucket_freq_sum[6,pnt] += ss[0]
                        bucket_freq_cnt[6,pnt] += 1
                        # Bin for histogram
                        bin_index = get_bin_index(ss[0],Dscore_Used_bins[1],float(Dscore_Used_bins[4]))
                        Dscore_Used_bins[0][bin_index] += 1
                        if probs:
                            if a_prob:
                                bin_index = get_bin_index(ss[0],Dscore_Used_bins_prob[1],float(Dscore_Used_bins_prob[4]))
                                Dscore_Used_bins_prob[0][bin_index] += 1
                            else:
                                bin_index = get_bin_index(ss[0],Dscore_Used_bins_no_prob[1],float(Dscore_Used_bins_no_prob[4]))
                                Dscore_Used_bins_no_prob[0][bin_index] += 1

            elif pick[1] == 0:
                # No connection made
                # Update past_scores to prevent duplication
                del past_scores[past[14]]
                notrack += 1
            else:
                err_num = 3
                smsg = "\n\tFail Check %d: No discard or use of a connection" % (err_num)
                msg = "\t\tpast usi and score" % (past[14],past_scores)
                tf.do_this(msg,smsg,date_stamp)
                if tf.exit_on_error: return (1,1)
        #print "\tLive Tracks cnt:",len(live_tracks)

        # #mike
        # corn = (1 if target_uci in live_tracks else 0)
        # if corn:
        #     print "\n\ttarget_uci still a live_track"
        # if mouse:
        #     if (1 if mouse in live_tracks else 0):
        #         print "\n\tMouse %s still a live_track" % mouse

        # Saved_tracks get large and slow to search so separate
        # tracks into 'live' (last entry was current datetime),
        # 'dead' (last entry was too old to possibly be used again).
        test = live_tracks.keys()
        test.sort()
        for usi in test:
            # See if last entry was today
            lasttrack = live_tracks[usi][-1][0]
            if lasttrack[:10] != deaddate:
                # Dead track removed from live_tracks
                dead_tracks[usi] = live_tracks.pop(usi)
        #print "\tDead Tracks cnt:",len(dead_tracks)

        # #mike
        # beef = (1 if target_uci in dead_tracks else 0)
        # if beef:
        #     print "\n\ttarget_uci now a dead_track",dead_tracks[target_uci]
        # if mouse:
        #     if (1 if mouse in dead_tracks else 0):
        #         print "\n\tMouse %s now a dead_track" % mouse

    # Move all remaining live tracks to dead... clearly these did
    # not terminate in the normal way, just as tracks starting on
    # t=0 might not be the full track.
    for usi in live_tracks.keys():
        dead_tracks[usi] = live_tracks.pop(usi)


    #from collections import Counter
    #cnt = Counter(cans)
    #ckeys = cnt.keys()
    #ckeys = sorted(ckeys)
    #csum = sum(cans)
    #for c in ckeys:
    #    print c,cnt[c],(float(cnt[c])/float(csum))*100.0

    #print "newtracks",cntrack
    #print "oldtracks",cotrack
    #print "notracks",notrack
    # #mike
    # cornb = (1 if target_uci in live_tracks else 0)
    # if cornb:
    #     print "\n*target_uci a live_track",live_tracks[target_uci]
    # cbeef = (1 if target_uci in dead_tracks else 0)
    # if cbeef:
    #     print "\n*target_uci a dead_track",dead_tracks[target_uci]

    # Make a dictionary of centers to
    # speed up recalls, note chronological
    # nature of centers is lost.
    centers = center_orig
    hits = {}
    for each in centers:
        hits[each[14]] = each

    #print "\tNumber of original centers",len(hits)
    #print "\tNumber of potential tracks",len(dead_tracks)

    if save_stats and save_plots:
        msg = "\tSaving Stats"
        if save_plots:
            msg += " and Making Plots..."
        else:
            msg += " ..."
        pbara = ProgressBarAlt(range(len(stat_groups)),msg,60)
        #pbara = range(len(stat_groups))
        for loopy in pbara:
            use_bins = big_bins[loopy]
            x = use_bins[1]
            y = use_bins[0]
            width = use_bins[4]
            title = "%s %s" % (model,clim_tag)
            xlab = stat_groups[loopy]
            stat_file = "%sstats/%s_%s_%s.txt" % (out_path,model,
                    stat_groups[loopy],tf.clim_tag)
            if save_plots == 1:
                pname = "%sfigs/pdfs/global/annual/%s_tf_hist_%s_%s%s" % (out_path,model,
                        stat_groups[loopy],clim_tag,fig_format)
                msg = plot_hist(x,y,width,stat_file,pname,title,xlab,save_plots)
                #print "\t\t"+msg
                if probs:
                    use_bins = big_bins_prob[loopy]
                    x = use_bins[1]
                    y = use_bins[0]
                    width = use_bins[4]
                    title = "%s %s" % (model,clim_tag)
                    xlab = stat_groups[loopy]
                    stat_file = "%sstats/%s_%s_%s_prob.txt" % (out_path,model,
                            stat_groups[loopy],clim_tag)
                    pname = "%sfigs/pdfs/global/annual/%s_tf_hist_%s_%s_prob%s" % (out_path,model,
                                stat_groups[loopy],clim_tag,fig_format)
                    msg = plot_hist(x,y,width,stat_file,pname,title,xlab,save_plots)
                    use_bins = big_bins_no_prob[loopy]
                    x = use_bins[1]
                    y = use_bins[0]
                    width = use_bins[4]
                    title = "%s %s" % (model,clim_tag)
                    xlab = stat_groups[loopy]
                    stat_file = "%sstats/%s_%s_%s_no_prob.txt" % (out_path,model,
                            stat_groups[loopy],clim_tag)
                    pname = "%sfigs/pdfs/global/annual/%s_tf_hist_%s_%s_no_prob%s" % (
                                out_path,model,stat_groups[loopy],clim_tag,fig_format)
                    msg = plot_hist(x,y,width,stat_file,pname,title,xlab,save_plots)
            bave = numpy.zeros(im*jm,dtype=numpy.float)
            for i in xrange(im*jm): # loop over each grid
                #if readin_centers[i] > 0:
                #    # Frequency of total centers
                #    bave[i] = numpy.divide(bucket_freq_sum[loopy,i],readin_centers[i])
                if bucket_freq_cnt[loopy,i] >= 1:
                    # Average value of centers applied to
                    bave[i] = numpy.divide(bucket_freq_sum[loopy,i],
                            bucket_freq_cnt[loopy,i])
                else:
                    bave[i] = -1.0
            pname = "%sfigs/maps/global/annual/%s_tf_ave_%s_%s%s" % (
                    out_path,model,stat_groups[loopy],
                    clim_tag,fig_format)
            if save_plots == 1:
                fplot = plotmap(missing=-1.0,color_scheme="jet",lon_0=0.0)
                fplot.create_fig()
                fplot.add_field(lons,lats,bave,ptype='pcolor',)
                fplot.finish(pname)
            pname = pname.replace("/figs/maps/global/annual","/netcdfs")
            pname = pname.replace(fig_format,".nc")
            save_it = Save_NetCDF(bave,lons,lats,pname,-1,
                name=stat_groups[loopy],long_name=stat_groups[loopy],
                units = 'Average value Among Filtered Centers')
                #units='Frequency of Occurrence Among Candidate Centers')
        if save_plots == 1 and psrose and save_stats:
            # PS Rose requires pdf or eps to look right
            if fig_format not in [".pdf",".eps"]:
                fg = ".pdf"
            else:
                fg = fig_format
            pname = "%sfigs/pdfs/global/annual/%s_tf_hist_rose_%s" % (
                    out_path,model,clim_tag)
            use_bins = big_bins[4]
            x = use_bins[1]
            y = use_bins[0]
            rose_plot(y,pname,use_bins[1][0],use_bins[1][-1],big_bins[4][4],fg)
            if probs:
                pname = "%sfigs/pdfs/global/annual/%s_tf_hist_rose_%s_prob" % (
                        out_path,model,clim_tag)
                use_bins = big_bins_prob[4]
                x = use_bins[1]
                y = use_bins[0]
                rose_plot(y,pname,use_bins[1][0],use_bins[1][-1],big_bins[4][4],fg)
                pname = "%sfigs/pdfs/global/annual/%s_tf_hist_rose_%s_no_prob" % (
                        out_path,model,clim_tag)
                use_bins = big_bins_no_prob[4]
                x = use_bins[1]
                y = use_bins[0]
                rose_plot(y,pname,use_bins[1][0],use_bins[1][-1],big_bins[4][4],fg)

    return (dead_tracks,hits)
