#!/usr/bin/env python -tt
from __future__ import division

# Standard library imports
import sys
import os

# Third party module imports
import numpy
import netCDF4 as NetCDF
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
import scipy.stats
import cPickle
import gzip

# MCMS module imports
from plot_map import plotmap
from save_netcdf import Save_NetCDF
from file_exists import FileExists
from parse_run_file import ParseRunFile
from print_columns import PrintColumns
from inflate_gdict import inflate_gdict
from injest_setup import InjestSetup

# Example: python util/hotspots.py rf_files/rf_nra2_cf.py

Summerize = scipy.stats.describe
Percentile = scipy.stats.scoreatpercentile
Median = numpy.median
Mode = scipy.stats.mode

def main(argv):

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------

    # Define the base run_file template
    here_path = os.path.abspath(os.path.dirname(__file__))
    here_path = here_path.replace("/util","")
    run_file_template = "%s/cfg/mcms_center_finder_template.py" % (here_path)
    if not FileExists(run_file_template,__name__): return 1

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------
    cf = ParseRunFile(run_file_template,argv,here_path)
    if cf == 1:
        return cf

    # ==============================================================================

    # # Recreate the passed counts netcdf file from a collection of centers.txt files.
    # import fileinput
    # in_dir = "/Volumes/SSD/output/merrac/"
    # file_list = os.listdir(in_dir)
    # file_list = [x for x in file_list if x.endswith(".txt")]
    # file_list.sort()
    # gridids = []
    # for src_file in file_list:
    #     print "Opening %s" % src_file
    #     for line in fileinput.input(in_dir+src_file):
    #         # YYYY MM DD HH JD CoLat Lon GridID GridSLP RegSLP GridLAP Flags Intensity Dissimilarity UCI USI
    #         #%04d %02d %02d %02d %10d %05d %05d %07d %07d %07d %05d %02d %02d %04d %20s %20s
    #         if len(line.split(" ")) >= 8:
    #             gridids.append(int(line.split(" ")[7]))
    # gridids = sorted(gridids)
    # search_path = "%snetcdfs/" % cf.out_path
    # fmt = "%s%s_cf_%s_%s%s"
    # ofile = fmt % (search_path,cf.model,"passed_fast","2000",".nc")
    # if os.path.exists(ofile):
    #     os.remove(ofile)
    # lons = numpy.array(cf.flons)
    # lats = numpy.array(cf.flats)
    # im = len(lons)
    # jm = len(lats)
    # maxid = im*jm
    # cnts = numpy.zeros(maxid)
    # for gridid in gridids:
    #     cnts[gridid] += 1
    # save_it = Save_NetCDF(cnts,lons,lats,ofile,0,
    #         name="passed",long_name="Centers",
    #         units='Counts')
    # sys.exit()

    # ==============================================================================

    ## Recreate the passed counts netcdf file from a single centers.txt file.
    #from acquire_centers import acquire_centers_year
    #search_path = "%snetcdfs/" % cf.out_path
    #fmt = "%s%s_cf_%s_%s%s"
    #mstag = "%04d-%04d" % (cf.start_year,cf.end_year)
    #ofile = fmt % (search_path,cf.model,"passed",mstag,".nc")
    #if os.path.exists(ofile):
        #os.remove(ofile)
    #lons = numpy.array(cf.flons)
    #lats = numpy.array(cf.flats)
    #im = len(lons)
    #jm = len(lats)
    #maxid = im*jm
    #cnts = numpy.zeros(maxid)
    #for missing_year in xrange(cf.start_year,cf.end_year):
        #readit,centers,ee,cc = acquire_centers_year(cf,missing_year)
        #ids = readit.ids
        #for center in centers:
           #cgrid = readit.center_holder[center][ids['GridID']]
           #cnts[cgrid] += 1
    #cnts.shape = (jm,im)
    #save_it = Save_NetCDF(cnts,lons,lats,ofile,0,
            #name="passed",long_name="Centers",
            #units='Counts')
    #sys.exit()

    # ==============================================================================

    # In case missed one.
    # Make at least the "passed" statistic report from the centers file
    #from acquire_centers import acquire_centers_year
    #search_path = "%snetcdfs/" % cf.out_path
    #fmt = "%s%s_cf_%s_%s%s"
    #missing_year = 2005
    #mstag = "%04d-%04d" % (missing_year,missing_year)
    #ofile = fmt % (search_path,cf.model,"passed_fast",mstag,".nc")
    #if os.path.exists(ofile):
    #    os.remove(ofile)
    #lons = numpy.array(cf.flons)
    #lats = numpy.array(cf.flats)
    #im = len(lons)
    #jm = len(lats)
    #maxid = im*jm
    #cnts = numpy.zeros(maxid)
    #readit,centers,ee,cc = acquire_centers_year(cf,missing_year)
    #ids = readit.ids
    #for center in centers:
    #   cgrid = readit.center_holder[center][ids['GridID']]
    #   cnts[cgrid] += 1
    #cnts.shape = (jm,im)
    #save_it = Save_NetCDF(cnts,lons,lats,ofile,0,
    #        name="passed",long_name="Centers",
    #        units='Counts')
    #sys.exit()

    # ==============================================================================
    # ==============================================================================
    # ==============================================================================

    # Also /util/quick_cnt.py for way to make summery count netcdf from a bunch
    #   of centers.txt files.

    # If using the results of consecutive fast_run tests, and therefore have
    #   a bunch of cf_passed_fast_Year-Year.nc files you can concatenate them below
    concat = 0
    if concat:
        search_path = "%snetcdfs/" % cf.out_path
        file_list = os.listdir(search_path)
        file_list.sort()
        pfiles = [x for x in file_list
            if (x.find("passed_fast") != -1 and x.endswith(".nc"))]
        pfiles = [x for x in pfiles if not x.find("freq") != -1]
        pfiles.sort()
        pyears = [pfile.split("-")[1][:4] for pfile in pfiles]
        pyears = sorted(list(set(pyears)))
        cf.start_year = pyears[0]
        cf.end_year = int(pyears[-1])+1
        nstag = "%s-%s" % (pyears[0],pyears[-1])
        fmt = "%s_cf_%s_%s%s"
        ofile = fmt % (cf.model,"passed",nstag,".nc")
        if os.path.exists(search_path+ofile):
            print "\n\n\tConcatenated File already exists! %s" % ofile
            print "\tDeleting... please re-run"
            os.remove(search_path+ofile)
            sys.exit()
        ofile = search_path+ofile
        if len(pfiles) == 1: return
        first = 1
        for pfile in pfiles:
            cdf = NetCDF.Dataset(search_path+pfile,'r', format='NETCDF3_CLASSIC')
            data_raw = cdf.variables["passed"]
            data_in = data_raw[:] # put into a numpy array
            if first:
                first = 0
                data_out = data_raw[:]
                lons = cdf.variables["lon"][:]
                lats = cdf.variables["lat"][:]
                # Remove mask (set to 0)
                data_out = numpy.ma.MaskedArray.filled(data_out,fill_value=0.0)
            else:
                data_in = data_raw[:]
                data_in = numpy.ma.MaskedArray.filled(data_in,fill_value=0.0)
                data_out = numpy.add(data_out,data_in)
            cdf.close()
            del data_in,data_raw
        # Save
        save_it = Save_NetCDF(data_out,lons,lats,ofile,0,
                name="passed",long_name="Centers",units='Counts')
        cf.fast_run = 0

    # Get the troubled_centers for this model
    fnc_out,inputs = InjestSetup("%ss_dat.pz" % (cf.shared_path),0)
    troubled_centers = fnc_out[inputs.index("troubled_centers")]
    del fnc_out

    use_tracking = 0

    # Read in the 'passed' file from center or track finding
    if use_tracking:
        cf.end_year = cf.end_year - 1
        stag = "%04d_%04d" % (int(cf.start_year),int(cf.end_year))
    else:
        stag = "%04d-%04d" % (int(cf.start_year),int(cf.end_year))
    if use_tracking:
        fmt = "%snetcdfs/%s_tf_%s_%s%s"
        ofile = fmt % (cf.out_path,cf.model,"tracking_passed_annual",stag,".nc")
    else:
        fmt = "%snetcdfs/%s_cf_%s_%s%s"
        ofile = fmt % (cf.out_path,cf.model,"passed",stag,".nc")
        if cf.fast_run:
            stag = "%04d-%04d" % (int(cf.start_year),int(cf.start_year))
            ofile = fmt % (cf.out_path,cf.model,"passed_fast",stag,".nc")
    hfile = ofile.replace("passed","hotspots")
    if not os.path.exists(ofile):
        print "\tCan't locate file: %s" % ofile
        return 1
    #ofile = "/Volumes/SSD/output/nra2_raw_pure/nra2_cf_pure_canidates_1979-2013_Raw.nc"
    cdf = NetCDF.Dataset(ofile,'r',format='NETCDF3_CLASSIC')
    #data_raw = cdf.variables['pure candidates']
    data_raw = cdf.variables['passed']
    data_in = data_raw[:]
    lons = cdf.variables["lon"][:]
    lats = cdf.variables["lat"][:]

    # Hotspots are gridIDs with an inordinate number of centers
    #
    #   Isolated... Large difference in cnt with neighbors
    ishape = data_in.shape

    # Histogram center cnts
    non_zero = data_in.ravel()[numpy.flatnonzero(data_in)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    fmter = "Cnt    : %6d\nMode   : % 6d\nMedian : % 6d\nMean   : % 6d\n95%%    : % 6d\n99%%    : % 6d\nMax    : % 6d"
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])

    # Find GridIDs whose center counts exceed the 99th Percentile
    hot_spots = numpy.nonzero(numpy.greater_equal(data_in.ravel(),per99))
    hot_spots = sorted(list(hot_spots[0]))

    # Isolate hot_spots that are already troubled_centers. These might want
    #   to be outright dropped as the troubled_centers treatment isn't working
    #   for them.
    troubled_hot_spots = [x for x in hot_spots if x in troubled_centers[0]]
    troubled_hot_spots.extend([x for x in hot_spots if x in troubled_centers[1]])
    hot_spots = [x for x in hot_spots if x not in troubled_hot_spots]

    # Find a halo of gridIDs surrounding each hot_spot (to prevent the hotspot
    #   from simply shifting over a grid).
    im = ishape[1]
    jm = ishape[0]
    gdict = inflate_gdict(im,jm)
    halo = []
    troubled_halo = []
    for hp in hot_spots:
        halo.extend(gdict[hp])
    halo = [x for x in halo if x not in hot_spots]
    halo = sorted(list(set(halo)))
    for hp in troubled_hot_spots:
        troubled_halo.extend(gdict[hp])
    troubled_halo = [x for x in troubled_halo if x not in troubled_hot_spots]
    troubled_halo = sorted(list(set(troubled_halo)))

    # Save hotspots to files
    if use_tracking:
        hot_file = "%s%s_hotspots_tf.pz" % (cf.shared_path,cf.model)
    else:
        hot_file = "%s%s_hotspots.pz" % (cf.shared_path,cf.model)
    cPickle.dump((hot_spots,halo,troubled_hot_spots,troubled_halo),gzip.open(hot_file, "wb",-1))
    print "\tMade: %s\n" % (hot_file)

    print "HotSpot GridIDs (%d):" % len(hot_spots)
    #PrintColumns(hot_spots,indent_tag="\t\t",fmt="% d",cols=4,width=15)
    print "HotSpot Halo GridIDs (%d):" % len(halo)
    #PrintColumns(halo,indent_tag="\t\t",fmt="% d",cols=4,width=15)
    print "Troubled HotSpot GridIDs (%d):" % len(troubled_hot_spots)
    print "Troubled Halo GridIDs (%d):" % len(troubled_halo)
    print "\nHotSpots Stats"
    print stitle

    # Save hotspots as a netcdf file
    maxid = ishape[0]*ishape[1]
    hspots = numpy.zeros(maxid)
    hspots[hot_spots] = 1
    hspots[halo] = 2
    hspots[troubled_hot_spots] = 3
    hspots[troubled_halo] = 4
    hspots.shape = ishape
    save_it = Save_NetCDF(hspots,lons,lats,hfile,0,
            name="hotspots",long_name="",units='Counts')

    # Save an example passed file without the hotspots
    example = data_in.copy()
    example.shape = maxid
    example[hot_spots] = 0
    example[troubled_hot_spots] = 0
    example.shape = ishape
    efile = hfile.replace("hotspots","passed_no_hotspots")
    save_it = Save_NetCDF(example,lons,lats,efile,0,
            name="passed",long_name="",units='Counts')

    # Save an example passed file without the hotspots+halo
    example = data_in.copy()
    example.shape = maxid
    example[hot_spots] = 0
    example[halo] = 0
    example[troubled_hot_spots] = 0
    example[troubled_halo] = 0
    example.shape = ishape
    efile = hfile.replace("hotspots","passed_no_hotspots_halo")
    save_it = Save_NetCDF(example,lons,lats,efile,0,
            name="passed",long_name="",units='Counts')

    # Plot
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"hotspots",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')

    minus_hotspots = data_in.copy()
    minus_hotspots.shape = im*jm
    minus_hotspots[hot_spots] = 0.0
    minus_hotspots.shape = ishape
    non_zero = minus_hotspots.ravel()[numpy.flatnonzero(minus_hotspots)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    #ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID (excluding hotspots)")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"no_hotspots",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    print "\nHotSpots Stats (minus hotspots)"
    print stitle

    minus_hotspots = data_in.copy()
    minus_hotspots.shape = im*jm
    minus_hotspots[hot_spots] = 0.0
    minus_hotspots[halo] = 0.0
    minus_hotspots.shape = ishape
    non_zero = minus_hotspots.ravel()[numpy.flatnonzero(minus_hotspots)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    #ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID (excluding hotspots+halo)")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"no_hotspots_halo",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    print "\nHotSpots Stats (minus hotspots+halo)"
    print stitle

    minus_hotspots = data_in.copy()
    minus_hotspots.shape = im*jm
    minus_hotspots[troubled_hot_spots] = 0.0
    minus_hotspots.shape = ishape
    non_zero = minus_hotspots.ravel()[numpy.flatnonzero(minus_hotspots)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    #ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID (excluding troubled hotspots)")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"no_troubled_hotspots",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    print "\nTroubled HotSpots Stats (minus troubled hotspots)"
    print stitle

    minus_hotspots = data_in.copy()
    minus_hotspots.shape = im*jm
    minus_hotspots[troubled_hot_spots] = 0.0
    minus_hotspots[troubled_halo] = 0.0
    minus_hotspots.shape = ishape
    non_zero = minus_hotspots.ravel()[numpy.flatnonzero(minus_hotspots)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    #ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID (excluding troubled hotspots+halo)")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"no_troubled_hotspots_halo",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    print "\nTroubled HotSpots Stats (minus troubled hotspots+halo)"
    print stitle

    minus_hotspots = data_in.copy()
    minus_hotspots.shape = im*jm
    minus_hotspots[hot_spots] = 0.0
    minus_hotspots[halo] = 0.0
    minus_hotspots[troubled_hot_spots] = 0.0
    minus_hotspots[troubled_halo] = 0.0
    minus_hotspots.shape = ishape
    non_zero = minus_hotspots.ravel()[numpy.flatnonzero(minus_hotspots)]
    fnc_out = Summerize(non_zero)
    per95 = Percentile(non_zero,95.0)
    per99 = Percentile(non_zero,99.0)
    med = Median(non_zero)
    mode = Mode(non_zero)[0][0]
    stitle = fmter % (fnc_out[0],mode,med,int(fnc_out[2]),per95,per99,fnc_out[1][1])
    fig = plt.figure()
    ax = fig.add_subplot(111)
    # the histogram of the data
    n, bins, patches = ax.hist(non_zero,50,normed=0,facecolor='0.7')
    #ax.set_xlim(0,fnc_out[1][1]/3)
    #ax.set_ylim(0, 0.03)
    ax.grid(True)
    ax.set_xlabel('Center Count')
    ax.set_ylabel('Cnt')
    plt.title("Number of Center Counts per GridID (excluding hot)")
    plt.suptitle(stitle, fontsize=9,x=0.75,y=0.88,horizontalalignment='left')
    fmt1 = "%sfigs/pdfs/global/annual/%s_cf_%s_%s%s"
    pname = fmt1 % (cf.out_path,cf.model,"no_hot",stag,cf.fig_format)
    if use_tracking:
        pname = pname.replace("_cf_","_tf_")
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    print "\nAll Hot Stats (minus hot)"
    print stitle

    return

if __name__ == "__main__":
    sys.exit( main(sys.argv) )
