#!/usr/bin/env python -tt
"""This module recreates gdict info from minimum that is stored.

Options/Arguments:

Returns/Creates:

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

def inflate_gdict(im,jm):
    import util.make_gdict
    import numpy as np
    make_gdict = util.make_gdict.make_gdict_np
    halfway = int(im/2)
    c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)
    gdict = {}
    for true_row in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
        for true_col in xrange(im):
            gridid = (im*true_row) + true_col
            gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
    return gdict

def inflate_gdict_limited(targets,c_halo,im,jm):
    import util.make_gdict
    make_gdict = util.make_gdict.make_gdict_np
    halfway = int(im/2)
    gdict = {}
    for gridid in targets:
        true_col = gridid%im
        true_row = gridid/im
        gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
    return gdict

def inflate_gdict_outer(im,jm):
    import util.make_gdict_outer
    import numpy as np
    make_gdict = util.make_gdict_outer.make_gdict_outer_np
    halfway = int(im/2)
    c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)
    gdict = {}
    for true_row in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
        for true_col in xrange(im):
            gridid = (im*true_row) + true_col
            gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
    return gdict

def inflate_gdict_outer_limited(targets,c_halo,im,jm):
    import util.make_gdict_outer
    make_gdict = util.make_gdict_outer.make_gdict_outer_np
    halfway = int(im/2)
    gdict = {}
    for gridid in targets:
        true_col = gridid%im
        true_row = gridid/im
        gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
    return gdict

#def test_np():
#    import numpy as np
#    make_gdict = util.make_gdict.make_gdict_np
#    im = 540
#    jm = 361
#    halfway = int(im/2)
#    maxid = im*jm
#    c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)
#    gdict = {}
#    for true_row in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
#        for true_col in xrange(im):
#            gridid = (im*true_row) + true_col
#            gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
#    return

#def test_c():
#    import util.make_gdict
#    make_gdict = util.make_gdict.make_gdict_c
#    im = 540
#    jm = 361
#    halfway = int(im/2)
#    maxid = im*jm
#    gdict = {}
#    for true_row in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
#        for true_col in xrange(im):
#            gridid = (im*true_row) + true_col
#            gdict[gridid] = make_gdict(true_row,true_col,halfway,im,jm)
#    return

#def test():
#    def get_true_row(trow,tcol,halfway,im,jm):
#        if 0 <= trow < jm: 
#            # Find as is.
#            return trow,tcol
#        elif trow < 0:
#            # South Pole roll over
#            #   A trow of -1 lands on the same row, but the column sifted im/2
#            return trow+1,tcol+halfway
#        elif trow >= jm: 
#            # North Pole roll over
#            #   A trow of jm lands on jm-1, but the column sifted im/2
#            return trow-1,tcol+halfway
#        else:
#            return 1
#    def make_gdict(trow,tcol,c_halo,halfway,im,jm):
#         tmp = [get_true_row(trow+joff,tcol+ioff,halfway,im,jm) 
#                 for joff in [1,0,-1] for ioff in [-1,0,1]]
#         frows = [x[0] for x in tmp]
#         fcols = [c_halo[x[1]+im] for x in tmp]
#         tmp = [(frows[x],fcols[x]) for x in xrange(9)]
#         gdict = [(im*x[0]+x[1]) for x in tmp]
#         return gdict
#    im = 540
#    jm = 361
#    halfway = int(im/2)
#    maxid = im*jm
#    c_halo = [x  for y in xrange(3) for x in xrange(im)]
#    gdict = {}
#    for true_row in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
#        for true_col in xrange(im):
#            gridid = (im*true_row) + true_col
#            gdict[gridid] = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
#    return

#---Start of main code block.
if __name__=='__main__':
    
    test_inflate_outer = 1
    if test_inflate_outer:
        import util.print_gdict
        PrintGdict = util.print_gdict.PrintGdict
        import util.shift_gridid
        import numpy as np
        shift_gridid =  util.shift_gridid.shift_gridid
        
        im = 8
        jm = 8
        
        # Simple Example of inflating gdict for NRA/NRA2
        #im = 144
        #jm = 73

        # Simple Example of inflating gdict for Merra
        #im = 540
        #jm = 361
        
        halfway = int(im/2)
        c_halo = np.array([x for y in xrange(3) for x in xrange(im)],dtype=np.int32)

        gdict = inflate_gdict_outer(im,jm)
        #gdict = inflate_gdict(im,jm)
        
    
        #print len(gdict)
         
        for tr in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
            for tc in xrange(im):
                if tr not in [0,jm-1]:
                    if tc not in [0,im-1]:
                        continue
                gridid = im*tr+tc
                print "\t\tGridID = % 6d\tRow = % 3d\tColumn = % 3d" % (gridid,tr,tc)
                print PrintGdict(gdict[gridid],tabs=2)
           
                g = [shift_gridid(gridid,dc,dr,c_halo,halfway,im,jm) 
                    for dr in [3,0,-3] for dc in [-3,0,3] ]
                print PrintGdict(g,tabs=2)


                import sys; sys.exit("Stop Here")


        #for tr in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
        #        for tc in xrange(im):
        #            if just_first_row:
        #                if tr != 0: continue
        #            if just_last_row:
        #                if tr != jm-1: continue
        #            if just_last_col:
        #               if tc != im-1: continue     
        #            if just_first_col:
        #                if tc != 0: continue
        #                # just 1st couple of rows if 1 < tr < jm-2: continue    
        #            true_row = tr + shift_row
        #            true_col = tc + shift_col
        #            gdict = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
        #            d = dict((k,1) for k in gdict)
        #            fake_gridids.update(d)
        #            for k in gdict:
        #                gridid_cnts[k] += 1
        #            gridid = (im*true_row) + true_col
        #            print "\t\tGridID = % 6d\tRow = % 3d\tColumn = % 3d" % (gridid,true_row,true_col)
        #            print PrintGdict(gdict,tabs=2)


    test_shift_grid = 0
    if test_shift_grid:
        # Test shift_gridid
        import util.shift_gridid
        import numpy as np
        import util.print_gdict
        PrintGdict = util.print_gdict.PrintGdict
        shift_gridid =  util.shift_gridid.shift_gridid
        
        im = 144
        jm = 73
        
        # (See docs/working_with_grids.pdf)
        im = 8
        jm = 8
        
        halfway = int(im/2)
        drow = 0
        dcol = 0
        c_halo = np.array([x for y in xrange(3) for x in xrange(im)],dtype=np.int32)

        # Make a gdict (See docs/working_with_grids.pdf)
        #   upm     upc       upp     0 1 2  (x,y) -1,+1  0,+1  +1,+1 
        #   cnm     cnt       cnp     3 4 5        -1, 0  0, 0  +1, 0
        #   dnm     dnc       dnp     6 7 8        -1,-1  0,-1  +1,-1
        #gdict_offsets = [(dc,dr) for dr in [1,0,-1] for dc in [-1,0,1]]
        for gridid in xrange(im*jm):
            gdict = [shift_gridid(gridid,dc,dr,c_halo,halfway,im,jm) 
                    for dr in [1,0,-1] for dc in [-1,0,1] ]
            if gridid in [0,halfway,im-1,(jm-1)*im,(jm-1)*im+halfway,jm*im-1]:
                print "\tGridID = % 6d" % (gridid)
                print PrintGdict(gdict,tabs=2)

        # Make a gdict_outer (gdict of gdict) of all 9-pnts around gridid.
        #
        # Note: for the im=jm=8 in (See docs/working_with_grids.pdf) this
        #   looks a little odd because +/- 3 is close to halfway.
        #   
        # Level 1: around gridID cnt
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   ---   ---   ---   ---   ---   ---    List Order
        #    ---   ---   upm   upc   upp   ---   ---        0 1 2
        #    ---   ---   cnm   cnt   cnp   ---   ---        3 4 5
        #    ---   ---   dnm   dnc   dnp   ---   ---        6 7 8
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #
        # Level 2: Around gridID cnt (i.e., upp of upp or gdict[gdict[cnt][1]][1])
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   upm   ---   upc   ---   upp   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   cnm   ---   cnt   ---   cnp   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   dnm   ---   dnc   ---   dnp   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #
        # Level 3: Around gridID cnt
        #    upm   ---   ---   upc   ---   ---   upp   -3,+3  +0,+3   +3,+3
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #    cnm   ---   ---   cnt   ---   ---   cnp   -3,+0  +0,+0   +3,+0
        #    ---   ---   ---   ---   ---   ---   ---
        #    ---   ---   ---   ---   ---   ---   ---
        #    dnm   ---   ---   dnc   ---   ---   dnp   -3,-3  +0,-3   +3,-3
        #gdict_offsets = [(dc,dr) for dr in [3,0,-3] for dc in [-3,0,3]]
        for gridid in xrange(im*jm):
            gdict = [shift_gridid(gridid,dc,dr,c_halo,halfway,im,jm) 
                    for dr in [3,0,-3] for dc in [-3,0,3] ]
            if gridid in [0,halfway,im-1,(jm-1)*im,(jm-1)*im+halfway,jm*im-1]:
                print "\tGridID = % 6d" % (gridid)
                print PrintGdict(gdict,tabs=2)
        
        # Test shift everything 
        #    +/- combos
        for drow in [0,1,-1,4,-4]:
            for dcol in [0,1,-1,4,-4]:
                for gridid in xrange(im*jm):
                    sgridid = shift_gridid(gridid,dcol,drow,c_halo,halfway,im,jm)
                    if gridid < im:
                        if gridid == 0: 
                            print "\nFirst Row: drow % 2d dcol % 2d" % (drow,dcol)
                        print "\t% 6d -> % 6d" % (gridid,sgridid) 
                    elif gridid >= (jm-1)*im:
                        if gridid == (jm-1)*im: 
                            print "\nLast Row: drow % 2d dcol % 2d" % (drow,dcol)
                        print "\t% 6d -> % 6d" % (gridid,sgridid)

        import sys; sys.exit("Stop Here")

    # -------------------------------------------------------------------------        
    # Speed tests (uncomment above, and in make_grid.pyx)
    # -------------------------------------------------------------------------    
    #   Pure python  3.88004183769 sec
    #   cython       11.4786009789 sec
    #   cython+Numpy 0.721824884415 sec
    speed_test = 0
    if speed_test:
        import timeit  
        print "\n\n"
        t = timeit.Timer("test()","from __main__ import test") 
        print "Pure python ", t.timeit(1), "sec"    
        t = timeit.Timer("test_c()","from __main__ import test_c") 
        print "cython      ", t.timeit(1), "sec"    
        t = timeit.Timer("test_np()","from __main__ import test_np") 
        print "cython+Numpy", t.timeit(1), "sec"    
        import sys; sys.exit("Stop Here")
    # -------------------------------------------------------------------------

    # -------------------------------------------------------------------------
    # Example of inflating gdict 
    # -------------------------------------------------------------------------
    make_example = 0
    if make_example:
        import util.print_gdict
        PrintGdict = util.print_gdict.PrintGdict
        
        # Simple Example of inflating gdict for NRA/NRA2
        im = 144
        jm = 73

        # Simple Example of inflating gdict for Merra
        #im = 540
        #jm = 361

        gdict = inflate_gdict(im,jm)
        print len(gdict)
        print PrintGdict(gdict[134],tabs=2)
        import sys; sys.exit("Stop Here")

    # -------------------------------------------------------------------------
    # TESTS
    # -------------------------------------------------------------------------
    run_simple_tests = 0
    run_big_tests = 0
    if run_simple_tests:
        import util.print_gdict
        import util.make_gdict
        import numpy as np
        make_gdict = util.make_gdict.make_gdict_np
        PrintGdict = util.print_gdict.PrintGdict
        
        # Case where even number of longitudes (im)
        #   See docs/working_with_grids.pdf
        im = 8
        jm = 8
        maxid = im*jm    
        halfway = int(im/2)
        c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)

        for test in xrange(5):
            just_first_col = just_last_col = just_first_row = just_last_row = everything = 0
            if test == 0:
                just_first_col = 1
                print "\n%s\n\t\t\tTest just the first column (Even im)\n%s" % ("*"*80,"*"*80)
            elif test == 1:
                just_last_col = 1
                print "\n%s\n\t\t\tTest just the last column (Even im)\n%s" % ("*"*80,"*"*80)            
            elif test == 2:
                just_first_row = 1
                print "\n%s\n\t\t\tTest just the first row (Even im)\n%s" % ("*"*80,"*"*80)
            elif test == 3:
                just_last_row = 1
                print "\n%s\n\t\t\tTest just the last row (Even im)\n%s" % ("*"*80,"*"*80)
            elif test == 4:
                everything = 1
                print "\n%s\n\t\t\tTest Everything (Even im)\n%s" % ("*"*80,"*"*80)
            
            if everything:
                # All Grids
                # Length 
                #   with duplicates  = maxid*9
                #   w/out duplicates = maxid
                dup_cnt = "% 4d" % (maxid*9)
                no_dup_cnt = "% 4d" % (maxid)
                correct_gridids = []

                j = jm - 1
                jvals = [(im*j+(i+halfway),im*j+i+halfway-im)[i+halfway > im - 1] for i in xrange(-1,im+1)]
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s" % (jm,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                for j in xrange(jm-1,-1,-1):
                    jvals = [im*j+im-1]
                    jvals.extend([im*j+i for i in xrange(im)])
                    jvals.extend([im*j])
                    correct_gridids.extend(jvals)
                    #print "j % 3d -> %s" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                j = -1
                jvals = [(i+halfway,i+halfway-im)[i+halfway > im - 1] for i in xrange(-1,im+1)]
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s" % (-1,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                correct_gridids = set(correct_gridids)

            if just_first_col:
                # Just 1st col
                # Length 
                #   with duplicates  = jm*9
                #   w/out duplicates = jm*3 + (3*2)
                # NP Roll over 
                dup_cnt = "% 4d" % (jm*9)
                no_dup_cnt = "% 4d" % (jm*3 + 6)
                correct_gridids = []
                jvals = []
                j = jm
                jvals = [im*(jm-1) + c_halo[(im-1+halfway+im)]]
                jvals.extend([im*(jm-1)+(ioff+halfway) for ioff in [0,1]])
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s *NP Wrap" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                # Main column
                for j in xrange(jm-1,-1,-1):
                    jvals = [im*j+(im-1)]
                    jvals.extend([(im*j)+ioff for ioff in [0,1]])
                    correct_gridids.extend(jvals)
                    #print "j % 3d -> %s" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                # SP Roll over
                j = -1
                jvals = []    
                jvals = [0 + c_halo[(im-1+halfway+im)]]
                jvals.extend([0 +(ioff+halfway) for ioff in [0,1]])
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s *SP Wrap" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                correct_gridids = set(correct_gridids)
            
            if just_last_col:
                # Just last col
                # Length 
                #   with duplicates  = jm*9
                #   w/out duplicates = jm*3 + (3*2)
                dup_cnt = "% 4d" % (jm*9)
                no_dup_cnt = "% 4d" % (jm*3 + 6)        
                # NP Roll over 
                correct_gridids = []
                jvals = []
                j = jm
                jvals = [im*(jm-1) + c_halo[(im-2+halfway+im)]]
                jvals.extend([im*(jm-1)+(ioff-1+halfway) for ioff in [0,1]])
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s *NP Wrap" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                # Main column
                for j in xrange(jm-1,-1,-1):
                    jvals = [im*j+(im-2)]
                    jvals.extend([((im*j)+(im-1+ioff),(im*j)+ioff-1)[im-1+ioff > im - 1] for ioff in [0,1]])            
                    correct_gridids.extend(jvals)
                    #print "j % 3d -> %s" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                # SP Roll over
                j = -1
                jvals = []    
                jvals = [0 + c_halo[(im-2+halfway+im)]]
                jvals.extend([0 +(ioff-1+halfway) for ioff in [0,1]])
                correct_gridids.extend(jvals)
                #print "j % 3d -> %s *SP Wrap" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
                correct_gridids = set(correct_gridids)

            if just_first_row:        
                # Just 1st row
                # Length 
                #   with duplicates  = im*9
                #   w/out duplicates = im*2
                dup_cnt = "% 4d" % (im*9)
                no_dup_cnt = "% 4d" % (im*2)
                correct_gridids = []
                jvals = [(i+halfway,i+halfway-im)[i+halfway > im - 1] for i in xrange(im)]
                jvals.extend(range(im))
                jvals.extend([(im*1)+i for i in xrange(im)])
                correct_gridids.extend(jvals)
                correct_gridids = set(correct_gridids)

            if just_last_row:        
                # Just last row
                # Length 
                #   with duplicates  = im*9
                #   w/out duplicates = im*2
                dup_cnt = "% 4d" % (im*9)
                no_dup_cnt = "% 4d" % (im*2)
                correct_gridids = []
                j = jm - 1
                jvals = [(im*j+(i+halfway),im*j+i+halfway-im)[i+halfway > im - 1] for i in xrange(im)]
                jvals.extend([im*j+i for i in xrange(im)])
                jvals.extend([(im*(j-1))+i for i in xrange(im)])
                correct_gridids = set(jvals)

            shift_row = 0
            shift_col = 0
            gridid_cnts = [0]*maxid
            fake_gridids = {}
            for tr in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
                for tc in xrange(im):
                    if just_first_row:
                        if tr != 0: continue
                    if just_last_row:
                        if tr != jm-1: continue
                    if just_last_col:
                       if tc != im-1: continue     
                    if just_first_col:
                        if tc != 0: continue
                        # just 1st couple of rows if 1 < tr < jm-2: continue    
                    true_row = tr + shift_row
                    true_col = tc + shift_col
                    gdict = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
                    d = dict((k,1) for k in gdict)
                    fake_gridids.update(d)
                    for k in gdict:
                        gridid_cnts[k] += 1
                    gridid = (im*true_row) + true_col
                    print "\t\tGridID = % 6d\tRow = % 3d\tColumn = % 3d" % (gridid,true_row,true_col)
                    print PrintGdict(gdict,tabs=2)
            
            for k in xrange(len(gridid_cnts)):
                if gridid_cnts[k] > 0:
                    print "\t\tGridID % 5d % 5d" % (k,gridid_cnts[k])
            print "\t\tNumber unique grids %d v. Expected %s" % (len(correct_gridids),no_dup_cnt)
            print "\t\tNumber grids used %d v. Expected %s" % (sum(gridid_cnts),dup_cnt)
                    
            fake_gridids = set(fake_gridids.keys())
            gridid_diff = correct_gridids.symmetric_difference(fake_gridids)
            if gridid_diff:
                print "\t\tError correct_gridids.symmetric_difference(fake_gridids)",gridid_diff

    if run_big_tests:
        import util.print_gdict
        import util.make_gdict
        import numpy as np
        make_gdict = util.make_gdict.make_gdict_np
        PrintGdict = util.print_gdict.PrintGdict
        
        im = 144
        jm = 73
        halfway = int(im/2)
        maxid = im*jm
        lats = [90, 87.5, 85, 82.5, 80, 77.5, 75, 72.5, 70, 67.5, 65, 62.5, 60, 57.5,
                 55, 52.5, 50, 47.5, 45, 42.5, 40, 37.5, 35, 32.5, 30, 27.5, 25, 22.5, 20,
                 17.5, 15, 12.5, 10, 7.5, 5, 2.5, 0, -2.5, -5, -7.5, -10, -12.5, -15,
                 -17.5, -20, -22.5, -25, -27.5, -30, -32.5, -35, -37.5, -40, -42.5, -45,
                 -47.5, -50, -52.5, -55, -57.5, -60, -62.5, -65, -67.5, -70, -72.5, -75,
                 -77.5, -80, -82.5, -85, -87.5, -90]
        lats = lats[::-1] 
        lons = [0, 2.5, 5, 7.5, 10, 12.5, 15, 17.5, 20, 22.5, 25, 27.5, 30, 32.5, 35,
                 37.5, 40, 42.5, 45, 47.5, 50, 52.5, 55, 57.5, 60, 62.5, 65, 67.5, 70,
                 72.5, 75, 77.5, 80, 82.5, 85, 87.5, 90, 92.5, 95, 97.5, 100, 102.5, 105,
                 107.5, 110, 112.5, 115, 117.5, 120, 122.5, 125, 127.5, 130, 132.5, 135,
                 137.5, 140, 142.5, 145, 147.5, 150, 152.5, 155, 157.5, 160, 162.5, 165,
                 167.5, 170, 172.5, 175, 177.5, 180, 182.5, 185, 187.5, 190, 192.5, 195,
                 197.5, 200, 202.5, 205, 207.5, 210, 212.5, 215, 217.5, 220, 222.5, 225,
                 227.5, 230, 232.5, 235, 237.5, 240, 242.5, 245, 247.5, 250, 252.5, 255,
                 257.5, 260, 262.5, 265, 267.5, 270, 272.5, 275, 277.5, 280, 282.5, 285,
                 287.5, 290, 292.5, 295, 297.5, 300, 302.5, 305, 307.5, 310, 312.5, 315,
                 317.5, 320, 322.5, 325, 327.5, 330, 332.5, 335, 337.5, 340, 342.5, 345,
                 347.5, 350, 352.5, 355, 357.5] 
        lons = [((x+180.0)%360)-180.0 for x in lons]
        
        # True_GridID points to the interior of the computational grid 0...MaxID-1
        #
        # Halo_GridID points to the interior of the halo grid, which is a 3x3 grid
        #   of the computational grid, with the True_GridIDs being the center of this
        #   halo.

        # Halo Grid
        #
        #  |............maxid-1
        #  |...................|.....................|.......................|
        #  |0              im-1|im              2im-1|2im               3im-1|
     
        # GridID = IM*(True_Row + JM) + (True_Col + IM) 


        #c_halo = [0,.....,im-1,im,.......,2*im-1,2*im,......,3*im-1]
        c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)
        msg = "\n%s\n\tGridID   = % 7d\n\tTrue_Row = % 7d (% 7d)\n\tFake_Lat = % 7.1f\n\t"
        msg += "True_Col = % 7d (% 7d)\n\tFake_Col = % 7d\n\tFake_Lon = % 7.1f"

        # All Grids
        # Length 
        #   with duplicates  = maxid*9
        #   w/out duplicates = maxid
        dup_cnt = "% 4d" % (maxid*9)
        no_dup_cnt = "% 4d" % (maxid)
        j = jm - 1
        jvals = [(im*j+(i+halfway),im*j+i+halfway-im)[i+halfway > im - 1] for i in xrange(-1,im+1)]
        correct_gridids = jvals 
        #print "j % 3d -> %s" % (jm,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
        for j in xrange(jm-1,-1,-1):
            jvals = [im*j+im-1]
            jvals.extend([im*j+i for i in xrange(im)])
            jvals.extend([im*j])
            correct_gridids.extend(jvals)
            #print "j % 3d -> %s" % (j,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
        j = -1
        jvals = [(i+halfway,i+halfway-im)[i+halfway > im - 1] for i in xrange(-1,im+1)]
        correct_gridids.extend(jvals)
        #print "j % 3d -> %s" % (-1,"[%s]" % ", ".join(("% 6d" % x for x in jvals)))
        correct_gridids = set(correct_gridids)
        
        gdict = {}
        shift_row = 0
        shift_col = 0
        gridid_cnts = [0]*maxid
        fake_gridids = {}
        just_first_col = just_last_col = just_first_row = just_last_row = 0
        #just_first_col = 1
        #just_last_col = 1
        #just_first_row = 1
        #just_last_row = 1
        msg = "\tGridID = % 6d\tRow = % 3d\tColumn = % 3d\tLat = % 7.2f\tLon = % 7.2f"
        for tr in xrange(jm-1,-1,-1): # plot so 1st row on bottom of print out
            for tc in xrange(im):   
                if just_first_row:
                    if tr != 0: continue
                if just_last_row:
                    if tr != jm-1: continue
                if just_last_col:
                   if tc != im-1: continue     
                if just_first_col:
                    if tc != 0: continue
                    # just 1st couple of rows if 1 < tr < jm-2: continue                  
                true_row = tr + shift_row
                true_col = tc + shift_col
                tmp_gdict = make_gdict(true_row,true_col,c_halo,halfway,im,jm)
                d = dict((k,1) for k in tmp_gdict)
                fake_gridids.update(d)
                for k in tmp_gdict:
                    gridid_cnts[k] += 1
                gridid = (im*true_row) + true_col
                print msg % (gridid,true_row,true_col,lats[true_row],lons[true_col])
                print PrintGdict(tmp_gdict,tabs=2)
                gdict[gridid] = tmp_gdict 
        if not any([just_first_col,just_last_col,just_first_row,just_last_row]):
            #for k in xrange(len(gridid_cnts)):
            #    if gridid_cnts[k] > 0:
            #        print "\t\tGridID % 5d % 5d" % (k,gridid_cnts[k])
            print "\t\tNumber unique grids %d v. Expected %s" % (len(correct_gridids),no_dup_cnt)
            print "\t\tNumber grids used %d v. Expected %s" % (sum(gridid_cnts),dup_cnt)
                    
            fake_gridids = set(fake_gridids.keys())
            gridid_diff = correct_gridids.symmetric_difference(fake_gridids)
            if gridid_diff:
                print "\t\tError correct_gridids.symmetric_difference(fake_gridids)",gridid_diff
        
        #print len(gdict)
        #print PrintGdict(gdict[134],tabs=2)
        
    # -------------------------------------------------------------------------
    # END TESTS
    # -------------------------------------------------------------------------
