#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os
import gzip
import cPickle

# Third party module imports

def InjestSetup(fname,which_set):
    """
    Read setup file and return requested values.

    Input:
    ^^^^^
    fname - String
        Full path to requested setup file.
    which_set - Integer
        Which setup file to be read
        0 - Base Setup File
        1 - Center Finding Setup File
        2 - Track Finding Setup File
        3 - Attribution Finding Setup File

    Output:
    ^^^^^^
    sendback - List of names (Strings), filled values
        Names and values requested.

    Examples:
    ^^^^^^^^

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2011/01 MB - File created.
    """

    # See recipe inline_1 in inline.py
    # Get some definitions. Note must have run setup_vx.py already!
    try:
        fnc_out = cPickle.load(gzip.open(fname))
    except:
        sys.exit("\n\tWARNING: Error reading or finding %s." % (fname))

    # Parse fnc_out
    inputs = []
    if which_set == 0:
        # Base Setup file
        # Import the API written by mcms_setup.py for this file.   
        f = os.path.split(fname)[0]
        sys.path.append(f)
        import s_dat
        inputs = s_dat.inputs[:]
    elif which_set == 1:
        # Import the API written by mcms_setup.py for this file.   
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import cf_dat
        inputs = cf_dat.inputs[:]
    elif which_set == 2:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import tf_dat
        inputs = tf_dat.inputs[:]
    elif which_set == 3:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import df_dat
        inputs = df_dat.inputs[:]
    elif which_set == 4:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import time_dat
        inputs = time_dat.inputs[:]
    else:
        sys.exit("Warning unknowned which_set in InjestSetup")
    return fnc_out,inputs

