#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os

def Is_It_Available(program):
    """
    Is some program available on this system?

    Simple test to see if a program is available for use by MCMS.

    Input:
    ^^^^^
    program - String
        The name of the program for the existence/availability test.

    Output:
    ^^^^^^
    result - Boolean
        True/False if program is found on the system.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> Is_It_Available("python")
    True

    >>> Is_It_Available("bython")
    False

    Notes/Warnings:
    ^^^^^^^^^^^^^^
    Pretty much a duplicate of the recipe suggested under the question
    'Test if executable exists in Python?' at http://stackoverflow.com

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2009/11 MB - File created.
    """
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK)
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True
    return False


