import numpy

def l2g2(lon,lat,i_lons,j_lats,n):
    """Converts lon, lat into nearest gridID
    This version is 10x faster!
    """
    # Convert longitude to 0-360 if not already
    lon = (lon + 360.0)%360.0

    if i_lons[-1] < lon <= 360.0:
        # Special Case problem when lon falls between 0/360
        #   and the next longitude over say 357 in which
        #   case the minimization below won't work as it'll
        #   never see say 359.3 being closer to 0 than to
        #   357. So in these cases convert everything to
        #   the +- 180 system 1st.
        # Convert longitude to +-180 if not already
        lon = ((lon+180.0)%360.0)-180.0
        lons = numpy.array([((x+180.0)%360.0)-180.0 for x in i_lons])
    else:
        lons = i_lons

    # Find i_lon closest to lon
    i = (numpy.abs(lons-lon)).argmin()
    # Find j_lat closest to lat
    j = (numpy.abs(j_lats-lat)).argmin()
    gridid = j*n + i
    return gridid,i_lons[i],j_lats[j]

def l2g(lon,lat,i_lons,j_lats):
    """Converts lon, lat into nearest gridID"""

    # Convert longitude to 0-360 if not already
    lon = (lon + 360.0)%360.0

    #if lon < 0.0:
    #    lon += 360.0

    # Find i_lon closest to lon
    lon_diffs = [abs(lon-x) for x in i_lons]
    tmp = lon_diffs[:] # store unsorted copy
#    for i in xrange(len(i_lons)):
#        print i,i_lons[i],lon_diffs[i]
    lon_diffs.sort() #ADDED!=
    closest = lon_diffs[0]
#    print closest
    # i-index of closest center longitude
    i = tmp.index(closest)
#    print i

    # Find j_lat closest to lat
    lat_diffs = [abs(lat-x) for x in j_lats]
#    for j in xrange(len(j_lats)):
#        print j,j_lats[j],lat_diffs[j]
    tmp = lat_diffs[:] # store unsorted copy
    lat_diffs.sort()
    closest = lat_diffs[0]
#    print closest
    # j-index of closest center latgitude
    j = tmp.index(closest)
#    print j

    # Determine gridID
    if i < len(i_lons) and j < len(j_lats):
        gridid = j*len(i_lons) + i
    else:
        import sys;sys.exit("ERROR in l2g")

#    print "\tConverted % 4d % 4d to % 6d" % (lon,lat,gridid)

    return gridid,i_lons[i],j_lats[j]

# #---Start of main code block.
# if __name__=='__main__':
# 
#     lons = [0.0, 2.5, 5.0, 7.5, 10.0, 12.5, 15.0, 17.5, 20.0, 22.5, 25.0,
#             27.5, 30.0, 32.5, 35.0, 37.5, 40.0, 42.5, 45.0, 47.5, 50.0, 52.5
#             , 55.0, 57.5, 60.0, 62.5, 65.0, 67.5, 70.0, 72.5, 75.0, 77.5,
#             80.0, 82.5, 85.0, 87.5, 90.0, 92.5, 95.0, 97.5, 100.0, 102.5,
#             105.0, 107.5, 110.0, 112.5, 115.0, 117.5, 120.0, 122.5, 125.0,
#             127.5, 130.0, 132.5, 135.0, 137.5, 140.0, 142.5, 145.0, 147.5,
#             150.0, 152.5, 155.0, 157.5, 160.0, 162.5, 165.0, 167.5, 170.0,
#             172.5, 175.0, 177.5, 180.0, 182.5, 185.0, 187.5, 190.0, 192.5,
#             195.0, 197.5, 200.0, 202.5, 205.0, 207.5, 210.0, 212.5, 215.0,
#             217.5, 220.0, 222.5, 225.0, 227.5, 230.0, 232.5, 235.0, 237.5,
#             240.0, 242.5, 245.0, 247.5, 250.0, 252.5, 255.0, 257.5, 260.0,
#             262.5, 265.0, 267.5, 270.0, 272.5, 275.0, 277.5, 280.0, 282.5,
#             285.0, 287.5, 290.0, 292.5, 295.0, 297.5, 300.0, 302.5, 305.0,
#             307.5, 310.0, 312.5, 315.0, 317.5, 320.0, 322.5, 325.0, 327.5,
#             330.0, 332.5, 335.0, 337.5, 340.0, 342.5, 345.0, 347.5, 350.0,
#             352.5, 355.0, 357.5]
# 
#     lats = [-89.375, -87.5, -85.0, -82.5, -80.0, -77.5, -75.0, -72.5, -70.0,
#             -67.5, -65.0, -62.5, -60.0, -57.5, -55.0, -52.5, -50.0, -47.5,
#             -45.0, -42.5, -40.0, -37.5, -35.0, -32.5, -30.0, -27.5, -25.0,
#             -22.5, -20.0, -17.5, -15.0, -12.5, -10.0, -7.5, -5.0, -2.5, 0.0,
#             2.5, 5.0, 7.5, 10.0, 12.5, 15.0, 17.5, 20.0, 22.5, 25.0, 27.5, 30.0,
#             32.5, 35.0, 37.5, 40.0, 42.5, 45.0, 47.5, 50.0, 52.5, 55.0, 57.5,
#             60.0, 62.5, 65.0, 67.5, 70.0, 72.5, 75.0, 77.5, 80.0, 82.5, 85.0,
#             87.5, 89.375]
# 
#     # Convert to +-180
#     lons_pm = numpy.array([((lon+180.0)%360.0)-180.0 for lon in lons])
# 
#     lons = numpy.array(lons)
#     lats = numpy.array(lats)
#     n = len(lons)
# 
#     import first_last_lons_v5
#     row_start,row_end = first_last_lons_v5.first_last_lons(len(lats),len(lons))
#     row_start_index = row_start.index
#     #test_lons = [1.7899,-5.09e-06,0.01,-1.7899]
#     start = 2.5
#     end = -2.5
#     count = 20
#     delta = (end-start)/float(count)
#     test_lons = [start,] + map( lambda x:delta*x + start, range( 1, count ) ) + [end, ]
#     test_lats = [-64.85]*len(test_lons)
#     for i in range(len(test_lons)):
#         gridid,ilon,ilat = l2g2(test_lons[i],test_lats[i],lons,lats,n)
#         print test_lons[i],test_lats[i]
#         print "\t",gridid,ilon,ilat
#         row_guess = [x for x in row_start if x <= gridid]
#         row_guess = row_start_index(row_guess[-1])
#         print "\t\t",row_guess,row_start[row_guess],row_end[row_guess]
#         print
#     import sys; sys.exit("Stop Here")
# 
#     results = {}
#     for lat in lats:
#         for lon in lons:
#             gridid,ilon,ilat = l2g(lon,lat,lons,lats)
#             gridid2,ilon2,ilat2 = l2g2(lon,lat,lons_pm,lats,n)
#             if gridid-gridid2:
#                 print lat,lon
#                 print gridid,ilon,ilat
#                 print gridid2,ilon2,ilat2
#                 print lon,(lon + 360.0)%360.0
#                 import sys; sys.exit("Stop Here")
#             results[gridid] = 1
#     if len(results.keys()) != len(lons)*len(lats):
#         print len(results.keys()),len(lons)*len(lats)

#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    import sys
    import cPickle
    shared_path = "/Volumes/scratch/output/nra2_files/"            
    # Get some definitions. Note must have run setup_vx.py already!
    sf_file = "%ss_dat.p" % (shared_path)
    fnc_out = cPickle.load(open(sf_file))
    (im,jm,maxid,lats,lons,timestep,dx,dy,dlon,dlat,start_lat,start_lon,
            dlon_sq,dlat_sq,two_dlat,model_flag,eq_grid,tropical_n,tropical_s,
            bot,mid,top,row_start,row_end,tropical_n_alt,tropical_s_alt,
            bot_alt,top_alt,lon_shift,lat_flip,the_calendar,found_years,
            super_years,dim_lat,dim_lon,dim_time,var_lat,var_lon,var_time,
            var_slp,var_topo,var_land_sea_mask,file_seperater,no_topo,
            no_mask,slp_path,model,out_path,shared_path,lat_edges,lon_edges,
            land_gridids,troubled_centers,faux_grids) = fnc_out
    # Save memory
    del troubled_centers
    del lat_edges
    del lon_edges
    del fnc_out

    print l2g2(53.19,69.67,lons,lats,im)
