#!/usr/bin/env python -tt
"""This module creates a gdict 3x3 grid.

Options/Arguments:

Returns/Creates:

Examples:

Notes:
     upm     upc       upp     0 1 2  (x,y) -1,+1  0,+1  +1,+1 
     cnm     cnt       cnp     3 4 5        -1, 0  0, 0  +1, 0
     dnm     dnc       dnp     6 7 8        -1,-1  0,-1  +1,-1

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
import numpy as np
cimport numpy as np
DTYPE = np.int32
ctypedef np.int32_t DTYPE_i
cimport cython
@cython.boundscheck(False)
@cython.wraparound(False)
@cython.nonecheck(False)

# This much fast as more purely C than python
def make_gdict_np(int trow, int tcol, np.ndarray[DTYPE_i,ndim=1,mode='c'] c_halo, 
        int halfway, int im, int jm):    
     cdef int i,joff,ioff,frows[9],fcols[9]
     cdef list gdict
     assert c_halo.dtype == DTYPE
     i = 0
     for joff in [1,0,-1]:
        for ioff in [-1,0,1]:
            if 0 <= trow+joff < jm:
                frows[i] = trow+joff
                fcols[i] = tcol+ioff
            if trow+joff < 0:
                frows[i] = trow+joff + 1
                fcols[i] = tcol+ioff + halfway
            elif trow+joff >= jm: 
                frows[i] = trow+joff - 1
                fcols[i] = tcol+ioff + halfway
            i = i + 1
     for i in xrange(9):
        fcols[i] = c_halo[fcols[i]+im] 
     gdict = []        
     for i in xrange(9):
         gdict.append(im*frows[i]+fcols[i])
     return gdict

#def make_gdict_np(int trow, int tcol, np.ndarray[DTYPE_i,ndim=1,mode='c'] c_halo, 
#        int halfway, int im, int jm):    
#     cdef list tmp,frows,fcols,gdict
#     cdef int joff,ioff
#     assert c_halo.dtype == DTYPE 
#     tmp = [get_true_row(trow+joff,tcol+ioff,halfway,im,jm) 
#             for joff in [1,0,-1] for ioff in [-1,0,1]]
#     frows = [x[0] for x in tmp]
#     fcols = [c_halo[x[1]+im] for x in tmp]
#     tmp = [(frows[x],fcols[x]) for x in xrange(9)]
#     gdict = [(im*x[0]+x[1]) for x in tmp]
#     return gdict

#def get_true_row(int trow, int tcol, int halfway, int im, int jm):
#    if 0 <= trow < jm: 
#        # Find as is.
#        return trow,tcol
#    elif trow < 0:
#        # South Pole roll over
#        #   A trow of -1 lands on the same row, but the column sifted im/2
#        return trow+1,tcol+halfway
#    elif trow >= jm: 
#        # North Pole roll over
#        #   A trow of jm lands on jm-1, but the column sifted im/2
#        return trow-1,tcol+halfway
#    else:
#        return 1

#def make_gdict_c(int trow, int tcol,int halfway, int im, int jm):
#     cdef list c_halo = [x  for y in xrange(3) for x in xrange(im)]
#     cdef list tmp,frows,fcols,gdict
#     tmp = [get_true_row(trow+joff,tcol+ioff,halfway,im,jm) 
#             for joff in [1,0,-1] for ioff in [-1,0,1]]
#     frows = [x[0] for x in tmp]
#     fcols = [c_halo[x[1]+im] for x in tmp]
#     tmp = [(frows[x],fcols[x]) for x in xrange(9)]
#     gdict = [(im*x[0]+x[1]) for x in tmp]
#     return gdict

