#!/usr/bin/env python -tt
"""
Generic class for making selections

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.


Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
import os

# Third party module imports
from pprint import pformat

# ----------------------------------------------------------------------------

class pick_factory():
    '''Generic class for making selections'''

    def __init__(self,*args):
        # Name of the specific object being created.
        self.function_name = args[0]
        # Must be a dictionary
        self.selections = args[1]
        # All string format statements
        self.report_fmt = args[2]
        self.error_fmt = args[3]
        self.usage_fmt = args[4]

    def make_pick(self,pick):
        """Processes the selection"""
        if pick in self.selections:
            self.pick = pick
            self.selection = self.selections[self.pick][1]
            return self.report()
        else:
            sys.exit(self.error_fmt % (self.function_name,repr(pick),
                self.usage(tabs=2),self.available(tabs=3)))

    def set_path(self,the_path):
        """Set some paths"""
        if the_path:
            if not os.path.exists(the_path):
                sys.exit(self.error_fmt % (self.function_name,the_path))
            self.the_path = the_path
        else:
            self.the_path = ''
        tmp = ("Use over_write: %s" % self.the_path if self.the_path else
                "Use default from setup_vX.py: %s" % self.the_path)
        return self.report_fmt % (self.the_path,tmp)

    def set_list(self,the_list):
        """Set some range in a list"""
        if the_list:
            self.the_list = the_list
        else:
            self.the_list = []
        tmp = ("Use over_write: %s" % repr(self.the_list) if self.the_list else
                "Use default from setup_vX.py: %s" % self.the_list)
        return self.report_fmt % (self.the_list,tmp)

    def usage(self,tabs=0):
        """Returns usage message"""
        return self.usage_fmt % ("\t"*tabs,self.function_name,
                repr(self.selections.keys()))

    def available(self,tabs=0):
        """Returns valid selections"""
        tmp = pformat(self.selections)
        if tabs:
            rplace = "\n"+"\t"*tabs
            tmp = "\t"*tabs+tmp
            tmp = tmp.replace("\n",rplace)
        return tmp

    def report(self,):
        """Returns a string of what was selected"""
        return self.report_fmt % (self.pick,
                self.selections[self.pick][0],
                self.selections[self.pick][1])

def inject_defs(fname,user_file_template,import_me,base_defs):
    if not os.path.exists(fname):
        # Create an empty user source file
        fn = open(fname,'w')
        fn.writelines(user_file_template)
        fn.close()
    # Read user source file
    __import__(import_me)
    user_defs = sys.modules[import_me]
    # See if any additions
    del user_defs.user_selections["name"]
    return user_defs

def define_something(fname,base_selections,NAME):
    template = '''#!/usr/bin/env python -tt
"""   User Provided NAME definitions for the MCMS project.
   See mcms_NAME_base_defs.py for examples.
"""
user_selections = {"name" : ["long name","shortname"]}
'''
    template = template.replace("NAME",NAME)
    import_me = "cfg.%s_defs" % NAME
    object_name = "pick_%s" % NAME
    report_fmt = "NAME:\t%s (aka %s or %s)"
    report_fmt = report_fmt.replace("NAME",NAME.upper())
    error_fmt = "\tERROR in %s: Invalid pick '%s'.\n%s\n%s.\n"
    usage_fmt = "%sUsage: %s[pick] where pick is from %s"
    user_inputs = inject_defs(fname,template,import_me,base_selections)
    if user_inputs.user_selections.keys():
        base_selections.update(user_inputs.user_selections)
    # Pass into pick_factory
    return pick_factory(object_name,base_selections,report_fmt,
        error_fmt,usage_fmt)

#---Start of main code block. This show how to use the above code
# if __name__=='__main__':
#
#     report_fmt = "Model/Source:\t%d (aka %s or %s)"
#     error_fmt = "\tERROR in %s: Invalid pick '%s'.\n%s\n%s.\n"
#     usage_fmt = "%sUsage: %s[pick] where pick is from %s"
#     model_selections = {0: ["Test/Debug","test"],
#             1 : ["NCEP/NCAR Reanalysis 1","nra"],
#             2 : ["NCEP-DOE Reanalysis 2","nra2"],
#             3 : ["NASA GISS GCM ModelE","giss"],
#             4 : ["GFDL GCM","gfdl"],
#             5 : ["ERA-Interim Reanalysis","erai"],
#             6 : ["ERA-40 Reanalysis","era40"]
#             }
#         self.pick_model = pick_factory("pick_model",model_selections,
#                 report_fmt,error_fmt,usage_fmt)
