"""Use matplotlib to make 1d histograms or PDFs"""

def plot_2d_histo(*args,**kwargs):
    """Use matplotlib to make 1d histograms or PDFs"""
    
    # Standard modules
    import sys
    import os
    import numpy
    import math
    
    # Third party modules
    import matplotlib.pyplot as plt
    from scipy import interpolate

    #import matplotlib.cm as cm
    
    #import matplotlib.dates as mdates
    #import matplotlib.ticker as ticker
    #import matplotlib.mlab
    import scipy.stats
    
    # MCMS modules
    #from mcms_track_finder import process_data, rose_plot
    
    # Median = numpy.median
    # PDF = matplotlib.mlab.normpdf
    # Summerize = scipy.stats.describe
    NA = numpy.array
    Median = numpy.median
    # NMean = numpy.mean
    # NSTD = numpy.std
    # NAve = numpy.average
    # Mode = scipy.stats.mode
    # where_mode = numpy.argmax
    Percentile = scipy.stats.scoreatpercentile
    Mode = scipy.stats.mode
    # Skew = scipy.stats.skew
    # Kurtosis = scipy.stats.kurtosis   

    # Main data to be ploted
    xpile = args[0]
    ypile = args[1]
    # Base name for plot
    pname_base = args[2]
    # plot script
    plot_cmd = args[3]

    x_scp = args[4].split(";")
    y_scp = args[5].split(";")

    key_word_args = ["title","xlab","xupper","xlower"]
    for kw in key_word_args:
        cmd = [x.lstrip() for x in x_scp if x.find(kw) != -1][0]
        exec(cmd)
        cmd = [x.lstrip() for x in y_scp if x.find(kw) != -1][0]
        if kw == "xlab":
            cmd = cmd.replace("xlab","ylab")
        if kw == "xlower":
            cmd = cmd.replace("xlower","ylower")
        if kw == "xupper":
            cmd = cmd.replace("xupper","yupper")       
        exec(cmd)            
 
    # Name: title
    # Purpose: String to use as title of plot
    #
    # Default: "Title"
    if not title:
        title = "Title"

    # Name: xlab
    # Purpose: String to use as label for x-axis
    #
    # Default: "X-Axis"
    if not xlab:
        xlab = "X-Axis"
    
    # Name: ylab
    # Purpose: String to use as label for y-axis
    #
    # Default: "Y-Axis"
    #
    # Note: Watch for state of normalized.
    if not ylab:
        ylab = "Y-Axis"

    # Name: no_title
    # Purpose: Do not add a title to the plot
    #
    # Default: True
    if kwargs.has_key('no_title'):
        no_title = kwargs['no_title']
    else:
        no_title = 1

    # Name: use_grid
    # Purpose: add a grid to the plot
    #
    # Default: True
    if kwargs.has_key('use_grid'):
        use_grid = kwargs['use_grid']
    else:
        use_grid = 1

    # Name: fig_format
    # Purpose: format of the output plots
    #
    # Default: png
    #
    if kwargs.has_key('fig_format'):
        fig_format = kwargs['fig_format']
    else:
        fig_format = '.png'

    # Name: cbar
    # Purpose: Draw colarbar (the string is the label, otherwise empty)
    #
    # Default: False
    if kwargs.has_key('cbar'):
        cbar = True
        clabel = kwargs['cbar']
    else:
        cbar = False
        
    # Make a numpy array
    xpilen = NA(xpile)
    ypilen = NA(ypile)

    # Test for all the same value.
    if abs(xpilen.max()-xpilen.min()) <= 0.00000000001:
        return "Flat Skipped: %s" % pname_base
    if abs(ypilen.max()-ypilen.min()) <= 0.00000000001:
        return "Flat Skipped: %s" % pname_base

    # Create a figure (histogram) using matplotlib
    # --------------------------------------------
    pname = '%s%s' % (pname_base,fig_format)

    # Start figure
    golden_ratio = 1.61803399
    width = 10.0
    hieght = 10.0#width/golden_ratio
    if cbar:
        width = 9.0
        hieght = 7.0#width/golden_ratio
    else:
        width = 7.0
        hieght = 7.0#width/golden_ratio

    fig = plt.figure(figsize=(width,hieght))
    ax = fig.add_subplot(111)

    # Execute Plot
    import tmp
    #__import__(plot_cmd)
    tmp.plot_cmd_2d_histo(xpilen,ypilen,fig,ax,plt)

    #if isinstance(plot_cmd,str):
    #    for each in plot_cmd:
    #        print each
    #        exec(each)
    #else:
    #    plot_cmd(xpilen,ypilen,fig,ax,plt)
  
    # Add Labels and such
    ax.set_xlabel(xlab)
    ax.set_ylabel(ylab)

    if not no_title:
        # Add title
        ax.set_title(title)
                        
    if use_grid:
        ax.grid(True)
     
    if cbar:
        cb = plt.colorbar(orientation='vertical',shrink=0.75)
        cb.set_label(clabel)
        
    # Save to File
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    #import sys; sys.exit("Stop Here")
      
    return "Made %s" % (pname) 
