from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

import numpy
import matplotlib.pyplot as plt

def plot_hist(x,y,width,stat_file,pname,title,xlab,make_plot):
    total = y.sum()
    if total < 1:
        msg = "plot_hist %s skipped total count 0" % pname
        return msg
    cumsum = [(z/total)*100.0 for z in numpy.cumsum(y)]
    stat_save = open(stat_file,"w")
    msg = ''.join(["%8.3f" % (z) for z in x])
    stat_save.writelines("Bins  "+msg+"\n")
    msg = ''.join(["%8.3f" % (z) for z in cumsum])
    stat_save.writelines("CFrac "+msg+"\n")
    hmean = x.sum()/len(x)
    hmin = 0
    for tmp in y:
        if tmp > 0.0:
            break
        hmin += 1
    hmax = -1
    for tmp in y[::-1]:
        if tmp > 0.0:
            break
        hmax -= 1
    msg = "%7.3f" % (hmean)
    stat_save.writelines("Mean  "+msg+"\n")
    msg = "%7.3f" % (x[hmin])
    stat_save.writelines("Min   "+msg+"\n")
    msg = "%7.3f" % (x[hmax])
    stat_save.writelines("Max   "+msg+"\n")
    stat_save.close()
    if not make_plot:
        return 

    fig = plt.figure()
    ax = fig.add_subplot(111)
    plt.bar(x,y,width=width,color='0.6',edgecolor='k')
    # Add Labels and such
    ax.set_xlabel(xlab)
    ax.set_ylabel("Count")
    # Add title
    ax.set_title(title)

    #        ax.grid(True)
    # Save to File
    fig.savefig(pname,dpi=144,orientation='landscape')
    plt.close('all')
    return "Made %s" % (pname)
