"""Use matplotlib to make 1d histograms or PDFs"""

def plot_histograms(*args,**kwargs):
    """Use matplotlib to make 1d histograms or PDFs"""
    
    # Standard modules
    import sys
    import os
    import numpy
    import math
    
    # Third party modules
    import matplotlib.pyplot as plt
    import matplotlib.dates as mdates
    import matplotlib.ticker as ticker
    import matplotlib.mlab
    import scipy.stats
    
    # MCMS modules
    #from mcms_track_finder import process_data, rose_plot
    
    Median = numpy.median
    PDF = matplotlib.mlab.normpdf
    Summerize = scipy.stats.describe
    NA = numpy.array
    NMean = numpy.mean
    NSTD = numpy.std
    NAve = numpy.average
    Mode = scipy.stats.mode
    where_mode = numpy.argmax
    Percentile = scipy.stats.scoreatpercentile
    Skew = scipy.stats.skew
    Kurtosis = scipy.stats.kurtosis   

    # Main data to be ploted
    pile = args[0]
    # Extra data, usually dates
    extra_pile = args[1]
    # Base name for plot
    pname_base = args[2]
    # plot script
    plot_cmd = args[3]
    # plot ranges etc
    p_crits = args[4].split(";")

    # Parse p_crits
    cmd = [x.lstrip() for x in p_crits if x.find("bin_width") != -1][0]
    exec(cmd)
    
    cmd = [x.lstrip() for x in p_crits if x.find("bins_left_edge") != -1][0]
    exec(cmd)
    
    bins_width = abs(bins_left_edge[0]-bins_left_edge[1])
    bins_centers = bins_left_edge + 0.5*bins_width
    #bins_right_edge = bins_left_edge + bins_width
    
    # Setup increments for histogram X-axis labels
    #   nlabels = X # number of desired labels
    #   bininc = int(abs(max_bin_value - min_bin_value)/nlabels)
    #   binskip = int(bininc/bins_width) # number of bins to skip as labeling
    #  Example
    #   nlabels = 6
    #   bininc = abs(2400 - 0)/6 = 400
    #   binskip = 400/100 = 4 # use every forth bin as a label
    global binskip
    nlabels = 6
    bininc = int(abs(bins_centers[-1]-bins_centers[0])/nlabels)
    # The extra 1 is because python indices start with 0
    binskip = int(bininc/bins_width)+1
    if binskip < 1: binskip = 1
    
    # This scales the normalized histo counts so that they are % of total count.
    global nscale
    nscale  = 100.0*bins_width
    
    key_word_args = ["normalized","title","xlab","ylab","sum_type","xlabt",
        "ylabt","ytupper","ytlower","xlab2d","ylab2d","yuppern","xupper",
        "xlower","trim_max","trim_min","may_trim"]
    for kw in key_word_args:
        cmd = [x.lstrip() for x in p_crits if x.find(kw) != -1][0]
        exec(cmd)

    #normalized = 0    
    # Name: normalized
    # Purpose: Counts normalized to form a probability density,
    #          i.e., n/(len(x)*dbin).
    #
    # Default: False or ""

    # Name: may_trim
    # Purpose: Allow adjustment of bins edges.
    #
    # Default: False

    # Name: title
    # Purpose: String to use as title of plot
    #
    # Default: "Title"
    if not title:
        title = "Title"

    # Name: xlab
    # Purpose: String to use as label for x-axis
    #
    # Default: "Bins"
    if not xlab:
        xlab = "Bins"

    # Name: ylab
    # Purpose: String to use as label for y-axis
    #
    # Default: "Count"
    #
    # Note: Watch for state of normalized.
    if not ylab:
        if normalized:
            ylab = "Normalized Fraction of Total"
        else:
            ylab = "Count"

    # Name: ytlower
    # Purpose: Set lower value for y-axis time-series
    #
    # Default: None
    if not ytlower:
        ytlower = None

    # Name: ytupper
    # Purpose: Set upper value for y-axis time-series
    #
    # Default: None
    if not ytupper:
        ytupper = None

    # Name: yuppern
    # Purpose: Set upper value for y-axis normalized histograms
    #
    # Default: None (set by matplotlib
    if not yuppern:
        yuppern = None

    # Name: xupper
    # Purpose: Set upper value for x-axis for histograms
    #
    # Default: None (set by matplotlib
    if not xlower:
    #if xupper == -999:
        xupper = None
        #xupper = bins_left_edge[-1]+bin_width*0.5
        
    # Name: xlower
    # Purpose: Set lower value for x-axis for histograms.
    #
    # Default: 0
    #
    # Note: Only used if xupper used
    if not xlower:
    #if xlower == -999:
        xlower = 0
        ##xlower = bins_left_edge[0]-bin_width*0.5

    # Name: trim_max
    # Purpose: Set upper value for y-axis time-series trim
    #
    # Default: None
    if not trim_max:
        trim_max = None

    # Name: trim_min
    # Purpose: Set upper value for y-axis time-series trim
    #
    # Default: None
    if not trim_min:
        trim_min = None

    # Name: stat_2d
    # Purpose: Add stats to the plot (x-axis values)
    #
    # Default: False
    if kwargs.has_key('stat_2d'):
        stat_2d = kwargs['stat_2d']
    else:
        stat_2d = False

    # Name: no_title
    # Purpose: Do not add a title to the plot
    #
    # Default: True
    if kwargs.has_key('no_title'):
        no_title = kwargs['no_title']
    else:
        no_title = 1

    # Name: no_stats
    # Purpose: Do not add stats to the plot (y-axis values)
    #
    # Default: True
    if kwargs.has_key('no_stats'):
        no_stats = kwargs['no_stats']
    else:
        no_stats = 1

    # Name: polar
    # Purpose: Makeing a polar-rose diagram
    #
    # Default: False
    #
    if kwargs.has_key('polar'):
        polar = 1
    else:
        polar = 0

    # Name: use_grid
    # Purpose: add a grid to the plot
    #
    # Default: True
    if kwargs.has_key('use_grid'):
        use_grid = kwargs['use_grid']
    else:
        use_grid = 1

    # Name: time
    # Purpose: plot timeseries with very wide figure
    #
    # Default: False
    #
    if kwargs.has_key('time'):
        time = kwargs['time']
    else:
        time = 0

    # Name: fig_format
    # Purpose: format of the output plots
    #
    # Default: png
    #
    if kwargs.has_key('fig_format'):
        fig_format = kwargs['fig_format']
    else:
        fig_format = '.png'

    # Name: skip_plot
    # Purpose: Don't make plot, just get stats
    #
    # Default: False
    #
    if kwargs.has_key('skip_plot'):
        skip_plot = kwargs['skip_plot']
    else:
        skip_plot = False
 
    # Name: no_report
    # Purpose: Don't make find stats
    #
    # Default: False
    #
    if kwargs.has_key('no_report'):
        no_report = kwargs['no_report']
    else:
        no_report = False    

    # Test for nothing
    if len(pile) < 1:
       return "Nothing to plot: %s" % pname_base

    # Make a numpy array
    pile2 = NA(pile)

    # Test for all the same value.
    if abs(pile2.max()-pile2.min()) <= 0.00000000001:
        return "Flat Skipped: %s" % pname_base

    # Create a figure (histogram) using matplotlib
    # --------------------------------------------
    pname = '%s%s' % (pname_base,fig_format)

    # Start figure
    if not polar:
        if time:
            golden_ratio = 1.61803399*2.0
            width = 40.0
        else:
            golden_ratio = 1.61803399
            width = 10.0
        hieght = width/golden_ratio

        fig = plt.figure(figsize=(width,hieght))
        ax = fig.add_subplot(111)

    # Execute Plot
    for each in plot_cmd:
        exec(each)

    # Add Labels and such
    ax.set_xlabel(xlab)
    if not polar:
        ax.set_ylabel(ylab)

    if not no_title:
        # Add title
        ax.set_title(title)

    report_data = {}  
    if not no_report or not no_stats:        
        fnc_out = Summerize(pile)
        modei = where_mode(n)
        modev = bins_centers[modei]
        # print
        # print pname
        # print "bins",len(bins),"\t",bins
        # print "n",len(n),"\t",n,"\t",numpy.sum(n * numpy.diff(bins))
        # print "modei",modei
        # print "modev",modev
        # print "\nbins_width",bins_width
        # print "bins_left_edge",len(bins_left_edge),"\t",bins_left_edge
        # print "bins_centers",len(bins_centers),"\t",bins_centers
        # print "bins_right_edge: ",len(bins_left_edge),"\t",bins_left_edge + bins_width
    
        report_data["Count"] = fnc_out[0]
        report_data["Min"] = fnc_out[1][0]
        report_data["Max"] = fnc_out[1][1]
        report_data["Mean"] = fnc_out[2]
        report_data["Median"] = Median(pile)
        report_data["Mode"] = modev
        report_data["STD"] = math.sqrt(fnc_out[3])
        report_data["Skew"] = Skew(pile)
        report_data["Kurt"] = Kurtosis(pile)
        report_data["per5"] = Percentile(pile,5.0)
        report_data["per95"] = Percentile(pile,95.0)
        report_data["ModeAlt"] = Mode(pile)[0][0]
        report_data["n"] = n
        report_data["bins"] = bins

    if not no_stats:
#         fmter = "Count: %d\nMin/Max: %d/%d\nMean/Median/Mode: %.2f/%.2f/%.2f\nSTD: %.2f"
#         fnc_out = Summerize(pile)
#         stitle = fmter % (fnc_out[0],fnc_out[1][0],fnc_out[1][1],fnc_out[2],Median(pile),
#                           Mode(pile)[1][0],fnc_out[3])
        # Note mode can take a very long time for very large arrays
        fmter = "Count: %d\nMin/Max: %d/%d\nMean/Median: %.2f/%.2f\nMode: %.2f\nSTD: %.2f"
        
        stitle = fmter % (report_data["Count"],report_data["Min"],report_data["Max"],report_data["Mean"],report_data["Median"],modev,report_data["STD"])
        #stitle = fmter % (fnc_out[0],fnc_out[1][0],fnc_out[1][1],fnc_out[2],Median(pile),modev,fnc_out[3])
        if time:
            plt.suptitle(stitle, fontsize=4,x=0.875,y=0.93,horizontalalignment='left')
        else:
            #plt.suptitle(stitle, fontsize=4,x=0.8,y=0.95,horizontalalignment='left')
            plt.suptitle(stitle, fontsize=4,x=0.8,y=0.96,horizontalalignment='left')
            
    if use_grid:
        ax.grid(True)
        
    # Save to File
    if not skip_plot:
        fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')
    
    return ("Made %s" % (pname),report_data)