def plot_lifeplots(*args,**kwargs):
    """Use matplotlib to make line plots"""
    
    # Standard modules

    # Third party modules
    import numpy
    import matplotlib.pyplot as plt

    box_data = args[0]
    box_dates = args[1]
    box_index = args[2]
    averages = args[3]
    stds = args[4]
    counts = args[5]
    cname = args[6]
    ylabel = args[7]
    title = args[8]

    NAN = numpy.nan
    NSQRT = numpy.sqrt
    
    golden_ratio = 1.61803399*2.0
    #width = 40.0
    #hieght = width/golden_ratio
    #hieght = 10.0
    #width = 11.0   
    
    #hieght = 5.0
    #width = 5.5   
    
    hieght = 7.0
    width = hieght*1.1  
   
    # For Line plots with error bars
    #------------------------------
    bp = plt.boxplot(box_data, notch=0, sym='', vert=1, whis=1.5)
    numBoxes = len(bp['medians'])   

    fig = plt.figure(figsize=(width,hieght))
    ax = fig.add_subplot(111)
    std_error = []
    for i in xrange(numBoxes):
        if counts[i]:
            std_error.append(2.0*(stds[i]/NSQRT(counts[i])))
        else:
            std_error.append(NAN)
    plt.errorbar(box_index,averages,yerr=std_error,marker="o",ms=5,color='k',
        ecolor='k',linestyle="-",elinewidth=1,lw=2,capsize=5)
    # Thin light grey line along y-axis
    #ax.grid()
    #ax.yaxis.grid(True,linestyle='-',which='major',color='lightgrey',alpha=0.5)
    # Hide these grid behind plot objects
    #
    xmin1, xmax1 = plt.xlim()
    plt.xlim(xmax=xmax1+1)
    # vertical line over 0-hour
    plt.axvline(x=box_dates.index(0)+1,linestyle='-',color='k',alpha=0.75)

    # Change Labels
    # Set font size
    for labeltick in ax.xaxis.get_majorticklabels():
        labeltick.set_fontsize(11)
    # Alter label
    locs, labels = plt.xticks()
    plt.xticks(box_index,box_dates)
    # Axis & Title
    ax.set_xlabel('Hours Relative to Lifetime Minimum SLP',verticalalignment='top')
    ax.set_ylabel(ylabel)
    if title:
        ax.set_title(title)

    fig.savefig(cname,dpi=144)
    plt.clf()
    plt.close('all')
    return "\t\t\tMade ",cname