import numpy
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import scipy.stats
import matplotlib.mlab 

def plot_stats(*args,**kwargs):
    """Use matplotlib to make 1d histograms or PDFs"""

    Median = numpy.median
    PDF = matplotlib.mlab.normpdf
    Summerize = scipy.stats.describe
    NA = numpy.array
    NMean = numpy.mean
    NSTD = numpy.std

    pile = args[0]
    extra_pile = args[1]
    pname_base = args[2]
    bins_left_edge = args[3]
    bins_centers = args[4]
    bins_width = args[5]
    tail = args[6]

    # pass in a string command for the particular plot
    plot_cmd = args[7]

    # Name: normalized
    # Purpose: Counts normalized to form a probability density,
    #          i.e., n/(len(x)*dbin).
    #
    # Default: False
    if kwargs.has_key('normalized'):
        normalized = kwargs['normalized']
    else:
        normalized = False

    # Name: may_trim
    # Purpose: Allow adjustment of bins edges.
    #
    # Default: False
    #
    # Note:
    #
    if kwargs.has_key('may_trim'):
        may_trim = kwargs['may_trim']
    else:
        may_trim = 0

    # Name: title
    # Purpose: String to use as title of plot
    #
    # Default: "Title"
    #
    if kwargs.has_key('title'):
        title = kwargs['title']
    else:
        title = "Title"

    # Name: xlab
    # Purpose: String to use as label for x-axis
    #
    # Default: "Bins"
    #
    if kwargs.has_key('xlab'):
        xlab = kwargs['xlab']
    else:
        xlab = "Bins"

    # Name: ylab
    # Purpose: String to use as label for y-axis
    #
    # Default: "Count"
    #
    # Note: Watch for state of normalized.
    #
    if kwargs.has_key('ylab'):
        ylab = kwargs['ylab']
    else:
        if normalized:
            ylab = "Normalized Fraction of Total"
        else:
            ylab = "Count"

    # Name: ytlower
    # Purpose: Set lower value for y-axis time-series
    #
    # Default: None
    #
    # Note:
    #
    if kwargs.has_key('ytlower'):
        ytlower = kwargs['ytlower']
    else:
        ytlower = None

    # Name: ytupper
    # Purpose: Set upper value for y-axis time-series
    #
    # Default: None
    #
    # Note:
    #
    if kwargs.has_key('ytupper'):
        ytupper = kwargs['ytupper']
    else:
        ytupper = None

    # Name: yuppern
    # Purpose: Set upper value for y-axis normalized histograms
    #
    # Default: None (set by matplotlib
    #
    # Note:
    #
    if kwargs.has_key('yuppern'):
        yuppern = kwargs['yuppern']
    else:
        yuppern = None

    # Name: xupper
    # Purpose: Set upper value for x-axis for histograms
    #
    # Default: None (set by matplotlib
    #
    # Note:
    #
    if kwargs.has_key('xupper'):
        xupper = kwargs['xupper']
    else:
        xupper = None

    # Name: xlower
    # Purpose: Set lower value for x-axis for histograms.
    #
    # Default: 0
    #
    # Note: Only used if xupper used
    #
    if kwargs.has_key('xlower'):
        xlower = kwargs['xlower']
    else:
        xlower = 0

    # Name: trim_max
    # Purpose: Set upper value for y-axis time-series trim
    #
    # Default: None
    #
    # Note:
    #
    if kwargs.has_key('trim_max'):
        trim_max = kwargs['trim_max']
    else:
        trim_max = None

    # Name: trim_min
    # Purpose: Set upper value for y-axis time-series trim
    #
    # Default: None
    #
    # Note:
    #
    if kwargs.has_key('trim_min'):
        trim_min = kwargs['trim_min']
    else:
        trim_min = None

    # Name: stat_2d
    # Purpose: Add stats to the plot (x-axis values)
    #
    # Default: False
    #
    if kwargs.has_key('stat_2d'):
        stat_2d = kwargs['stat_2d']
    else:
        stat_2d = 0

    # Name: no_title
    # Purpose: Do not add a title to the plot
    #
    # Default: True
    #
    if kwargs.has_key('no_title'):
        no_title = kwargs['no_title']
    else:
        no_title = 1

    # Name: no_stats
    # Purpose: Do not add stats to the plot (y-axis values)
    #
    # Default: True
    #
    if kwargs.has_key('no_stats'):
        no_stats = kwargs['no_stats']
    else:
        no_stats = 1

    # Name: polar
    # Purpose: Makeing a polar-rose diagram
    #
    # Default: False
    #
    if kwargs.has_key('polar'):
        polar = 1
    else:
        polar = 0

    # Name: use_grid
    # Purpose: add a grid to the plot
    #
    # Default: True
    #
    if kwargs.has_key('use_grid'):
        use_grid = kwargs['use_grid']
    else:
        use_grid = 1

    # Name: time
    # Purpose: plot timeseries with very wide figure
    #
    # Default: False
    #
    if kwargs.has_key('time'):
        time = kwargs['time']
    else:
        time = 0

    # Name: fig_format
    # Purpose: format of the output plots
    #
    # Default: png
    #
    if kwargs.has_key('fig_format'):
        fig_format = kwargs['fig_format']
    else:
        fig_format = '.png'

    # Test for nothing
    if len(pile) < 1:
       return "Nothing to plot"

    # Make a numpy array
    pile2 = NA(pile)

    # Test for all the same value.
    if abs(pile2.max()-pile2.min()) <= 0.00000000001:
        return "Flat"

    # Create a figure (histogram) using matplotlib
    # --------------------------------------------
    pname = '%s%s' % (pname_base,fig_format)

    # Start figure
    if not polar:
        if time:
            golden_ratio = 1.61803399*2.0
            width = 40.0
        else:
            golden_ratio = 1.61803399
            width = 10.0
        hieght = width/golden_ratio

        fig = plt.figure(figsize=(width,hieght))
        ax = fig.add_subplot(111)

    # Execute Plot
    for each in plot_cmd:
        exec(each)

    # Add Labels and such
    ax.set_xlabel(xlab)
    if not polar:
        ax.set_ylabel(ylab)

    if not no_title:
        # Add title
        ax.set_title(title)

    if not no_stats:
#         fmter = "Count: %d\nMin/Max: %d/%d\nMean/Median/Mode: %.2f/%.2f/%.2f\nSTD: %.2f"
#         fnc_out = Summerize(pile)
#         stitle = fmter % (fnc_out[0],fnc_out[1][0],fnc_out[1][1],fnc_out[2],Median(pile),
#                           Mode(pile)[1][0],fnc_out[3])
        # Note mode can take a very long time for very large arrays
        fmter = "Count: %d\nMin/Max: %.2f/%.2f\nMean/Median: %.2f/%.2f\nSTD: %.2f"
        fnc_out = Summerize(pile)
        stitle = fmter % (fnc_out[0],fnc_out[1][0],fnc_out[1][1],fnc_out[2],Median(pile),fnc_out[3])
        if time:
            plt.suptitle(stitle, fontsize=4,x=0.875,y=0.93,horizontalalignment='left')
        else:
            plt.suptitle(stitle, fontsize=4,x=0.8,y=0.95,horizontalalignment='left')

    if use_grid:
        ax.grid(True)

    # Save to File
    fig.savefig(pname,dpi=144)
    plt.clf()
    plt.close('all')

    return "Made %s" % (pname)

