#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports

# Third party module imports

# ----------------------------------------------------------------------------
def PrintColumns(the_list,indent_tag,fmt,cols,width,sort_me=1):
    """

    Prints a nicely formated list.

    Input:
    ^^^^^
    the _list - List
        List of values to print
    indent_tag - String
        String containing the tabs wanted in the formatted output
    fmt - String
        Format statement for the individual elements of the list
    cols - Integer
        The number of columns to create
    width - Integer
        The interacts with cols and fmt to set spacing of the output.
    sort_me - Boolean
        The default is for a copy ofthe_list to be sorted before printing.

    Output:
    ^^^^^^
    None - String
        Message sent to sys.stdout

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    #>>> PrintColumns(list(range(100,120)),indent_tag="\t",fmt="% 3d",cols=5,width=5) # doctest: +NORMALIZE_WHITESPACE
       100 101 102 103 104 105 106
       107 108 109 110 111 112 113
       114 115 116 117 118 119

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/08 MB - File created.
    """

    line = 0
    if not the_list:
        print(indent_tag+"   None")
        return 1
    if sort_me:
        a_list = sorted(the_list)
    else:
        a_list = the_list
    wide = width*cols
    end = len(a_list)
    i = 0
    for g in a_list:
        i += 1
        tag = ""
        if not line:
            tag = indent_tag
        if i == end:
            print(tag+fmt % (g),end="\n")
        elif line <= wide:
            print(tag+fmt % (g),end="")
            line += width
        else:
            print(tag+fmt % (g),end="\n")
            line = 0
    return 0


