#!/usr/bin/env python -tt
"""
Simple program to read centers.txt files and create a count netcdf.

Example: python util/quick_cnt.py path_to_directory_with_centers.txt_files

"""
from __future__ import division

# Standard library imports
import sys
import os

# Third party module imports
import numpy
import netCDF4 as NetCDF
import scipy.stats
import cPickle
import gzip

# MCMS module imports
from save_netcdf import Save_NetCDF
from file_exists import FileExists
from parse_run_file import ParseRunFile
from injest_setup import InjestSetup
from print_columns import PrintColumns
from query_yes_no import query_yes_no
from rf_files import oroot
from rf_files import droot
from save_netcdf import Save_NetCDF
from acquire_centers import acquire_centers_year

def main(argv):
    class Object(object):
        pass
    tf = Object()

    if len(argv) != 2:
        print "Use python util/quick_cnt.py path_to_directory_with_centers.txt_files"
        return 1
    tf.out_path = argv[1]
    if not tf.out_path.endswith("/"):
        tf.out_path = tf.out_path + "/"

    file_list = os.listdir(tf.out_path)
    file_list.sort()
    pfiles = [x for x in file_list
            if (x.find("centers") != -1 and x.endswith(".txt"))]
    pfiles = [x for x in pfiles if not x.find("progress") != -1]
    pfiles = [x for x in pfiles if not x.find("dumped") != -1]
    pfiles = [x for x in pfiles if not x.find("report") != -1]
    pfiles.sort()
    nfiles = len(pfiles)
    print "\n\tFound %d files in directory %s:\n" % (nfiles,tf.out_path)
    PrintColumns(pfiles,indent_tag="\t",fmt=" %40s",cols=0,width=40)

    # Guess model
    tf.model = pfiles[0].split("_")[1]
    print "\n\tGuess model as '%s'" % tf.model

    # Guess years
    tf.years = [int(x.split("_")[2]) for x in pfiles]
    print "\n\tGuess %d years:\n" % len(tf.years)
    PrintColumns(tf.years,indent_tag="\t",fmt=" %04d",cols=5,width=8)
    tf.start_year = tf.years[0]
    tf.end_year = tf.years[-1]

    # Ask if this seems correct
    answer = query_yes_no("\n\tIs this correct?")
    if not answer: return 1

    # Get some definitions.
    tf.shared_path = "%s%s_files/" % (oroot,tf.model)
    fnc_out,inputs = InjestSetup("%ss_datf.pz" % (tf.shared_path),0)
    tf.maxid = fnc_out[inputs.index("maxid")]
    tf.flons = fnc_out[inputs.index("lons")]
    tf.flats = fnc_out[inputs.index("lats")]
    tf.tail = "_centers.txt"
    tf.just_center_table = 1
    tf.detail_tracks = ""
    tf.include_att = 0
    tf.include_atts = 0
    tf.row_end = fnc_out[inputs.index("row_end")]
    tf.row_start = fnc_out[inputs.index("row_start")]
    tf.tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tf.tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    tf.tropical_end = tf.row_end[tf.tropical_n_alt]
    tf.tropical_start = tf.row_start[tf.tropical_s_alt]
    tf.land_gridids = []
    tf.jd_fake = 0
    del fnc_out,inputs

    stag = "%04d-%04d" % (int(tf.start_year),int(tf.end_year))
    tf.tail = "_centers.txt"
    tf.cut_tail = len(tf.tail)
    fmt = "%s%s_cf_%s_%s%s"
    do_djf_jja = 1
    if do_djf_jja:
        odfile = fmt % (tf.out_path,tf.model,"passed_djf",stag,".nc")
        ojfile = fmt % (tf.out_path,tf.model,"passed_jja",stag,".nc")
        if os.path.exists(odfile):
            os.remove(odfile)
        if os.path.exists(ojfile):
            os.remove(ojfile)
    ofile = fmt % (tf.out_path,tf.model,"passed",stag,".nc")
    if os.path.exists(ofile):
        os.remove(ofile)
    lons = numpy.array(tf.flons)
    lats = numpy.array(tf.flats)
    im = len(lons)
    jm = len(lats)
    maxid = im*jm
    cnts = numpy.zeros(maxid)
    if do_djf_jja:
        dcnts = numpy.zeros(maxid)
        jcnts = numpy.zeros(maxid)
        m2s = { 1 : 1, 2 : 1, 3 : 0, 4 : 0, 5 : 0, 6 : 2, 7 : 2, 8 : 2, 9 : 0,
            10 : 0, 11 : 0, 12 : 1}
    print "\tReading Center Files....\n"
    for f in pfiles:
        loop_year = int(f.split("_")[2])
        readit,centers,ee,cc = acquire_centers_year(tf,loop_year)
        print "\t\tRead %d centers for year %04d" % (len(centers),loop_year)
        ids = readit.ids
        for center in centers:
          cgrid = readit.center_holder[center][ids['GridID']]
          cnts[cgrid] += 1
          if do_djf_jja:
            mm = readit.center_holder[center][ids['MM']]
            if m2s[mm] == 1:
                dcnts[cgrid] += 1
            elif m2s[mm] == 2:
                jcnts[cgrid] += 1
    save_it = Save_NetCDF(cnts,lons,lats,ofile,0,
           name="passed",long_name="Centers",
           units='Counts')
    print "\tSaved %s" % ofile
    if do_djf_jja:
        save_it = Save_NetCDF(dcnts,lons,lats,odfile,0,
               name="passed",long_name="Centers",
               units='Counts')
        print "\tSaved %s" % odfile
        save_it = Save_NetCDF(jcnts,lons,lats,ojfile,0,
               name="passed",long_name="Centers",
               units='Counts')
        print "\tSaved %s" % ojfile

if __name__ == "__main__":
    sys.exit( main(sys.argv) )