#!/usr/bin/env python -tt
'''Read histogram output from check.py and collect into a dictionary'''

def read_histo(rh_what_do):
    """Read data file and put data into proper form"""

    # Standard modules
    import sys
    import os
    import cPickle
    import gzip
    from collections import defaultdict
    import math

    # Third party modules
    import gzip

    # MCMS modules
    import make_circle

    find_eq_radius = make_circle.find_eq_radius
    rh_max_grid_count = rh_what_do["max_grid_count"]
    rh_bkeys = rh_what_do["bkeys"]
    rh_stat_defs = rh_what_do["stat_defs"]
    rh_min_trk_cnt = rh_what_do["min_trk_cnt"]
    rh_keep_slp = rh_what_do["keep_slp"]
    rh_min_travel = rh_what_do["min_travel"]
    rh_cyclolysis = rh_what_do["cyclolysis"]
    rh_cyclogenesis = rh_what_do["cyclogenesis"]
    rh_seasons = rh_what_do["seasons"]
    rh_hemis = rh_what_do["hemis"]
    rh_intensities = rh_what_do["intensities"]
    rh_super_run = rh_what_do["super_run"]
    rh_loop_year = rh_what_do["loop_year"]
    rh_summary_plots_only = rh_what_do["summary_plots_only"]
    rh_sg1_fmt = rh_what_do["sg1_fmt"]
    rh_verbose = rh_what_do["verbose"]
    rh_use_usi_intensity = rh_what_do["use_usi_intensity"]
    rh_min_loc = rh_what_do["min_loc"]
    rh_use_slp_min = rh_what_do["use_slp_min"]
    rh_m2s = rh_what_do["m2s"]
    rh_mlats = rh_what_do["use_mlats"]
    rh_only_ocean = rh_what_do["only_ocean"]
    rh_continuous = rh_what_do["continuous"]
    rh_uci_screen = rh_what_do["uci_screen"]
    rh_what_does = rh_what_do["what_does"]
    rh_nlats = rh_what_do["use_nlats"]
    rh_only_land =  rh_what_do["only_land"]
    rh_basin = rh_what_do["basin"]

    if rh_verbose:
        print "\t\t\tReading data files ..."
        print "\t\t\t\tReading: %s" % rh_what_do["in_file"]
    ## Start Special Bill Rossow Modification (comment out)
    #import numpy,datetime,calendar
    #leap_year = calendar.isleap(rh_loop_year)
    ## data_index [day,hemi,intensity,center,att,total]
    #di = [365,len(rh_hemis ),len(rh_intensities),3]
    ## Basic idea
    ##   For each day I need the average coverage. This is found by
    ##   taking the area for each cyclone and accumulating it in the correct
    ##   dayofyear bin. I need to now how many *timesteps* not centers that
    ##   went into the dayofyear bin so I can get the day average.
    #mean_sums = numpy.zeros(di,dtype=numpy.float)
    ##mean_cnts = numpy.zeros(di,dtype=numpy.float)
    #data_index = [365,len(rh_hemis ),len(rh_intensities),3,4]
    #tsteps_used = numpy.zeros(data_index,dtype=numpy.float)
    ## End Special Bill Rossow Modification (comment out)

    # If using diff_engine screen input now
    uci_screen = []
    if rh_uci_screen:
        no_mod = ["usi","spane","span","spanb","ampb","ampe","asymmetry",
                "amplitude","amp-asymmetry","track-length","track-range"]
        try:
            uci_screen = cPickle.load(gzip.open(rh_uci_screen))
        except IOError:
            print "Error Reading: %s" % (rh_uci_screen)
        # Because tracks can extend into or out of boarding years read in
        #   these too if they are available.
        #print rh_uci_screen
        #print len(uci_screen)
        if rh_loop_year != rh_super_run[0]:
            before_year = rh_loop_year - 1
            tuci = rh_uci_screen.replace("%s.p" % str(rh_loop_year),"%s.p" % str(before_year))
            #print tuci
            if os.path.exists(tuci):
                try:
                    t_screen = cPickle.load(gzip.open(tuci))
                except IOError:
                    print "Error Reading: %s" % (tuci)
                uci_screen.extend(t_screen)
                #print "added",tuci
                #print len(uci_screen)
        if rh_loop_year != rh_super_run[-1]:
            after_year = rh_loop_year + 1
            tuci = rh_uci_screen.replace("%s.p" % str(rh_loop_year),"%s.p" % str(after_year))
            #print tuci
            if os.path.exists(tuci):
                try:
                    t_screen = cPickle.load(gzip.open(tuci))
                except IOError:
                    print "Error Reading: %s" % (tuci)
                uci_screen.extend(t_screen)
                #print "added",tuci
                #print len(uci_screen)
          
        # Put in track data base because mcms_check files stored
        #   by usi not uci
        import util.read_mcms
        Read_MCMS = util.read_mcms.Read_MCMS
        readit = Read_MCMS(**rh_what_does)
        readit.check_time()
        readit.check_place()
        # Read center file and return sorted tracks dbase.
        readit.fetch_centers()
        #print len(readit.sorted_tracks)

    # Create data structure.
    tmpTracks = {}
    pastDataHisto = defaultdict(list)

    # Load whole file into memory
    try:
        lines = gzip.open(rh_what_do["in_file"],"rb").readlines()
    except IOError:
        print "Error reading: %s" % rh_what_do["in_file"]
    if rh_verbose: print "\t\t\t\tRead %d centers" % len(lines)
    
    #print "---"
    #print "rh_uci_screen",rh_uci_screen
    #print "len(uci_screen)",len(uci_screen)
    #print "uci_screen[0]",uci_screen[0]
    #print 'rh_what_do["in_file"]',rh_what_do["in_file"]
    #print "len(lines)",len(lines)
    #print 'rh_what_does["in_file"]',rh_what_does["in_file"]
    #print "len(readit.sorted_tracks)",len(readit.sorted_tracks)
    #print "readit.sorted_tracks.keys()[100]",readit.sorted_tracks.keys()[100]
    #print "++++"

    # Process each line into a track database keyed by usi
    #
    # Example (short version)
    # tmpTracks['19790824120250012249'] = {
    #     'gridids': [8977, 9120, 9118, 9118, 9118, 9261, 9260, 9261, 9260, 9121],
    #     'spane': '8',
    #     'bearing': [-1, 348.4, 313.5, 312.6, 315.6, 316.5, 309.2, 315.5, 306.9, -1],
    #     }
    skip_day = 0
    for line in lines:
        shattered = line.split()
        key = shattered[0].rstrip(":")
        
        ##testing start
        ## For Testing only keep cyclones that start first 14th or 15th of each
        ## Month. This keeps out issues at beginning and end of record.
        #if key == "usi":
        #    skip_day = 0
        #    if int(str(shattered[1][6:8])) not in [14,15]: skip_day = 1
        #if skip_day: continue        
        ##testing end

        if key == "usi":
            # Initialize new track
            tmpTracks[shattered[1]] = {}
            metakey = shattered[1]    
        elif len(shattered) > 2:
            # Add list quantities
            if key in rh_stat_defs["as_ints"]:
                if key.find("area") != -1:
                    #  Make area a radius of an equivalent circle on Sphere.
                    tmparea = [(find_eq_radius(float(x)),int(x))[x == '-1'] for x in shattered[1:]]
                    ## Start Special Bill Rossow Modification (comment out)
                    #tmparea = [int(x) for x in shattered[1:]]
                    ## End Special Bill Rossow Modification (comment out)
                    tmpTracks[metakey][key] = [(x,-1)[x == 0] for x in tmparea]
                ## Start Special Bill Rossow Modification (comment out)
                #elif key.find("dates") != -1:
                #    tmpTracks[metakey][key] = [int(x) for x in shattered[1:]]
                #    # Convert dates into days of the year 1..365
                #    tmpTracks[metakey]['dayofyear'] = [datetime.date(int(str(x[:4])),int(str(x[4:6])),int(str(x[6:8]))).timetuple().tm_yday for x in shattered[1:]]
                ## End Special Bill Rossow Modification (comment out)
                elif key.find("natt") != -1:
                    # Make natts a % of total grids in a hemisphere.
                    tmpTracks[metakey][key] = [((float(x)/rh_max_grid_count)*100.0,int(x))[x == '-1'] for x in shattered[1:]]
                else:
                    tmpTracks[metakey][key] = [int(x) for x in shattered[1:]]
            else:
                tmpTracks[metakey][key] = [float(x) for x in shattered[1:]]
        else:
            # Add single value quantities
            if key in rh_stat_defs["as_ints"]:
                if key.find("area") != -1:
                    #  Make area a radius of an equivalent circle on Sphere.
                    tmpTracks[metakey][key] = (find_eq_radius(float(shattered[1])),int(shattered[1]))[shattered[1] == '-1']
                #    # Start Special Bill Rossow Modification (comment out)
                #    tmpTracks[metakey][key] = int(shattered[1])
                #    # End Special Bill Rossow Modification (comment out)
                ## Start Special Bill Rossow Modification (comment out)
                #elif key.find("dates") != -1:
                #    tmpTracks[metakey][key] = int(shattered[1])
                #    # Convert dates into days of the year 1..365
                #    tmpTracks[metakey]['dayofyear'] = datetime.date(int(str(shattered[1][:4])),
                #            int(str(shattered[1][4:6])),int(str(shattered[1][6:8]))).timetuple().tm_yday
                #    # End Special Bill Rossow Modification (comment out)
                elif key.find("natt") != -1:
                    # Make natts a % of total grids in a hemisphere.
                    tmpTracks[metakey][key] = ((float(shattered[1])/rh_max_grid_count)*100.0,int(shattered[1]))[shattered[1] == '-1']
                else:
                    tmpTracks[metakey][key] = int(shattered[1])
            else:
                tmpTracks[metakey][key] = float(shattered[1])

    # unPickle datetime objects:
    try:
        (dtimes,start_check,end_check) = cPickle.load(gzip.open(rh_what_do["dtime_file"]))
    except IOError:
        print "Error Reading: %s" % (rh_what_do["dtime_file"])

    # Compile data for histograms... need to separate by season,hemi,inten,year
    usis = tmpTracks.keys()
    usis = sorted(usis)
#tmp
    #usis = usis[:200]
    #usis = [usis[0]]
    for usi in usis:
        if uci_screen:
            # Screen for membership in uci_screen in this case remove all
            #   entries
            # Warning: Some values like amplitude will not be correct unless
            #   EVERY center for this track passes uci_screen. 
            # Mask Track
            if usi not in readit.sorted_tracks:
                # Track not found!
                #print usi,"Missing"
                continue
            if len(readit.sorted_tracks[usi]) != int(tmpTracks[usi]['span']):
                # Likely a case where some of the uci in this track come from
                #   over/under year track extension that is not caught... just
                #   skip to be sure
                #print "Mismatch"
                continue
            track_mask = [(0,1)[x in uci_screen] for x in readit.sorted_tracks[usi]]
            nt = sum(track_mask)
            #print "\n===="
            #print usi
            #print readit.sorted_tracks[usi]
            #print int(tmpTracks[usi]['span']),nt
            #print track_mask
            #print tmpTracks[usi]['aged-indices']
            if sum(track_mask) == 0:
                # drop the whole track
                #print "No Good"
                continue
            elif sum(track_mask) != int(tmpTracks[usi]['span']):
                # Modify
                #print "Modify"
                tn = int(tmpTracks[usi]['span'])
                tmpTracks[usi]['span'] = nt
                for var in rh_bkeys:
                    if var not in no_mod:
                        #print var,tmpTracks[usi][var]
                        tmpTracks[usi][var] = [tmpTracks[usi][var][x] for x in xrange(tn) if track_mask[x]]
                        #print var,tmpTracks[usi][var]
            #else:
            #    print "All Good"

        # Because data stored in arrays it's possible for some data to
        #   belong to one season and some another etc.
        tlen = int(tmpTracks[usi]['span'])
        # Apply some filters to remove tracks that because they are
        #   near the beginning or end of the records may be partial
        #   and thus did not have the track_finder filters applied
        #   and are not known to be representative of full cyclones
        #   as defined by this method
        if tlen < rh_min_trk_cnt:
            # Track less than minimum retained value
            continue
        if min([float(x) for x in tmpTracks[usi]['slp']]) > rh_keep_slp:
            # Track lifetime SLP too high
            continue
        if float(tmpTracks[usi]['track-range']) < rh_min_travel:
            # Track didn't travel enough.
            continue
        if rh_use_usi_intensity:
            peak_inten_index = tmpTracks[usi]['aged-indices'].index(rh_min_loc)
            if rh_continuous:
                if tmpTracks[usi]['intensity'][peak_inten_index] < 33:
                    peak_inten = 3 # Strong
                elif tmpTracks[usi]['intensity'][peak_inten_index] > 66:
                    peak_inten= 1 # Weak
                else:
                    peak_inten = 2 # Moderate
            else:
                peak_inten = tmpTracks[usi]['intensity'][peak_inten_index]
        if rh_use_slp_min:
            trakloop = [tmpTracks[usi]['aged-indices'].index(rh_min_loc)]
        else:
            trakloop = xrange(tlen)

        for trak in trakloop:
            if rh_cyclolysis and trak != tlen-1: continue
            if rh_seasons == ['annual']:
                this_season = [0]
            else:
                this_season = [0,rh_m2s[int(str(tmpTracks[usi]['dates'][trak])[4:6])]]
            if rh_hemis == ['global']:
                this_hemi = [0]
            else:
                this_hemi = [0,(1 if tmpTracks[usi]['lats'][trak] > 0 else 2)]
            if rh_intensities == ["all"]:
                this_inten = [0]
            else:
                if rh_use_usi_intensity:
                    this_inten = [0,peak_inten]
                elif rh_continuous:
                    if tmpTracks[usi]['intensity'][trak] < 33:
                        this_inten =[0,3] # Strong
                    elif tmpTracks[usi]['intensity'][trak] > 66:
                        this_inten= [0,1] # Weak
                    else:
                        this_inten = [0,2] # Moderate
                else:
                    this_inten = [0,tmpTracks[usi]['intensity'][trak]]
            if rh_summary_plots_only:
                this_year = [rh_super_run]
            else:
                this_year = [rh_super_run,rh_loop_year]
            nyears = len(this_year)
            #print "this_season",this_season,"this_hemi",this_hemi,"this_inten",this_inten
            if rh_mlats:
                # Limit to centers between 40-70 latitude
                if math.fabs(tmpTracks[usi]['lats'][trak]) < 40:
                    continue
                if math.fabs(tmpTracks[usi]['lats'][trak]) > 70:
                    continue
            if rh_nlats:
                # Only to centers outside 40-70 latitude
                if math.fabs(tmpTracks[usi]['lats'][trak]) <= 70:
                    if math.fabs(tmpTracks[usi]['lats'][trak]) >= 40:
                       if rh_only_land:
                            # Limit to centers over land even if not in nlat zone
                            #   That is, this is non-mid all and mid-lat land
                            if tmpTracks[usi]['gridids'][trak] not in rh_only_land:
                                continue
                       elif rh_only_ocean:
                            # That is, this is non-mid all and mid-lat land
                            if tmpTracks[usi]['gridids'][trak] in rh_only_ocean:
                                continue
                       else:
                            # Standard nlat screen
                            continue
            if rh_only_ocean and not rh_nlats:
                # Limit to centers over ocean
                if tmpTracks[usi]['gridids'][trak] in rh_only_ocean:
                    continue
            if rh_only_land and not rh_nlats:
                # Limit to centers over land
                if tmpTracks[usi]['gridids'][trak] not in rh_only_land:
                    continue            
            if rh_basin:
                # Limit to a basin
                if tmpTracks[usi]['gridids'][trak] not in rh_basin:
                    continue
            #Add to proper bin ...
            bname = [(ss,hh,ii,yy) for ss in this_season
                for hh in this_hemi for ii in this_inten
                for yy in xrange(nyears)]
            bname = [rh_sg1_fmt % ("MARK",rh_seasons[b[0]],rh_hemis[b[1]],
                rh_intensities[b[2]],this_year[b[3]]) for b in bname]

            for var in rh_bkeys:
                #print var,tmpTracks[usi][var]
                if var in ["track-length","track-range",'asymmetry','amplitude','amp-asymmetry','ampb','ampe']:
                     # Single value not arrays.
                     if trak == 0:
                         for b in bname:
                             the_key = b.replace("MARK",var)
                             pastDataHisto[the_key].append(tmpTracks[usi][var])
                else:
                     for b in bname:
                         the_key = b.replace("MARK",var)
                         if not isinstance(tmpTracks[usi][var],list):
                             # Happens above with len(shattered) > 2
                             tmpTracks[usi][var] = [tmpTracks[usi][var]]
                         
                         ## Start Special Bill Rossow Modification (comment out)
                         #do_this = 0
                         #if var.find("att-area") != -1:
                         #    do_this = 2
                         #elif var.find("area-total") != -1:
                         #    do_this = 3
                         #elif var.find("gridids") != -1:
                         #    do_this = 1
                         ## Only to annual as seasons dealt with later
                         #if b.find("annual") == -1: 
                         #    do_this = 0
                         #if do_this:
                         #   #print "the_key",the_key
                         #   # Find day of year and accumulate area
                         #   if tmpTracks[usi][var][trak] != -1:
                         #       dayofyear = tmpTracks[usi]["dayofyear"][trak]
                         #       if the_key.find("_nh_") != -1:
                         #           hhh = 1
                         #       elif the_key.find("_sh_") != -1:
                         #           hhh = 2
                         #       else:
                         #           hhh = 0
                         #       if the_key.find("_str_") != -1:
                         #           sss = 3
                         #       elif the_key.find("_mod_") != -1:
                         #           sss = 2
                         #       elif the_key.find("_wea_") != -1:
                         #           sss = 1
                         #       else:
                         #           sss = 0
                         #       # [day,hemi,intensity,(center,att,total)]
                         #       if do_this == 1:
                         #           # Find Area of gridID, approximate
                         #           # 6371**2 * math.sin(math.radians(2.5))**2
                         #           zzz = 77228.0
                         #       else:
                         #           zzz = tmpTracks[usi][var][trak]
                         #       ##zzz = 510072000.0*0.25
                         #       # Deal with leap years... just drop Feb 29
                         #       #   and adjust
                         #       if leap_year:
                         #           if dayofyear == 59: 
                         #               continue
                         #           elif dayofyear > 59: 
                         #               dayofyear = dayofyear -1
                         #       #print "\t\t\tdo_this,dayofyear,hhh,sss",do_this,dayofyear,hhh,sss
                         #       mean_sums[dayofyear-1,hhh,sss,do_this-1] += zzz
                         #       # Count the number of tsteps for this day for this hemi,inten
                         #       #tstep = int(str(tmpTracks[usi]["dates"][trak])[8:10])              
                         #       #if tstep == 0:
                         #       #    tsteps_used[dayofyear-1,hhh,sss,do_this-1,0] += 1
                         #       #elif tstep == 6:
                         #       #    tsteps_used[dayofyear-1,hhh,sss,do_this-1,1] += 1
                         #       #elif tstep == 12:
                         #       #    tsteps_used[dayofyear-1,hhh,sss,do_this-1,2] += 1
                         #       #elif tstep == 18:
                         #       #    tsteps_used[dayofyear-1,hhh,sss,do_this-1,3] += 1
                         #       #print "\t\t\t\ttsteps_used[dayofyear-1,hhh,sss,do_this-1,:]",tsteps_used[dayofyear-1,hhh,sss,do_this-1,:]
                         ## End Special Bill Rossow Modification (comment out)
                             
                         pastDataHisto[the_key].append(tmpTracks[usi][var][trak])
                         #print pastDataHisto[the_key]
            if rh_cyclogenesis: break

    # Free Memory
    del lines
    del tmpTracks

    ## Start Special Bill Rossow Modification (comment out)
    ## Change what is returned, this is the daily mean accumulated area for this 
    ##   year
    #sstat_label = ["Center Area","ATT Area   ","Total Area "]
    #for dayofyear in xrange(0,365):
    #    for hemi in xrange(3):
    #        for inten in xrange(4):
    #            #print "\t",dayofyear,rh_hemis[hemi],rh_intensities[inten]
    #            for sstat in xrange(3):
    #                if mean_sums[dayofyear,hemi,inten,sstat] > 0:
    #                    #nsteps = float(numpy.sum(tsteps_used[dayofyear,hemi,inten,sstat,:]))
    #                    nsteps = 4.0
    #                    mean_sums[dayofyear,hemi,inten,sstat] = mean_sums[dayofyear,hemi,inten,sstat]/nsteps
    #                    #print dayofyear,hemi,inten,sstat,nsteps,mean_sums[dayofyear,hemi,inten,sstat]
    #                    #print "\t\t%s %20.2g km**2" % (sstat_label[sstat], mean_sums[dayofyear,hemi,inten,sstat])
    #pastDataHisto = mean_sums
    ## End Special Bill Rossow Modification (comment out)
    # Return data structure
    #return (pastDataHisto,dtimes)
 
    return pastDataHisto
