from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
#import math

# Third party module imports
import numpy

# MCMS module imports
from extras import ProgressBarAlt
from gcd import gcd
from rhumb_line_nav import rhumb_line_nav
from resort import resort
from util import write_mcms
from jd_key import jd_key

def refactor_tracks(tf,defs,dead_tracks,hits,troubled_centers):
    # Refactor Tracks:
    # 1) Update each center's usi, i.e., assign to a track
    # 2) Apply post-tracking filters
    #    a) Tropical Filter
    #    b) Minimum Lifetime
    #    c) Minimum Lifetime SLP
    #    d) Minimum Lifetime Travel

    save_stats = tf.save_stats
    save_plots = tf.save_plots
    might_be_long = tf.might_be_long
    date_stamps = tf.uci_stamps
    min_trk_cnt = tf.min_trk_cnt
    jm = tf.jm
    im = tf.im
    tracks_file = tf.tracks_file
    dumped_centers_file = tf.dumped_centers_file
    center_fmt = write_mcms.center_fmt
    m2s = tf.m2s

#FIX work in the write_mcms.form_uci
    super_total_tracks = 0
    super_total_centers_cnt = [0]*tf.nflags
    if tf.save_plots:
        #flag_cnt = numpy.zeros((jm*im,tf.nflags),dtype=numpy.float)
        flag_cnt = numpy.zeros((jm*im,tf.nseasons,tf.nflags),dtype=numpy.float)
        # to see touch concerns
        #touch_concerns = numpy.zeros((tf.nseasons,jm*im),dtype=numpy.float)
        touch_concerns = numpy.zeros((jm*im,tf.nseasons),dtype=numpy.float)
    else:
        flag_cnt = touch_concerns = []

    if save_stats != 2:
        # Open files for storage
        tracks_save = open(tracks_file,"w")
        if defs.keep_discards:
            dumped_centers_save = open(dumped_centers_file,"w")
        else:
            dumped_file = ""

    # dead_tracks: dictionary of linked centers (key USI)
    # hits: dictionary of original found centers (key UCI)
    used_usi = dead_tracks.keys()
    used_usi.sort()

    # #mike
    # apples = (1 if '19790101060131203687' in used_usi else 0)
    # if apples:
    #     print "\napples", '19790101060131203687'

    # Correct for overly long systems, which almost always are due to excessively
    #   slow or stationary systems.
    for usi in used_usi:
        # Centers uci found with this usi
        caught = [hits[y[0]] for y in dead_tracks[usi]]

        # #mike
        # pear = (1 if usi == '19790101060131203687' else 0)
        # if pear:
        #    print "\n\tpear caught",caught
        #    print "\n\tpear dead_tracks[usi]",dead_tracks[usi]

        #   The problem here is because we are evaluating tracks on a center-to-center
        #   basis we are blind to gradual changes that result is large changes. Thus I
        #   find all the monstrously long cyclones are also slow moving so that they
        #   last a long time and even sometimes slowly make large course changes. Also
        #   find cases where SLP rises and dips multiple times but never such as to break
        #   they connection. Again some sort of history of course, speed, SLP change
        #   would fix this.

        # Maybe look for large jumps (accelerations) in either speed or bearing. Also
        #   large changes in SLP (although issue with tracks moving in background field).
        #   Maybe kill tracks that spend more than 24 hours in a single grid?

        # Fix using vector of trajectory
        #   Idea is to find the acceleration between centers and limit those
        #     tangential acceleration - a_t is due to the change in speed of traversal,
        #           and points along the curve in the direction of the velocity vector.
        #           That is, the cyclone is speeding up/down between centers along the
        #           direction of the starting vector.
        #     radial acceleration - a_r is due to the change in direction of the velocity
        #           vector and is normal to the trajectory, pointing toward the center
        #           of curvature of the path (centripetal component).
        #           That is, the cyclone is changing direction.
        #
        #   Maybe have a mean vector and build in a resistance to depart from it.

        #if len(caught) > might_be_long
        #    # Look for cases of long tracks where large jump between tracks is
        #    #   found.
        #
        #    continue

        # Get dissimilarity scores
        ds = [x[1] for x in dead_tracks[usi]]
        ds_high = [x for x in ds if x > 1.0 and x != ""]

        # #mike
        # if pear:
        #    print "\n\tpear ds, ds_high",ds,ds_high

        #if ds_high:
        if len(caught) > might_be_long and ds_high:
            # Very long system. If there is a large dissimilarity score
            #   break the system at each high dscore and reform into
            #   new tracks....
            # #mike
            # if pear:
            #     print "\nbreak usi,len(caught)",usi,len(caught)
            #     print "\tbreak ds_high",ds_high

            # Loop over dead_tracks[usi] until reach a high dscore
            new_tracks = {}
            new_tracks[usi] = [(usi,"")]
            using_usi = usi
            break_track = 0
            for trak in dead_tracks[usi]:
                if trak[1] == "": continue
                #print "\t\tTesting trak\tbreak_track,using_usi",trak,break_track,using_usi
                if break_track:
                    # Create a new track
                    break_track = 0
                    using_usi = trak[0]
                    new_tracks[using_usi] = [(using_usi,"")]
                    #print "\t\t\tNew Track %s" % using_usi
                elif trak[1] > 1.0:
                    # Break track, skip this center as the dissimilarity score
                    #   is very high
                    #print "\t\t\tbreak"
                    break_track = 1
                    using_usi = ""
                else:
                    # Extend track
                    #print "\t\t\textend %s" % using_usi
                    oldt = new_tracks[using_usi]
                    oldt.append(trak)
                    new_tracks[using_usi] = oldt

            # Delete original dead_track and update
            del dead_tracks[usi]
            dead_tracks.update(new_tracks)

            #print "\n\tdead_tracks[usi]",dead_tracks[usi]
            #print "\n\tnew_tracks"
            #for trak in new_tracks:
            #    print "\ttrak",trak,new_tracks[trak]
            #import sys; sys.exit("Stop Here")

    # Update used_usi
    used_usi = dead_tracks.keys()
    used_usi.sort()

    #for usi in used_usi:
    #    print "usi",usi
    #    print "dead_tracks[usi]",len(dead_tracks[usi])
    #import sys; sys.exit("\nStop Here")


    # #mike
    # grape = (1 if '19790101060131203687' in used_usi else 0)
    # if grape:
    #     print "\n\ngrape\n"

    # Loop over tracks and find member centers, choosing by dissimilarity
    #   scores if multiple
    pbara = ProgressBarAlt(used_usi,"\tRefactoring Tracks.... ",60)
    #pbara = used_usi
    for usi in pbara:
        #if usi in done: continue
        # #mike
        # print "\ngrape usi",usi
        # print "\n\tgrape dead_tracks[usi]",dead_tracks[usi]

        # Centers uci found with this usi
        caught = [hits[y[0]] for y in dead_tracks[usi]]

        # Get dissimilarity scores
        ds = [x[1] for x in dead_tracks[usi]]

        # #mike
        # print "\n\tgrape caught",caught
        # pork = (1 if usi == '19790101060131203687' else 0)
        # if pork:
        #     print "\n\t\tpork caught, ds\n",caught,ds
        # pork2 = (1 if [x[14] for x in caught if x[14] == '19790101060131203687']  else 0)
        # if pork2:
        #     print "\n\t\tpork2 caught, ds\n",caught,ds

        # Trim for speed as search pool shrinks... that is this whole usi
        #   is going to either be saved as a track or parts or the whole
        #   discarded.
        for each in caught:
            # #mike
            # bird = (1 if each[14] == '19790101060131203687' else 0)
            # if bird:
            #     print "\t\t\tbird",each
            #done.append(each[14])
            # Trim the list of uci
            del hits[each[14]]
        # Trim the list of usi
        del dead_tracks[usi]

        # Apply Tropical Test:
        #   Whole track discarded if track never leaves the tropics.
        extra_tropical_system = [x[5] for x in caught if
                           abs(90-x[5]*0.01) >= defs.tropical_boundary_alt]
        if not extra_tropical_system:
            if defs.keep_discards:
                for part in caught:
                    msg = center_fmt % (part[0],part[1],part[2],part[3],
                                              part[4],part[5],part[6],part[7],
                                              part[8],part[9],part[10],10,
                                              part[12],part[13],part[14],
                                              part[15])
                    if save_stats != 2:
                        dumped_centers_save.writelines(msg)
                    super_total_centers_cnt[10] += 1
                    if save_plots:
                        flag_cnt[part[7],m2s[part[1]],10] += 1
                        flag_cnt[part[7],0,10] += 1
            continue

        # Don't apply remaining filters is track starts in the very
        # 1st timestep as it could have been truncated and so
        # might incorrectly fail these tests. Same idea if last member
        # of the track in the last timestep.
        do_filter = 1
        if usi.startswith(date_stamps[0]):
            do_filter = 0
        if caught[-1][14].startswith(date_stamps[-1]):
            do_filter = 0

        #if save_plots and defs.troubled_filter:
        #    # Use here to see touch troubled grids among all centers. See below
        #    in_trouble = [x for x in caught if x[7] in troubled_centers]
        #    if in_trouble:
        #        for part in caught:
        #            # Record whole track if any center touches
        #            touch_concerns[part[7],0] += 1
        #            touch_concerns[part[7],m2s[part[1]]] += 1

        if do_filter:

            # Apply minimum lifetime
            if len(caught) < min_trk_cnt:

                if defs.keep_discards:
                    for part in caught:
                        msg = center_fmt % (part[0],part[1],part[2],part[3],
                                                  part[4],part[5],part[6],part[7],
                                                  part[8],part[9],part[10],6,
                                                  part[12],part[13],part[14],
                                                  part[15])
                        if save_stats != 2:
                            dumped_centers_save.writelines(msg)
                        super_total_centers_cnt[6] += 1
                        if save_plots:
                            flag_cnt[part[7],m2s[part[1]],6] += 1
                            flag_cnt[part[7],0,6] += 1
                continue
            # Apply minimum lifetime SLP
            min_val = min(x[8] for x in caught)

            if min_val > defs.keep_slp:
                if defs.keep_discards:
                    for part in caught:
                        msg = center_fmt % (part[0],part[1],part[2],part[3],
                                                  part[4],part[5],part[6],part[7],
                                                  part[8],part[9],part[10],8,
                                                  part[12],part[13],part[14],
                                                  part[15])
                        if save_stats != 2:
                            dumped_centers_save.writelines(msg)
                        super_total_centers_cnt[8] += 1
                        if save_plots:
                            flag_cnt[part[7],m2s[part[1]],8] += 1
                            flag_cnt[part[7],0,8] += 1
                continue

            # Apply Minimum Lifetime Travel:
            # Methods:
            #  1) total distance traveled between segments
            #  2) distance between start and end centers
            #  3) maximum displacement from start center
            #
            # Note option to discard tracks that never
            # leave high topography regions... likely noise
            #
            first = True
            total_travel = 0.0

            ## Method 1
            #for segment in caught:
            #    if first:
            #        first = False
            #        plon = segment[6]*0.01
            #        plat = 90.0 - segment[5]*0.01
            #        continue
            #    elon = segment[6]*0.01
            #    elat = 90.0 - segment[5]*0.01

            #    if defs.use_gcd:
            #        disty = gcd(plon,plat,elon,elat)
            #    else:
            #        fnc = rhumb_line_nav(elon,elat,plon,plat,True)
            #        disty = fnc[1]

            #    total_travel += disty
            #    plon = elon
            #    plat = elat

            # Method 3
            total_travel = 0.0
            for segment in caught:
                if first:
                    first = False
                    plon = segment[6]*0.01
                    plat = 90.0 - segment[5]*0.01
                    continue
                elon = segment[6]*0.01
                elat = 90.0 - segment[5]*0.01

                if defs.use_gcd:
                    disty = gcd(plon,plat,elon,elat)
                else:
                    fnc = rhumb_line_nav(elon,elat,plon,plat,True)
                    disty = fnc[1]

                if disty > total_travel:
                    total_travel = disty

            # Discard tracks that never leave troubled area
            if defs.troubled_filter:
                in_trouble = [x for x in caught if x[7] in troubled_centers]
                if len(in_trouble) == len(caught):
                    total_travel = 0.0

            if total_travel < defs.min_trk_travel:
                if defs.keep_discards:
                    for part in caught:
                        msg = center_fmt % (part[0],part[1],part[2],part[3],
                                                  part[4],part[5],part[6],part[7],
                                                  part[8],part[9],part[10],7,
                                                  part[12],part[13],part[14],
                                                  part[15])
                        if save_stats != 2:
                            dumped_centers_save.writelines(msg)
                        super_total_centers_cnt[7] += 1
                        if save_plots:
                            flag_cnt[part[7],m2s[part[1]],7] += 1
                            flag_cnt[part[7],0,7] += 1
                continue

        # Extract dissimilarity scores for track and alter usi
        ii = 0
        for part in caught:
            if ii == 0:
                dissimilarity = 0
            else:
                dissimilarity = int(ds[ii]*100.0)
            ii += 1
            msg = center_fmt % (part[0],part[1],part[2],part[3],part[4],
                                      part[5],part[6],part[7],part[8],part[9],
                                      part[10],part[11],part[12],dissimilarity
                                      ,part[14],usi)
            if save_stats != 2:
                tracks_save.writelines(msg)
            #Move and change
            super_total_centers_cnt[0] += 1
            if save_plots:
                flag_cnt[part[7],m2s[part[1]],0] += 1
                flag_cnt[part[7],0,0] += 1
        super_total_tracks += 1

        if save_plots and defs.troubled_filter:
            # Use here to see touch troubled grids among saved centers.
            in_trouble = [x for x in caught if x[7] in troubled_centers]
            if in_trouble:
                for part in caught:
                    # Record whole track if any center touches
                    touch_concerns[part[7],0] += 1
                    touch_concerns[part[7],m2s[part[1]]] += 1

    if save_stats != 2:
        tracks_save.close()

    if defs.keep_discards:
        # Dump untrackable centers
        for part in hits.values():
            msg = center_fmt % (part[0],part[1],part[2],part[3],part[4],
                                      part[5],part[6],part[7],part[8],part[9],
                                      part[10],5,part[12],part[13],part[14],
                                      part[15])
            if save_stats != 2:
                dumped_centers_save.writelines(msg)
            super_total_centers_cnt[5] += 1
            if save_plots:
                flag_cnt[part[7],m2s[part[1]],5] += 1
                flag_cnt[part[7],0,5] += 1
        if save_stats != 2:
            dumped_centers_save.close()

    #if save_plots and defs.troubled_filter:
    #    # Normalize by number of kept centers?
    #    touch_concerns = numpy.where(super_total_centers_cnt > 0,
    #    super_total_centers_cnt[0]

    #-------------------------------------------------------------------------
    # Clean up
    #-------------------------------------------------------------------------
    if defs.keep_discards:
        if save_stats != 2:
            dumped_centers_save.close()

    # Final check to ensure order
    if save_stats != 2:
        print "\tResorting Tracks..."
        resort(tracks_file,jd_key,center_fmt)
        if defs.keep_discards:
            resort(dumped_centers_file,jd_key,center_fmt)

    return (super_total_tracks,super_total_centers_cnt,touch_concerns,flag_cnt)
