import os
import sys
import commands

from process_data import process_data

def rose_plot(alldat,plot_name,minbin,maxbin,incbin,fig_format,title="Bearing"):

    """
    Use GMT to plot a histogram.
    """
    #if verbose :
    #    print "Number of bins : %d" % (len(alldat))
    #    print "Minimum bin    : %f" % (minbin)
    #    print "Maximum bin    : %f" % (maxbin)
    #    print "Bin interval   : %f" % (incbin)

    # Determine bin boundaries and dump to a txt file for GMT. Also, info for
    # fractional amount.
    peak_frac,min_x,max_x = process_data(alldat,"tmp.txt",minbin,incbin,verbose=0,raw=1)
    #if verbose:
    #    print "Peak Scaling Fraction %f" % (peak_frac)

    # Deal with various graphic output options
    defaults = "gmtset HEADER_FONT_SIZE 14p LABEL_FONT_SIZE 12p \
    ANOT_FONT_SIZE 10p PAPER_MEDIA letter DOTS_PR_INCH 600 \
    PAGE_ORIENTATION PORTRAIT"
    temp_ps      = "temp.ps"
    plot_name = plot_name + fig_format
    if fig_format == '.png':
        make_fig = "convert -trim -quality 90 -density 144 144 %s %s; rm -f %s" % (temp_ps,plot_name,temp_ps)
    elif fig_format == '.pdf':
        make_fig = "ps2eps --force --quiet --removepreview --rotate=+ --loose \
                temp.ps; ps2pdf13 -dEPSCrop -r144 temp.eps %s; rm temp.eps" % (plot_name)
        #make_pdf = "ps2pdf13 -dEPSCrop -r144 %s %s; rm -f %s" % (temp_ps,plot_name,temp_ps)
    elif fig_format == '.eps':
        make_fig = "ps2eps --force --quiet --removepreview \
        --rotate=+ --loose temp.ps ; mv temp.eps %s" % (plot_name)
    #if verbose: print "Making ",plot_name

    # Determine intervals etc.
    x_label     = "'Bearing'"
    y_label      = '"Frequency Of Occurrence (%)"'

    #draw_histo = "psrose %s -: -A%s -S1.8in -R0/1/0/360 -B0.2g0.2:'Relative Frequency Of Occurrence':/30g30:.'Bearing': -Gblack -W0.75p,black -D > %s" % \
    #draw_histo = "psrose %s -: -A%s -S1.8in -R0/1/0/360 -B0.2g0.2:'Relative Frequency Of Occurrence':/30g30:.'Bearing': -G124 -W0.75p,black -D > %s" % \
    #             ("tmp.txt",repr(incbin),temp_ps)
    
    # Labels and Title
    #draw_histo = "psrose %s -: -A%s -S1.8in -R0/1/0/360 -B0.2g0.2:'Relative Frequency Of Occurrence':/30g30:.'%s': -G124 -W0.75p,black -D > %s" % \
    #             ("tmp.txt",repr(incbin),title,temp_ps)

    # No label or title
    draw_histo = "psrose %s -: -A%s -S1.8in -R0/1/0/360 -B0.2g0.2/30g30 -L -F -G124 -W0.75p,black -D > %s" % \
                 ("tmp.txt",repr(incbin),temp_ps)
                 
    #draw_histo = "psrose %s -: -A%s -S1.8in -R0/1/0/360 -B0.2g0.2/30g30 -G125 > %s" % \
    #                 ("tmp.txt",repr(incbin),temp_ps)

    todos = [defaults,draw_histo,"rm -f tmp.txt"]

    todos.append(make_fig)

    for CMD in todos :
        # watch running things in background/threads as some operations
        # must complete before others start and too use of common names
        # could cause overrunning for slow processes...
        #if verbose: os.system("touch %s" % (plot_name))
        #else :
            #status = commands.getstatusoutput(CMD)
            ##print status,CMD
            #if status[0] != 0 :
            #    print "plothisto ERROR\n",status
            #    sys.exit()
        status = commands.getstatusoutput(CMD)
        #print status,CMD
        if status[0] != 0 :
            print "plothisto ERROR\n",status
            sys.exit()


