# Standard modules
import math

# Third party modules

# MCMS modules
from g2l import g2l
from gcd import gcd
from grid2ij import grid2ij

def set_search_radius(defs_grid,defs,rf,fixed_radius,rname,verbose):
    """
    """
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------
    
    # Pre-bind for speed
    jm = defs_grid["jm"]
    im = defs_grid["im"]
    start_lat = defs_grid["start_lat"]
    start_lon = defs_grid["start_lon"]
    row_end = defs_grid["row_end"]
    row_start = defs_grid["row_start"]
    dlon = defs_grid["dlon"]
    dlat = defs_grid["dlat"]
    dx = defs_grid["dx"]
    maxid = defs_grid["maxid"]
    
    use_all_lons = []
    search_radius = []
    regional_nys = []
    try:
        r = open(rname,"w")
    except:
        print "Error Createing %s" % rname
        return 1
    verbose_here = 0
    
    if verbose:
        print "\n\tSetup Search Radius"
    fmt = "\trow %02d/lat %6.2f, distx %8.2f, search_radius %8.2f, "
    fmt += "sr_extra %8.2f latitude circumference %8.2f, %s regional_nys %d"
    lmsg = "Spans all lons"
    # At least the equivalent of dlat
    search_radius_min = 1*math.degrees(dlat)*111.0
    if search_radius_min > 300.0:
        # very coarse models need adjustment
        search_radius_min = 2.5*111.0
    # Fuzzy factor for grid searches (1/2 the longitude grid spacing)
    sr_extra = {}
    for row in xrange(jm):
        start = row_start[row]
        starti,startj = grid2ij(start,im,jm)
        startlon = g2l(starti,start_lon,start_lat,dlon,dlat,jm=jm,
            lat_lon_flag="lon",edge_flag=False,center_flag="center",
            flag_360=True,faux_grids = defs.faux_grids)
        startlat = g2l(startj,start_lon,start_lat,dlon,dlat,jm=jm,
            lat_lon_flag="lat",edge_flag=False,center_flag="center",
            faux_grids = defs.faux_grids)
        end = row_end[row]
        endi,endj = grid2ij(end,im,jm)
        endlon = g2l(endi,start_lon,start_lat,dlon,dlat,jm=jm,
            lat_lon_flag="lon",edge_flag=False,center_flag="center",
            flag_360=True,faux_grids = defs.faux_grids)
        endlat = g2l(endj,start_lon,start_lat,dlon,dlat,jm=jm,
            lat_lon_flag="lat",edge_flag=False,center_flag="center",
            faux_grids = defs.faux_grids)
        
        # Deal with cases of 89.99999
        startlon = round(startlon,2)
        startlat = round(startlat,2)
        endlon = round(endlon,2)
        endlat = round(endlat,2)
        
        distx = round(gcd(startlon,startlat,endlon,endlat),2)
        if fixed_radius > 0.0:
            tmp = fixed_radius
            search_radius.append(tmp)
        else:
            tmp = 0.5 * (2.0 * math.pi * defs.earth_radius *
                math.cos(math.radians(startlat)))/float(defs.wavenumber)
            tmp = round(tmp,2)
            # Provide a minimum search_radius so that at high latitudes
            #   the search covers a minimum distance.
            if tmp <= search_radius_min:
                tmp = search_radius_min
            search_radius.append(tmp)
        
        # Find Width of a grid at each latitude
        if fixed_radius > 0.0:
            sr_extra[row] = 0.0
        else:
            sr_extra[row] = round(0.5*gcd(startlon,startlat,
                startlon+dx,startlat),2)
        # Circumference less than critical_radius*2 or grid spacing
        # effectively zero (i.e. centered on pole)
        if distx*im <= tmp*2.0 or distx <= 1.0:
            use_all_lons.append(row)
        # Maximum number of rows to check within critical_radius, minimum 2
        temp = int(round(tmp/111.0/math.degrees(dlat))) + 1
        if temp < 2:
            temp = 2
        regional_nys.append(temp)
        
        msg1 = " "*len(lmsg)
        if row in use_all_lons: msg1 = lmsg
        rmsg = fmt % (row,defs_grid["lats"][row],distx,search_radius[-1],
            sr_extra[row],distx*im,msg1,regional_nys[-1])
        r.writelines(rmsg+"\n")
        if verbose:
            print rmsg
    try:
        r.close()
    except:
        print "Error Closing %s" % rname
        return 1
    return (use_all_lons,search_radius,regional_nys,sr_extra)

