#!/usr/bin/env python -tt
"""This module pre-calculate things for mcms_center_finder.py

Options/Arguments:

Returns/Creates:

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard modules
import math

# Third party modules

# MCMS modules
from g2l import g2l
from grid2ij import grid2ij
from set_search_radius import set_search_radius
from fill_search import fill_search
from inflate_gdict import inflate_gdict
#from print_gdict import PrintGdict
#from shift_gridid import shift_gridid

def setup_center_finder(defs_grid,defs,rf):
    """Setup stuff for center_finder"""
    
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------

    # Pre-bind for speed
    jm = defs_grid["jm"]
    im = defs_grid["im"]
    start_lat = defs_grid["start_lat"]
    start_lon = defs_grid["start_lon"]
    #row_end = defs_grid["row_end"]
    #row_start = defs_grid["row_start"]
    #row_start_index = row_start.index
    dlon = defs_grid["dlon"]
    dlat = defs_grid["dlat"]
    maxid = defs_grid["maxid"]
    #eq_grid = defs_grid["eq_grid"]
    #eq_grid_on_edge = defs_grid["eq_grid_on_edge"]
    #pbot = defs_grid["pbot"]
    #ptop = defs_grid["ptop"]

    #
    # Threshold for Laplacian Test below which center discarded as unlikely
    # to be cyclone.
    # Units of hPa per degree latitude squared
    lapp_cutoff = 0.15
    #lapp_cutoff = 0.4
    #joy lapp_cutoff = 0.05

    #
    # Threshold for the peak horizontal pressure gradient.
    # Holton says the horizontal pressure gradient ~0.01 hPa/km.
    #
    hpg_cutoff = 0.07
    
    # Find latitudes where all longitudes fit with critical_radius.
    rname = "%ssearch_radius_report.txt" % rf.shared_path
    verbose_here = 0
    fnc = set_search_radius(defs_grid,defs,rf,defs.critical_radius,rname,
        verbose_here)
    use_all_lons,search_radius,regional_nys,sr_extra = fnc

    # Enumerate the model grid (GridID) and then make a dictionary with the 8
    #   grid points around each GridID ... adapting for polar and longitude
    #   wrap around.
    #tmpdict = inflate_gdict(im,jm)
    ## Trim to start_row
    #gdict = {}
    #for gid in tmpdict:
    #    if gid in row_start:
    #        gdict[gid] = tmpdict[gid]
    #del tmpdict
    gdict = inflate_gdict(im,jm)

 #   # Make a gdict_outer (gdict of gdict) of all 9-pnts around gridid.
 #   #
 #   # Note: for the im=jm=8 in (See docs/working_with_grids.pdf) this
 #   #   looks a little odd because +/- 3 is close to halfway.
 #   #   
 #   # Level 1: around gridID cnt
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   ---   ---   ---   ---   ---   ---    List Order
 #   #    ---   ---   upm   upc   upp   ---   ---        0 1 2
 #   #    ---   ---   cnm   cnt   cnp   ---   ---        3 4 5
 #   #    ---   ---   dnm   dnc   dnp   ---   ---        6 7 8
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #
 #   # Level 2: Around gridID cnt (i.e., upp of upp or gdict[gdict[cnt][1]][1])
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   upm   ---   upc   ---   upp   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   cnm   ---   cnt   ---   cnp   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   dnm   ---   dnc   ---   dnp   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #
 #   # Level 3: Around gridID cnt
 #   #    upm   ---   ---   upc   ---   ---   upp   -3,+3  +0,+3   +3,+3
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    cnm   ---   ---   cnt   ---   ---   cnp   -3,+0  +0,+0   +3,+0
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    ---   ---   ---   ---   ---   ---   ---
 #   #    dnm   ---   ---   dnc   ---   ---   dnp   -3,-3  +0,-3   +3,-3
 #   #gdict_offsets = [(dc,dr) for dr in [3,0,-3] for dc in [-3,0,3]]
 #   import numpy as np
 #   c_halo = np.array([x  for y in xrange(3) for x in xrange(im)],dtype=np.int32)
 #   polar = [x for x in xrange(0,pbot+1)]
 #   polar.extend([x for x in xrange(ptop,maxid+1)])
 #   gdict_outer = {}    
 #   for gridid in xrange(im*jm):
 #       if rf.model not in ["merra","ccsm"]:
 #           # For most models only use this at polar latitudes
 #           if gridid not in polar: continue
 #       gdict_outer[gridid] = [shift_gridid(gridid,dc,dr,c_halo,halfway,im,jm) 
 #               for dr in [3,0,-3] for dc in [-3,0,3] ]
# 
 #   #
 #   # For lower resolution models this is only applied to the polar regions
 #   #
 #   # Define row_info[gridid] = (row,row_start_gridid,row_end_gridid,offset)
 #   #   where row_start_gridid, row_end_gridid are the gridIDs for the row(latitidue) 
 #   #   that gridid comes from and offset is the number of columns the 
 #   # gridid is sifted relative to row_start.
 #   row_info = {}
 #   for gridid in xrange(maxid):
 #       row_guess = [x for x in row_start if x <= gridid]
 #       row_guess = row_start_index(row_guess[-1])
 #       row_info[gridid] = (row_guess,row_start[row_guess],row_end[row_guess],
 #               gridid - row_start[row_guess])      
 #   for gridid in xrange(maxid):
 #       if rf.model not in ["merra","ccsm"]:
 #           # For most models only use this at polar latitudes
 #           if gridid not in polar: continue
 #       if gridid != row_info[gridid][1]:
 #           # offset gridid, check if need to calculate
 #           if row_info[gridid][0] > 3 and  row_info[gridid][0] < jm-4:
 #               continue
 #       # Limit center finding by hemisphere (for speed)   
 #       #if rf.limit_nh and gridid <= eq_grid: continue
 #       #if rf.limit_sh and gridid >= eq_grid: continue                
 #       all_nine = gdict[gridid][:]
 #       new_nine = ["-"]*9
 #       new_nine[4] = gridid
 #       for get_this in [0,1,2,3,5,6,7,8]:
 #           # If SPole then the  dnm   dnc   dnp one level out mirrors the
 #           #   cnm   cnt   cnp one level out due to over the pole wrap over.
 #           #
 #           # If NPole then the  upm   upc   upp one level out mirrors the
 #           #   cnm   cnt   cnp one level out due to over the pole wrap
 #           if rf.model in ["merra","ccsm"]:
 #               new_nine[get_this] = gdict[gdict[all_nine[get_this]][get_this]][get_this]
 #           else:
 #               new_nine[get_this] = gdict[all_nine[get_this]][get_this]
 #       gdict_outer[gridid] = new_nine
 #   print len(gdict_outer)
 #   print gdict_outer.keys()
 #   import sys; sys.exit("Stop Here")


    # Define some lookup tables
    rdict = {} # define dictionary of regional grids
    ldict = {} # define dictionary for laplacian calculations
    ijdict = {} # define dictionary of i,j lon,lat for gridid
 
    half_way = int(im/2.0)
    verbose_here = 0
    if verbose_here:
        print "\n\tSetup rdict, ldict, ijdict\n"
    for gridid in xrange(maxid):

        # Populate lookup table that gives column,row,lat,lon by GridID
        col,row = grid2ij(gridid,im,jm)
        lon = g2l(col,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lon",
            edge_flag=False,center_flag="center",flag_360=True,
            faux_grids = defs.faux_grids)
        lat = g2l(row,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lat",
            edge_flag=False,center_flag="center",faux_grids = defs.faux_grids)
        ijdict[gridid] = [col,row,lon,lat]
        if verbose_here:
            print "\t%s" % ("-"*20)
            msg = "\tChecking gridid : %d (col:%03d,row:%03d,lat:%6.2f,lon:%7.2f)"
            lat = round(lat,2)
            lon = round(lon,2)
            print msg % (gridid,col,row,lat,lon)
        
        # Populate a lookup table that gives the Laplacian in spherical 
        #    coordinates (radians) for each latitude (same along longitude)
        #
        #  LAP_P = 1/a^2 * d^2P/dlat^2 + 1/a^2sin^2(lat) * d^2P/dlon^2
        #          + cot(lat)/a^2 * dP/dlat
        # Same for all longitudes along a latitude.
        if col == 0:
            rlat = math.radians(lat)
            temps = 0.5*(1.0+math.cos(rlat*2.0)) # equivalent sin^2(lat)
            part1 = 1
            # Note convert radius to m as pressure in Pa (kg/ms^2)
            if temps == 0.0:
                # Poles
                part1 = part2 = 0.0
            else:
                # 1/sin^2(lat) * 1/a^2
                part2 = 1.0/temps * (defs.inv_earth_radius_sq*0.0001)
            temps = math.tan(rlat)
            if temps == 0.0:
                # Equator
                part1 = part3 = 0.0
            else:
                #  cot(lat)/a^2
                part3 = 1.0/temps * (defs.inv_earth_radius_sq*0.0001)
            ldict[row] = [part1,part2,part3]

    # Populate a search lookup table
    #    
    #  * For fixed critical_radius the pattern should be zonally symmetric
    #    with the count increasing rapidly toward the poles because the number
    #    grids within critical_radius increases with latitude and over-the
    #    pole wrap-around leads to multiple hits.
    #
    #  * For wave-number the pattern should be the opposite as the number of
    #    search grids decreases with latitude. Thus the peak is around the
    #    equator.
    #   
    # Just do for the start_row and use shift_gridid to slide along longitudes.
    verbose_here = 0
    rdict = fill_search(defs_grid,defs,rf,regional_nys,search_radius,
        sr_extra,gdict,ijdict,use_all_lons,verbose_here)
    
    return (use_all_lons,search_radius,regional_nys,rdict,ldict,
            ijdict,lapp_cutoff,hpg_cutoff)

