# Standard modules
import sys
import os
import numpy
import math
from collections import defaultdict

# What sort of figures (png faster, eps/pdf better quality)
fig_format = ".png"
#fig_format = ".eps"
#fig_format = ".pdf"

# Third party modules
import matplotlib
if fig_format == ".png":        
    # Saves memory
    matplotlib.use('Agg')
elif fig_format == ".pdf":
    matplotlib.use('PDF')
elif fig_format == ".eps":
    matplotlib.use('PS')    

import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.cm as cm
from matplotlib.patches import Polygon
import matplotlib.ticker as ticker
import matplotlib.mlab
import scipy.stats

# MCMS modules
#import util.plot_map
#from mcms_track_finder import process_data, rose_plot

Median = numpy.median
PDF = matplotlib.mlab.normpdf

Summerize = scipy.stats.describe
NA = numpy.array
NMean = numpy.mean
NSTD = numpy.std
NAve = numpy.average
Mode = scipy.stats.mode
where_mode = numpy.argmax
Percentile = scipy.stats.scoreatpercentile
Skew = scipy.stats.skew
Kurtosis = scipy.stats.kurtosis    


def CompletedCallbackPbar(step):
    """
    Reports the process of a multiprocessing session.
    
    Same as CompletedCallback except intended for use with
    global defined ProgressBar instead of printing a string.
    
    """
    
    # Assumes the calling program has global ProgressBar defined.
    global results
    global mp_pbar
    results.append(step)
    mp_pbar.update_time(len(results),which_one=step)
    print(mp_pbar)
    
    return

def mp_test(rh_what_do):
    i = int(rh_what_do["loop_year"])
    for aslan in xrange(10000):
        i = i + 1
    return i

# class DefineHistogramStuff:
#     '''Class to define, setup and plot histograms and time series
#     for mcms_check.py. Completely depends on matplotlib and numpy.
#     
#     then can just define the histo_plot_defs.txt file of their liking
#     same with map!
#     
#     Author: Mike Bauer  <mbauer@giss.nasa.gov>
#     
#     Log:
#         2010/12  MB - File created.
#     '''
#     
#     def __init__(self,**kwargs):
#         """This is where the default values are set with the option to
#            alter then upon instantiation."""
#         # Name: defs_file
#         # Purpose: Full path to file that contains the details of how
#         #   to plot the data.
#         # Default: empty - raises error
#         if kwargs.has_key('defs_file'):
#             self.defs_file = kwargs['defs_file']
#         else:
#             msg = "Error DefineHistogramStuff: Requires defs_file"
#             sys.exit(msg)
#         # Name: source_title
#         # Purpose: Data source name for plot titles
#         # Default: empty
#         if kwargs.has_key('source_title'):
#             self.source_title = kwargs['source_title']
#         else:
#             self.source_title = ""
#         # Name: source_period
#         # Purpose: Time Period for plot titles
#         # Default: empty
#         if kwargs.has_key('source_period'):
#             self.source_period = kwargs['source_period']
#         else:
#             self.source_period = ""
#             
#         self.label = ""
#         if self.source_title:
#             if self.source_period:
#               msg = 'Source: %s Period: %s'
#                 self.label = msg % (self.source_title.upper(),self.source_period)
#             else:
#               msg = 'Source: %s'
#                 self.label = msg % (self.source_title.upper())
#     
#     def read_histo_plot_defs(self,):
#         self.histo_stuff = {}
#         tmp = {}
#         msg = ("make_plot = %d; normalized = %d; title = '%s'; xlab = '%s';",
#             "ylab = '%s'; bin_width = %f; bins_left_edge = %s; sum_type = %d;",
#             "xlabt = '%s'; ylabt = '%s'; ytupper = %f; ytlower = %f;",
#             "xlab2d = '%s'; ylab2d = '%s'; yuppern = %f; xupper = %f;",
#             "xlower = %f; trim_max = %f; trim_min = %f; may_trim = %d")
#         msg = " ".join(msg)
#         # File I/O
#         try:
#             r_file = open(self.defs_file,"r")
#         except IOError:
#             print "Error Opening: %s" % self.defs_file
#         # Read
#         try:
#             rline = r_file.readlines()
#         except IOError:
#             print "Error reading:" % self.defs_file
#         # Parse
#         for line in rline:
#             if line:
#                 rparts = line.split("|")
#                 if len(rparts) == 1:
#                     # New Variable Def
#                     new_var = rparts[0].rstrip()
#                     if new_var:
#                         tmp[new_var] = {}
#                         #self.histo_stuff[new_var] = {}
#                 else:
#                     do_var = rparts[0].rstrip()
#                     do_val = rparts[1].rstrip()
#                     if do_var == "title" and not do_val:
#                       do_val = self.label
#                     tmp[new_var][do_var] = do_val
#                     #self.histo_stuff[new_var][do_var] = do_val
#         # Convert into python string for matplotlib
#         for do_var in tmp:
#             do = tmp[do_var]
#             self.histo_stuff[do_var] = msg % (int(do["make_plot"]),
#             int(do["normalized"]),do["title"],do["xlab"],do["ylab"],
#             float(do["bin_width"]),do["bins_left_edge"],int(do["sum_type"]),
#             do["xlabt"],do["ylabt"],float(do["ytupper"]),float(do["ytlower"]),
#             do["xlab2d"],do["ylab2d"],float(do["yuppern"]),float(do["xupper"]),
#             float(do["xlower"]),float(do["trim_max"]),float(do["trim_min"]),
#             int(do["may_trim"]))
#         # Close
#         try:
#             r_file.close()
#         except IOError:
#             print "Error Closing: %s" % self.defs_file
#     

def setup_stats_report(ssr_stat_group,ssr_per_step_group):
    """
    Define format for reading and writing stats dump from compile_stats
    
    Compile_stats will create a data file storing each usi (putative cyclone)
       such that all the wanted data is laid out in the following format.
       
       Many of these statistics are arranged about the minimum lifetime SLP (MLS)
       for that cyclone/track. We can imagine hypothetical cyclone whose life
       history is perfectly symmetric about its MLS. That is, the central SLP
       begins and ends at the same value, which is also the maximum lifetime
       SLP for the cyclone. If cyclone development is likewise symmetric then
       the amount of time between the start of the cyclone and the MLS and that
       between the MLS and end of the cyclone are the same.
       
    Input:
    ^^^^^
    ssr_stat_group - List
        Names of all the statistics to collect for histograms and the like.
    ssr_per_step_group - List
        Names of per time step statistics that are now collected by this
        data structure.
    
    Output:
    ^^^^^^
    Y - Expected type,shape (string,float,integer,list, numpy array etc).
        Description ...
    
     usi - Unique cyclone/track identifier.
     span - Total track length (# of timesteps, includes the MLS timestep))
       span = 1 + spanb + spane
     spanb - Start to MLS span (excludes the MLS timestep).
       spanb = 0 if the MLS is at the beginning of the track.
       spanb = span-1 if the MLS is at the end of the track
     spane - MLS to end span (excludes the MLS timestep).
       spane = span-1 if the MLS is at the beginning of the track.
       spane = 0 if the MLS is at the end of the track
     asymmetry - Measure time asymmetry about the MLS.
       asymmetry = (spane-spanb)/(span-1)
                 = -1 when MLS is at the beginning of the track
                 =  0 when MLS is at the middle of the track
                 =  1 when MLS is at the end of the track
     amplitude - Maximum SLP difference between the beginning/end of the track
       and the MLS.
       amplitude = ampb IF ampb >= ampe ELSE ampe
     ampb - SLP at start of track minus that at the MLS.
       ampb = 0 if MLS is at the beginning of the track, amplitude = ampe
     ampe - SLP at end of track minus that at the MLS.
       ampe = 0 if MLS is at the end of the track, amplitude = ampb
     amp-asymmetry - Measure SLP amplitude asymmetry about the MLS.
       amp-asymmetry = (ampe - ampb)/amplitude
                     = -1 if MLS is at the beginning of the track
                     < 0 if ampb > ampe, MLS not at the beginning of the track
                     = 0 if ampb = ampe
                     > 0 if ampe > ampb, MLS not at the end of the track
                     = 1 MLS is at the end of the track
     aged-indices - array indices of the track data to follow so that they
       can be counted and summed in MLS relative arrays.
     usi
     span spanb spane asymmetry amplitude ampb ampe amp-asymmetry
     aged_indices
     track sequences based on stat_group.
       note that missing values are denoted with -1 to preserve their aged
       relative position and to allow for co-occurrence statistics such as
       correlation.
       Also, some quantities (e.g.,"track-length") are single
       value rather than sequences.
     dates - array to YYYYMMDDHH for determining season etc.
    
    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.
    
    
    Notes/Warnings:
    ^^^^^^^^^^^^^^
    
    
    Author: Mike Bauer  <mbauer@giss.nasa.gov>
    
    Log:
        0000/00 MB - File created.
    """
    
    # Basic life-cycle statistics
    ssr_stat_header = ['span','spanb','spane','asymmetry','amplitude',
            'ampb','ampe','amp-asymmetry','aged-indices']
    ssr_stat_holder = ssr_stat_header[:]
    
    # Additional statistics based on ssr_per_step_group.
    ssr_stat_body = [x for x in ssr_stat_group if x not in ssr_per_step_group]
    ssr_stat_holder.extend(ssr_stat_body)
    
    # Used to dump/reaq data as text to disk.
    ssr_stat_header_fmt = dict.fromkeys(ssr_stat_header,"%s: %s\n")
    ssr_stat_body_fmt = dict.fromkeys(ssr_stat_body,"%s: %s\n")
    
    return ssr_stat_header_fmt,ssr_stat_body_fmt


#---Start of main code block.
if __name__ == '__main__':

    # Assume run as from src/ with pyf util/setup_stats_report.py

    # Normally this would be run from inside check.py

    no_stats = 0
    no_title = 1
    cyclolysis = 0
    cyclogenesis = 0
    single_processor = 1
    make_box_plots = 0
    make_age_plots = 1    
    verbose = 0
    # Only Store Data at SLPmin
    use_slp_min = 1
    
    # Intensity evaluated instantaneously (default)
    #use_uci_intensity = 0
    # Intensity evaluated only at SLPmin
    use_usi_intensity = 0
    #if use_uci_intensity == use_usi_intensity:
    #    sys.exit("use_uci_intensity and use_usi_intensity must differ.")
    if use_slp_min:
        use_usi_intensity = 0
        make_age_plots = 0

    if cyclolysis and cyclogenesis:
        print "Can't have both cyclogenesis and cyclolysis"
        sys.exit()

    # Shortcut to limit what is done
    single_variable = ""
    #single_variable = "asymmetry_annual_global_all_1979_1980"
    single_variable = "atts-area"

    # Skip reading files from compile_stats and read pre-compiled
    #   files instead, for rapid tweaking of plots in make_plot.
    #   NOTE this applies to both the histogram and maps so if only
    #   wish to tweak one of these use comments as needed.
    recover_stat_files = 0

    if single_variable and not recover_stat_files:
        sys.exit("single_variable only works with recover_stat_files")

    # ==========================================================================
    # Start: This is done in check now but added here for demonstation.
    # ==========================================================================

    import gzip
    import extras.progress_bar
    import numpy
    import textwrap
    import mcms_utilities
    import cPickle
    import defs

    # --------------------------------------------------------------------------
    # Select options for this run.
    # --------------------------------------------------------------------------
    picks = {0 : "NCEP/NCAR Reanalysis 1",
             1 : "NCEP-DOE Reanalysis 2",
             2 : "NASA GISS GCM ModelE",
             3 : "GFDL GCM",
             4 : "ERA-Interim Reanalysis",
             5 : "ERA-40 Reanalysis"}
    pick = 1
    if pick not in picks:
        sys.exit("ERROR: pick not listed in picks.")

    # This next set of lines should be copied from setup_vX.py
    # Short names by which pick will be labeled.
    models = ["nra","nra2","giss","gfdl","erai","era40"]
    try:
        model = models[pick]
    except:
        sys.exit("ERROR: pick not listed in models.")

    # Length of file ending to replace if using year_loop
    tails = ["_att.txt","_tracks.txt","_centers.txt","_dumped_centers.txt"]
    tail = tails[0]
    cut_tail = len(tail)

    # Flags
    #  tracks: track info included in file
    #  att: attribute info included in file
    #
    # Note tweaked self.just_center_table: in mcms_read for center/track pre-att read
    # also watch detail_tracks names in template
    tracks = ""
    att = ""
    if tail.find("tracks") != -1:
        tracks = 1
    if tail.find("att") != -1:
        att = 1
        # Note att files can contain track info so if you want
        # track statistics for an att file manually set tracks
        # to 1 here.
        tracks = 1

    # Halt program on error or just warn?
    exit_on_error = 1

    # Plot map on error (requires matplotlib, also doubles or more memory
    # footprint)
    plot_on_error = 0

    # No Warnings: Turn on/off error checking completely (for debugging)
    error_check = 0

    # Make only summary plots (for whole records) and skip
    #   the year-by-by year plots
    summary_plots_only = 1

    # Find the STD maps using compile_stats
    save_std = 0
    if save_std:
        # On this run of compile_stats don't save the histogram
        #   data or mean files, instead read the mean files
        #   previously saved (from compile_stats) and find
        #   the standard deviation maps using the long term
        #   mean of the whole record.
        save_stat_files = 0
        save_mean_files = 0
    else:
        save_stat_files = 1
        save_mean_files = 1
    #print "#tmp save_stat_files = 0:"
    #save_stat_files = 0
    #save_mean_files = 0
    #print "#tmp end"


    # Store some maps and netcdf files for latter use
    store_netcdf = 1

    # Don't make plots (usually used with store_netcdf, for
    #   saving the data for making plots rather than making them).
    plots_no_draw = 0

    # Check for graphics programs
    #   The psrose plotting program from the GMT (The Generic Mapping Tools,
    #   http://gmt.soest.hawaii.edu/) project provides a much better wind
    #   rose plot than that from the matplotlib plotting package that is generally
    #   used by the MCMS project.
    psrose = mcms_utilities.Is_It_Available("psrose")
    if not psrose:
        msg = "Warning: GMT (The Generic Mapping Tools) not found. "
        msg += "Some plots will not be made."
        print msg
    try:
        import matplotlib
    except ImportError:
        print "Warning: matplotlib not available. Disabling plotting altogether."

    # Values only taken at center rather than over area (for mean maps)
    point_source = 0
    if not att:
        point_source = 1

    # Skip high latitudes for angle, separation, area. skip_high_lat is the absolute latitude
    #   above which no values retained so set to 90 to disable. Note for bearing this
    #   is hard coded to 85 no matter what the skip_high_lat value.
    skip_high_lat = 90

    season_selects = {0 : ["annual"],
            1 : ["annual","djf","mam","jja","son"],
            2 : ["annual","ndjfma","mjjaso"],
            3 : ["annual","ndjf","mamjjaso"],
            4 : ["annual","jan","feb","mar","apr","may","jun",
                "jul","aug","sep","oct","nov","dec"]
            }
    season_select = 2
    seasons = season_selects[season_select]
    mm2season = { 0 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0},
        1 : { 1 : 1, 2 : 1, 3 : 2, 4 : 2, 5 : 2, 6 : 3, 7 : 3, 8 : 3, 9 : 4,
            10 : 4, 11 : 4, 12 : 1},
        2 : { 1 : 1, 2 : 1, 3 : 1, 4 : 1, 5 : 2, 6 : 2, 7 : 2, 8 : 2, 9 : 2,
            10 : 2, 11 : 1, 12 : 1},
        3 : { 1 : 1, 2 : 1, 3 : 2, 4 : 2, 5 : 2, 6 : 2, 7 : 2, 8 : 2, 9 : 2,
            10 : 2, 11 : 1, 12 : 1},
        4 : { 1 : 1, 2 : 2, 3 : 3, 4 : 4, 5 : 5, 6 : 6, 7 : 7, 8 : 8, 9 : 9,
            10 : 10, 11 : 11, 12 : 12}}
    # How season months relate to the current year "0"
    mm2year = { 0 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0},
                1 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : -1},
                2 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : -1, 12 : -1},
                3 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : -1, 12 : -1},
                4 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0}}
    m2y = mm2year[season_select]
    m2s = mm2season[season_select]

    # FIX:
    # NOTE: For now djf, ndjfma etc only valid for summary (climatology as we don't
    #   carry the Dec Year X over to the JAN, FEB of Year X + 1 so only totals valid.
    #   could be done with m2s and the same read forward/backward used w/ tracks option.
    #   or could pass those back to main and into next year as needed.

    # Select break down by hemisphere
    hemisphere_selects = { 0 : ["global"],
            1 :  ["global","nh","sh"]
            }
    hemis_select = 1
    hemis = hemisphere_selects[hemis_select]

    # Select by system intensity
    intensity_selects = { 0 : ["all"],
            1 : ["all","wea", "mod", "str"]
            }
    intensity_select = 1
    intensities = intensity_selects[intensity_select]

    if intensity_select == 0:
        # Makes no sense
        use_usi_intensity = 0

    # --------------------------------------------------------------------------
    # Define all modules to be imported.
    # --------------------------------------------------------------------------

    # --------------------------------------------------------------------------
    # Alter default behavior found in either defs_vX.py or setup_vX.py
    # --------------------------------------------------------------------------

    # The default behavior is to read SLP data from the
    # directory slp_path defined in setup_vX.py.
    # Here you can elect to override this behavior.
    over_write_slp_path = ""
    #over_write_slp_path = "/data/nra2/"
    if over_write_slp_path and not over_write_slp_path.endswith("/"):
        sys.exit("paths must end in / see %s" % over_write_slp_path)

    # The default behavior is to save results
    # in the directory out_path defined in
    # setup_vX.py. Here you can elect to override
    # this behavior.
    over_write_out_path = ""
    #over_write_out_path = "/output/nra2/"
    if over_write_out_path and not over_write_out_path.endswith("/"):
        sys.exit("paths must end in / see %s" % over_write_out_path)

    # This next set of lines should be copied from setup_vX.py
    # Full path to the root directory where pick specific output will be stored.
    # Note it's possible that all of these directories are identical.
    #result_directories = ["/Volumes/scratch/output/",]
    # Uncomment to use same structure for all models
    result_directories = ["/Volumes/scratch/output/" for x in range(len(picks))]
    #result_directories = ["/output/" for x in range(len(picks))]
    # Uncomment to make current directory for all models
    #result_directories = [os.getcwd() for x in picks]
    try:
        result_directory = result_directories[pick]
        if not os.path.exists(result_directory):
            sys.exit("ERROR: result_directory not found.")
    except:
        sys.exit("ERROR: pick not listed in result_directories.")

    # Directory to be created for storing temporary pick specific files.
    shared_path = "%s%s_files/" % (result_directory,model)
    if shared_path and not shared_path.endswith("/"):
        sys.exit("paths must end in / see %s" % shared_path)

    # The default behavior is to run over all the
    # years found by setup_vX.py. Here you can
    # elect to override this behavior.
    over_write_years = []
    if pick == 5:
        over_write_years = [1961,1990]
    #over_write_years = [1979,1979]
    #over_write_years = [1996,1996]
    #over_write_years = [1979,1980]

    # See recipe inline_1 in inline.py
    # Get some definitions. Note must have run setup_vx.py already!
    sf_file = "%ss_dat.p" % (shared_path)
    try:
        fnc_out = cPickle.load(open(sf_file))
    except:
        sys.exit("\n\tWARNING: Error reading or finding %s" % (sf_file))
    # Parse fnc_out
    inputs = ("im","jm","maxid","lats","lons","timestep","dx","dy","dlon","dlat",
        "start_lat","start_lon","dlon_sq","dlat_sq","two_dlat","model_flag","eq_grid",
        "tropical_n","tropical_s","bot","mid","top","row_start","row_end",
        "tropical_n_alt","tropical_s_alt","bot_alt","top_alt","lon_shift","lat_flip",
        "the_calendar","found_years","super_years","dim_lat","dim_lon","dim_time",
        "var_lat","var_lon","var_time","var_slp","var_topo","var_land_sea_mask",
        "file_seperater","no_topo","no_mask","slp_path","model","out_path",
        "shared_path","lat_edges","lon_edges","land_gridids","troubled_centers",
        "faux_grids","polar_s","polar_n","pbot","ptop")
    # Use what wanted
    if not over_write_years:
        over_write_years = fnc_out[inputs.index("super_years")]
        over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
    if not over_write_out_path:
        over_write_out_path = fnc_out[inputs.index("out_path")]
    if not over_write_slp_path:
        over_write_slp_path = fnc_out[inputs.index("slp_path")]
    row_end = fnc_out[inputs.index("row_end")]
    row_start = fnc_out[inputs.index("row_start")]
    maxid = fnc_out[inputs.index("maxid")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    if model not in ["nra","nra2"]:
        # For needed for what_do/mcms_read
        land_gridids = fnc_out[inputs.index("land_gridids")]
    # Delete the rest
    del fnc_out

    # See recipe inline_3 in inline.py
    # Standard subdirectories to out_path:
    standard_subdirs = ['','comps/','pdfs/','stats/','stats/tmp/','netcdfs/']
    # Store figs by hemisphere and season to limit the number of
    #   files in each.
    fig_dirs = ["%s/%s/%s/" % (x,y,z) for x in
            ['figs/pdfs','figs/pdfs2d','figs/comps','figs/maps','figs/tseries'] for y in hemis
            for z in seasons]
    standard_subdirs.extend(fig_dirs)
    need_dirs = ("%s%s" % (over_write_out_path,x) for x in standard_subdirs)
    for nd in need_dirs:
        if not os.path.exists(nd):
            os.makedirs(nd)
            print "\tDirectory %s Created." % (nd)

    super_years = over_write_years
    out_path = over_write_out_path


#     tlen = 31
#     tmid = 16-1
#     tdat = range(tlen)
#     sold = 0
#     smax = 1015.0
#     smin = 980.0
#     #sdelta = (smax-smin)/(float(tmid))
#     sdelta = (smax-smin)/(float(tlen))
#     sdat = []
#
#     sold = smax+sdelta
#     polarity = -1.0
#
#     sold = smin
#
#     for trak in tdat:
#         if sold <= smin: polarity = 1.0
#         sval = sold + (sdelta*polarity)
# #        print "\t%d\t%8.2f\t%8.2f" % (trak,sold,sval)
#         sold = sval
#         sdat.append(sval)
#     mean = sum(sdat)/float(tlen)
#     print mean
#     cusum = []
#     sold = 0.0
#     for trak in tdat:
#         snew = sold + (sdat[trak] - mean)
#         cusum.append(snew)
#         sold = snew
#     pname = "%sfigs/other/aplot_nsym%s" % (out_path,".png")
#     width = 10
#     hieght = width/1.61803399
#     fig = plt.figure(figsize=(width,hieght),frameon=True)
#     ax = plt.subplot(111)
#     ax2 = plt.twinx()
#     ax.plot(range(tlen),sdat,'ko',linestyle='solid')
#     ax2.plot(range(tlen),cusum,color='red',linestyle='dashed')
#     fig.savefig(pname)
#     import sys; sys.exit("Stop Here")

    # ==========================================================================
    # End: This is done in check now but added here for demonstation.
    # ==========================================================================
    timestep = 6

    # Maximum number of timesteps to collect aged statistics.
    #   aged_centered_index is an array of length 1 + 2*max_age
    # Default 14 days on each side of min_loc
    max_age = int(14*timestep)
    # Note because python is zero indexed min_loc can be set to max_age
    min_loc = max_age
    age_len = 1+(2*min_loc)
    min_age = -1*(min_loc*timestep)
    age = min_age-timestep
    age_hours = {}
    for x in xrange(age_len):
        age += timestep
        age_hours[x] = age

    # Trim so just 3 days on each side of min_loc
    min_age_index = min_loc - int((3.0*24.0)/float(timestep))
    max_age_index = min_loc + int((3.0*24.0)/float(timestep))
    
    # Tracks near the start of end of the record didn't have all track filters
    #   applied.... don't use for statistics?

    # Fetch definitions and impose those set in cs_defs_set.
    defs_set = {}
    defs = defs.defs(**defs_set)

    # Shortest allowable track length (based on time elapsed between pnts, such
    # that 1 timestep spans 2 pnts).
    min_trk_cnt = int(defs.age_limit/timestep) + 1
    keep_slp = defs.keep_slp*0.001
    min_travel = defs.min_trk_travel

    # Define format for reading and writing stats dump from compile_stats
    stat_defs = check_define_stats(att,tracks)

    if verbose:
        print
        print "stat_defs['stat_body_fmt'] :",stat_defs["stat_body_fmt"]
        print "stat_defs['cds_age_pairs'] :",stat_defs['cds_age_pairs']
        print "stat_defs['cds_2d_pairs'] :",stat_defs['cds_2d_pairs']

    # Age Pair Labels
# FIX add this to  DefineHistogramStuff so get values from defs_file
    age_pair_label = {"title" : "Source: NRA2 Period: 1979-2010",
        "att-area" : "Cyclone ATT Area (Radius of Equivalent Circle, km)",
        "atts-area" : "Cyclone ATTS Area (Radius of Equivalent Circle, km)",
        "area-total" : "Total Cyclone Area (Radius of Equivalent Circle, km)",
        "att-depth" : "Cyclone ATT Depth (hPa)",
        "atts-depth" : "Cyclone ATTS Depth (hPa)",
        "depth-total" : "Cyclone Total Depth (hPa)",
        "sep" : "Distance to Nearest Center (Zonal Wavenumber)",
        "sep-alt" : "Distance to Nearest Center Alt (Zonal Wavenumber)",
        "dissimilarity" : "Dissimilarity",
        "fill" : "Deepening/Filling Rate (hPa/day)",
        "speed" : "Cyclone Speed (km/hr)",
        "slp" : "Central SLP (hPA)",
        "atts-compact" : "Compactness of ATTS Contour",
        "atts-aspect" : "Compactness of ATTS Contour",
        "att-compact" : "Compactness of ATT Contour",
        "att-aspect" :  "Compactness of ATT Contour"
        }

    # Trim
    if single_variable:
        new_list = []
        for the_key in stat_defs['cds_age_pairs']:
            if single_variable.find(the_key[1]) != -1:
                new_list.append(the_key)
        stat_defs['cds_age_pairs'] = new_list

# FIX add map stuff to DefineHistogramStuff then I can pass all these to make_plots

    # Instantiate histogram plotting routines.
    def_path = os.getcwd()+"/util/histo_plot_defs.txt"
    histodo = DefineHistogramStuff(defs_file=def_path,
            source_title="nra2",source_period= "1979-2010")
    if verbose: print "histodo.defs_file: ",histodo.defs_file

    # Read in histogram plotting definitions/settings.
    histodo.read_histo_plot_defs()

    if verbose: print
    if verbose: print "histodo.histo_stuff.keys(): ",histodo.histo_stuff.keys()
    if verbose: print
    if verbose: print "histodo.histo_stuff['cnts'] :",histodo.histo_stuff['cnts']
    if verbose: print

    # ==========================================================================
    # Start: This is done in check/make_plots now but added here for demonstation.
    # ==========================================================================

    # Maximum area per hemisphere
    h_area = 127524124.138
    depth_binsize = 2.0

    years = range( super_years[0], super_years[-1]+1)
    super_run = "%04d_%04d" % ( super_years[0], super_years[-1])

    # -------------------------------------------------------------------------
    # Setup
    # -------------------------------------------------------------------------

    #   By_Step: Which means the average value of all cyclones per timestep.
    #            For example, the average number of cyclone present per
    #            timestep.
    #   By_Cyclone: Which means the average value over all cyclones.
    #               For example, the average area per cyclone.
    stats_path = "%sstats/tmp/" % (out_path)
    super_titles = ["%ssuper_%s" % (stats_path,x) for x in stat_defs["per_step_group"]]
    super_group = {}
    dtime_seasons_super = {}
    dtime1_seasons_super = {}
    stat_group_ext = {}
    stat_group_ext_titles = []
    sg_fmt = '%s_%s_%s_%s'
    stat_group_ext_titles = [sg_fmt % (sn,ss,hh,ii)
            for sn in sorted(stat_defs["per_step_group"])
            for ss in seasons for hh in hemis
            for ii in intensities]
    # Skip exclusions via mp_sans_intensity
    skipi = [x.rstrip("_") for x in stat_defs["sans_intensity"]]
    skipe = [x for x in  intensities if x != "all"]
    stat_group_ext = dict.fromkeys(stat_group_ext_titles)
    ss_fmt = '%s%s_%s_%s'
    if use_usi_intensity:
        sf_fmt = "%sstats/tmp/super_rfile_%s_%s_piten.pz"
        sq_fmt = "%sstats/tmp/rtmp_%s_piten.txt.gz"
    elif use_slp_min:
        sf_fmt = "%sstats/tmp/super_rfile_%s_%s_slpmin.pz"
        sq_fmt = "%sstats/tmp/rtmp_%s_slpmin.txt.gz"        
    else:
        sf_fmt = "%sstats/tmp/super_rfile_%s_%s.pz"
        sq_fmt = "%sstats/tmp/rtmp_%s.txt.gz" 
    rfmt = "%sstats/tmp/rfile_%s_%d.pz"
    bfmt = "%sc_%s_%s_%s_%s_%s_%04d.txt.gz"
    dfmt = "%sstats/tmp/dtimes_%s_%s.pz"
    stats_fmt = "%sstats/tmp/stats_%s_%d.txt.gz"

   # Archive statistics about the maps being made.
    if use_usi_intensity:
        stats_report = "%sstats/check_histo_stats_%s_piten.txt" % (out_path,super_run)
    elif use_slp_min:
        stats_report = "%sstats/check_histo_stats_%s_slpmin.txt" % (out_path,super_run)
    else:
        stats_report = "%sstats/check_histo_stats_%s.txt" % (out_path,super_run)
    if single_variable:
        stats_report = stats_report.replace(".txt","_%s.txt" % single_variable)
    if verbose: print "\tCreating",stats_report
    try:
        report_save = open(stats_report,"w")
    except:
        sys.exit("\tError opening: %s" % (stats_report))
    # Add a header to stats_report
    header = mcms_utilities.CreateFileHeader()
    header.fill("Histogram Statistics Report","Statistical info.",
            __file__,model.upper(),"","Self Documenting")
    report_save.write(header.header_string+"\n")
    if verbose: print header.header_string
    report_fmts = setup_histo_report() 
    super_rfile = sf_fmt % (out_path,model,super_run)
    super_dfile = dfmt % (out_path,model,super_run)

    if summary_plots_only and recover_stat_files:
        # tweak years to only the last year done as that all that is
        #   wanted.
        years = [years[0]]

    pastTracks = {}
    fmt1 = "%s/stats/tmp/stats_%s_%d.txt.gz"
    # Create climatological bin storage for data covering all years.
    #   example: 'continued-usi_annual_global_all_1979_2009'
    sg1_fmt = '%s_%s_%s_%s_%s'
    pastDataHisto = defaultdict(list)
    pastDataHistoBaseKeys = [x for x in
        stat_defs["stat_body_fmt"].keys() if x not in ["dates"]]
    # Add some lifecycle stats
    extra_stats = [x for x in stat_defs["stat_header_fmt"].keys() if x in
        ['asymmetry', 'amplitude', 'amp-asymmetry','aged-indices']]
    pastDataHistoBaseKeys.extend(extra_stats)

    # Maximum number of grids per hemisphere
    tropical_end = row_end[tropical_n_alt]
    tropical_start = row_start[tropical_s_alt]
    max_grid_count = float(maxid - (tropical_end-tropical_start))

    pbara = extras.progress_bar.ProgressBarAlt

    # --------------------------------------------------------------------------
    # Run compile_stats
    # --------------------------------------------------------------------------
    #recover_stat_files = 0
    if recover_stat_files:
        
        # Load pre-saved files (good for quickly tweaking figures)
        #print "\tReading Histogram Data ..."
        try:
            pdump = gzip.open(super_rfile,"rb")
        except IOError:
            print "Error Opening: %s" % (super_rfile)        
        # Retrieve keys
        try:
            the_keys = cPickle.load(pdump)
        except IOError:
            print "Error Reading: %s" % super_rfile
        try:
            pdump.close()
        except IOError:
             print "Error Closing: %s" % super_rfile
        for var in pbara(the_keys,"\tRecovering Histogram Data ... ",60):
        #for var in the_keys:
            spname = sq_fmt % (over_write_out_path,var)  

            if single_variable:
                if var.find(single_variable) != -1: 
                    pass
                elif make_age_plots and var.find("aged-indices") != -1 :
                    pass
                else:
                    continue
            try:
                spfile = gzip.open(spname, 'rb')
            except IOError:
                print "Error Opening: %s" % spname                  
            try:
                pastDataHisto[var] = cPickle.load(spfile)
            except IOError:
                print "Error Reading: %s" % spname                
            try:
                spfile.close()
            except IOError:
                 print "Error Closing: %s" % spname
      
    elif single_processor:
        # This is a single processor version or for debugging...
        print "\tReading Histogram Data ..."
        pbar = years
        try:
            pdump = gzip.open(super_rfile,"wb",-1)
        except IOError:
            print "Error Opening: %s" % (super_rfile)

        for loop_year in pbar:
            print "\n\t\t=============%d=============" % (loop_year)
            what_do = {}
            what_do = {"in_file" : fmt1 % (over_write_out_path,model,loop_year),
                "dtime_file" : dfmt % (out_path,model,loop_year),
                "bkeys" : pastDataHistoBaseKeys,
                "stat_defs" : stat_defs,
                "sg1_fmt" : '%s_%s_%s_%s_%s',
                "min_trk_cnt" : min_trk_cnt,
                "keep_slp" : keep_slp,
                "max_grid_count" : max_grid_count,
                "min_travel" : min_travel,
                "cyclolysis" : cyclolysis,
                "cyclogenesis" : cyclogenesis,
                "seasons" : seasons,
                "hemis" : hemis,
                "intensities" : intensities,
                "super_run" : super_run,
                "loop_year" : loop_year,
                "summary_plots_only" : summary_plots_only,
                "verbose" : verbose,
                "use_usi_intensity" : use_usi_intensity,
                "use_slp_min" : use_slp_min,
                "min_loc" : min_loc
                }
            #result,dtimes = read_histo(what_do)
            result = read_histo(what_do)
            
            # Assemble results
            for var in result.keys():
                if var in pastDataHisto:
                    # Extend
                    old = pastDataHisto[var]
                    old.extend(result[var])
                    pastDataHisto[var] = old
                else:
                    # Create
                    pastDataHisto[var] = result[var]
 
        # Save for recover_stat_files
        
        # Save data as gziped lists... faster and lighter
        for var in pastDataHisto:
            spname = sq_fmt % (over_write_out_path,var)
            try:
                spfile = gzip.open(spname, 'wb')
            except IOError:
                print "Error Opening: %s" % spname
            try:
                cPickle.dump(pastDataHisto[var],spfile)
                #spfile.writelines(pastDataHisto[var])
            except IOError:
                print "Error Writting: %s" % spname
            try:
                spfile.close()
            except IOError:
                print "Error Closing: %s" % spname            
        try:
            cPickle.dump(pastDataHisto.keys(),pdump)
        except IOError:
            print "Error Writing: %s" % (super_rfile)         
               
        # try:
        #     cPickle.dump(pastDataHisto,pdump)
        # except IOError:
        #     print "Error Writing: %s" % (super_rfile)            
        
        # try:
        #     cPickle.dump(pastDataHisto.keys(),pdump)
        # except IOError:
        #     print "Error Writing: %s" % (super_rfile)       
        # try:
        #     for var in pastDataHisto:
        #         cPickle.dump(pastDataHisto[var],pdump)
        # except IOError:
        #     print "Error Writing: %s" % (super_rfile)
        
        try:
            pdump.close()
        except IOError:
             print "Error Closing: %s" % super_rfile

    else:
        # Test for python version with parallel processing capabilities.
        if sys.version_info >= (2, 6):
            import multiprocessing as mp
            import logging
            logger = mp.log_to_stderr()
            logger.setLevel(mp.SUBDEBUG)
            PROCESSORS = mp.cpu_count()
        else:
            print "Parallelism requires python 2.6 or higher."
            sys.exit()
            
        # Don't breakup by less than 1 year per processor
        max_processors = (years[-1]-years[0]+1)
        if PROCESSORS > max_processors:
            PROCESSORS = max_processors

        if PROCESSORS > mp.cpu_count():
            PROCESSORS = mp.cpu_count()

        # Can manually set processors by commenting out below.
        #PROCESSORS = PROCESSORS - 2

        msg = "\n\tStarting %s (process id %s) with %d processors"
        print msg % ('Reading Histogram Data',os.getpid(),PROCESSORS)

        # Spawn n=PROCESSORS processes
        pool = mp.Pool(PROCESSORS)
        results = []
        args_used = []
        checkin = CompletedCallbackPbar
        mp_pbar = extras.progress_bar.ProgressBar(len(range(years[0],years[-1])),"Finished")
        mp_pbar.fill_char = '-'        
        for loop_year in years:
            # Set definitions
            what_do = {}
            what_do = {"in_file" : fmt1 % (over_write_out_path,model,loop_year),
                "dtime_file" : dfmt % (out_path,model,loop_year),
                "bkeys" : pastDataHistoBaseKeys,
                "stat_defs" : stat_defs,
                "sg1_fmt" : '%s_%s_%s_%s_%s',
                "min_trk_cnt" : min_trk_cnt,
                "keep_slp" : keep_slp,
                "max_grid_count" : max_grid_count,
                "min_travel" : min_travel,
                "cyclolysis" : cyclolysis,
                "cyclogenesis" : cyclogenesis,
                "seasons" : seasons,
                "hemis" : hemis,
                "intensities" : intensities,
                "super_run" : super_run,
                "loop_year" : loop_year,
                "summary_plots_only" : summary_plots_only,
                "verbose" : verbose,
                "use_usi_intensity" : use_usi_intensity,
                "use_slp_min" : use_slp_min,
                "min_loc" : min_loc
                }
            args_used.append((what_do))
            #pool.apply_async(read_histo,args_used[-1],callback=checkin)
            #pool.apply_async(read_histo,args_used[-1],callback=results.append)
            pool.apply_async(mp_test,args_used[-1],callback=results.append)
        print logger
        import sys; sys.exit("Stop Here")
        pool.close()
        pool.join()
        print("\n\tFinished:")
        print results
        # Put assemble results. Note this results might not be in
        #   sequential order, but this shouldn't matter the accumulation.
        # Add to database
        for result in results:
            if type(result) is str:
                # Some sort of error occurred.
                print result
                sys.exit()
            print result.keys()
        import sys; sys.exit("Stop Here")    
            # Assemble results
            # for var in result.keys():
                 # if var in pastDataHisto:
                 #     # Extend
                 #     old = pastDataHisto[var]
                 #     old.extend(result[var])
                 #     pastDataHisto[var] = old
                 # else:
                 #    # Create
                 #    pastDataHisto[var] = result[var]
        
        # result,dtimes = read_histo(what_do)
        #  # Assemble results
        #  for var in result.keys():
        #      if var in pastDataHisto:
        #          # Extend
        #          old = pastDataHisto[var]
        #          old.extend(result[var])
        #          pastDataHisto[var] = old
        #      else:
        #          # Create
        #          pastDataHisto[var] = result[var]
                 
             # Need to crate a super pastDataHisto and extend w/ each result.
             #pastTracks.update(tmpTracks)
 # FIX NEED TO accumulate super_dtimes
 # FIX NEED TO save super_dtimes
         # Save for recover_stat_files
         #try:
         #    cPickle.dump(pastDataHisto,gzip.open(super_rfile,"wb",-1))
         #except IOError:
         #    print "Error Writing: %s" % (super_rfile)

    # -------------------------------------------------------------------------
    # Done Reading all years
    # -------------------------------------------------------------------------
    #import sys; sys.exit("skip plot")

    # Create Histograms
#    plot_cmd_1d_histo = ["use_bins_left_edge = bins_left_edge\nuse_bins_centers = bins_centers\nif may_trim:\n\tbmax = pile2.max()\n\tif bmax <= trim_max:\n\t\tuse_bins_left_edge = numpy.arange(trim_min,trim_max,bins_width)\n\t\tuse_bins_centers = use_bins_left_edge + 0.5*bins_width\nn, bins, patches = ax.hist(pile, bins=use_bins_left_edge.tolist(), range=None,normed=normalized, cumulative=0, bottom=None,histtype='bar',align='mid',orientation='vertical', rwidth=None, log=False,facecolor='grey', alpha=0.75)",
#             "if normalized:\n\tstdensity = PDF(use_bins_centers,NMean(pile2),NSTD(pile2))\n\tl = ax.plot(use_bins_centers, stdensity, 'r--', linewidth=1)\n\tif yuppern:\n\t\tax.set_ylim(0.0,yuppern)\nif xupper:\n\t\tax.set_xlim(xlower,xupper)"]
    plot_cmd_1d_histo = [
        "use_bins_left_edge = bins_left_edge\nuse_bins_centers = bins_centers\nif may_trim:\n\tbmax = pile2.max()\n\tif bmax <= trim_max:\n\t\tuse_bins_left_edge = numpy.arange(trim_min,trim_max,bins_width)\n\t\tuse_bins_centers = use_bins_left_edge + 0.5*bins_width\nn, bins, patches = plt.hist(pile, bins=use_bins_left_edge.tolist(), range=None,normed=normalized, cumulative=0, bottom=None,histtype='bar',align='mid',orientation='vertical', rwidth=None, log=False,facecolor='grey', alpha=0.75)",
        # use every N centered bin as a label
        "ax.set_xticks(use_bins_centers[0::binskip])",
        "if normalized:\n\tstdensity = PDF(use_bins_left_edge,NMean(pile2),NSTD(pile2))\n\tl = plt.plot(use_bins_left_edge, stdensity, 'r--', linewidth=1)\nif yuppern:\n\tax.set_ylim(0.0,yuppern)\nif xupper:\n\tax.set_xlim(xlower,xupper)",
        # Make y-axis in percentage if normalized.
        "if normalized:\n\tax.yaxis.set_major_formatter(ticker.FuncFormatter(lambda y, pos: ('%.2f')%(y*nscale)))"
        ]
    #print
    #for ikn in  plot_cmd_1d_histo: print ikn,"\n"        
    
# FIX should be able to make this a function and run in parallel as well.

    # Process 2d Histograms
    gridsize = int(age_len)
    xmin = 0
    xmax = age_len
    agedkeys = [x for x in pastDataHisto if x.split("_")[0] == 'aged-indices']
    agedkeys = sorted(agedkeys)
    if make_age_plots and agedkeys:
        for xvar in pbara(agedkeys,"\tPlotting Age Based Histograms ... ",60):
            parts = xvar.split("_") 
            the_end = xvar.replace('aged-indices_','')
            if xvar.find('intensity') != -1 and len(intensities) > 1 : 
                continue
            # Skip plots with no values
            if not pastDataHisto[xvar]: continue
            
            for yvar in stat_defs['cds_age_pairs']:
                the_key = xvar.replace('aged-indices',yvar[1])
                if single_variable:
                    if the_key.find(single_variable) == -1: continue
                stmp = "%s/%s/" % (parts[2],parts[1])
                tmp = "%sfigs/pdfs/%s%s_histoAge_%s_%s"
                pname_base = tmp % (out_path,stmp,model,yvar[1],the_end)
                if cyclolysis:
                    pname_base = pname_base+"_cyclolysis"
                elif cyclogenesis:
                    pname_base = pname_base+"_cyclogenesis"
                if use_usi_intensity:
                    pname_base = pname_base+"_pinten"
                if use_slp_min:
                    pname_base = pname_base+"_slpmin"
                # Trim zero or missing values...but keep 'aged-indices'
                #   lineed up.
                x_dat = []
                y_dat = []
                if yvar[1] not in ["fill","lats"]:
                    if yvar[1] in ['asymmetry','amp-asymmetry']:
                        idx = 0
                        for yd in pastDataHisto[the_key]:
                            if float(yd) >= -1.0:
                                y_dat.append(yd)
                                x_dat.append(pastDataHisto[xvar][idx])
                            idx += 1
                    else:
                        idx = 0
                        for yd in pastDataHisto[the_key]:
                            if float(yd) > 0.0:
                                y_dat.append(yd)
                                x_dat.append(pastDataHisto[xvar][idx])
                            idx += 1
                else:
                    x_dat = pastDataHisto[xvar]
                    y_dat = pastDataHisto[the_key]

                 ##if yvar[1] not in ["slp"]:
                 ##    continue

                # Plot plot it
                pname = '%s%s' % (pname_base,fig_format)
                bname = pname.replace("histoAge","boxAge")

                # For BoxPlot need to create an array of bins by age_index
                bbins = defaultdict(list)
                for idat in xrange(len(y_dat)):
                    bbins[int(x_dat[idat])].append(y_dat[idat])
                box_data=[]
                box_dates=[]
                box_index=[]
                xbox = []
                ybox =[]
                used_bins = 1
                medians=[]
                averages=[]
                stds=[]
                med_index=[]
                counts = []
                for idat in xrange(age_len):
                    if idat < min_age_index: continue
                    if idat > max_age_index: continue
                    if idat in bbins:
                        medians.append(Median(bbins[idat]))
                        averages.append(NAve(bbins[idat]))
                        stds.append(NSTD(bbins[idat]))
                        counts.append(len(bbins[idat]))
                        med_index.append(idat)
                        box_data.append(bbins[idat])
                        box_index.append(used_bins)
                        ybox.extend(bbins[idat])
                        xbox.extend([idat]*len(bbins[idat]))
                        box_dates.append(age_hours[idat])
                        used_bins +=1
                    else:
                        # Add missing data to bins
                        medians.append(numpy.nan)
                        averages.append(numpy.nan)
                        stds.append(numpy.nan)
                        counts.append(numpy.nan)
                        med_index.append(idat)
                        box_data.append([numpy.nan])
                        box_index.append(used_bins)
                        ybox.extend([numpy.nan])
                        xbox.extend([idat])
                        box_dates.append(age_hours[idat]) 
                        used_bins +=1

                golden_ratio = 1.61803399*2.0
                #width = 40.0
                #hieght = width/golden_ratio
                hieght = 10.0
                width = 11.0
                if make_box_plots:
                    # BoxPlots
                    #------------------------------
                    fig = plt.figure(figsize=(width,hieght))
                    ax = fig.add_subplot(111)
                    bp = plt.boxplot(box_data, notch=0, sym='', vert=1, whis=1.5)
                    plt.setp(bp['boxes'], color='black')
                    plt.setp(bp['whiskers'], color='black')
                    plt.setp(bp['medians'], color='black',lw=2)
                     #plt.setp(bp['medians'], color='black',lw=4)
                     #plt.setp(bp['fliers'], color='black', marker='')
            
                     # Now fill the boxes with desired colors
                    boxColors = ['0.85','1']
                    numBoxes = len(bp['medians'])
                    medians2 = range(numBoxes)
                    for i in range(numBoxes):
                        box = bp['boxes'][i]
                        boxX = []
                        boxY = []
                        for j in range(5):
                            boxX.append(box.get_xdata()[j])
                            boxY.append(box.get_ydata()[j])
                        boxCoords = zip(boxX,boxY)
                        # Alternate between Dark Khaki and Royal Blue
                        k = i % 2
                        boxPolygon = Polygon(boxCoords,facecolor=boxColors[k])
                        #boxPolygon = Polygon(boxCoords,facecolor=boxColors[k],hatch='/')
                        ax.add_patch(boxPolygon)
                        # Now draw the median lines back over what we just filled in
                        med = bp['medians'][i]
                        medianX = []
                        medianY = []
                        for j in range(2):
                            medianX.append(med.get_xdata()[j])
                            medianY.append(med.get_ydata()[j])
                            plt.plot(medianX, medianY, 'k')
                            medians2[i] = medianY[0]
                    # Finally, overplot the sample averages, with horixzontal alignment
                    # in the center of each box
                    boxcenter = [NAve(bp['medians'][i].get_xdata()) for i in xrange(len(bp['medians']))]
                    # Draw as line
                    #plt.plot(boxcenter,averages,'r.-',ms=1,linewidth=3)
                    # Draw as stars
                    for i in range(numBoxes):
                        #print i,boxcenter[i],averages[i]
                        #plt.plot(boxcenter[i],averages[i],color='w',marker='*',markeredgecolor='k',ms=15)
                        plt.plot(boxcenter[i],averages[i],color='w',marker='*',markeredgecolor='k',ms=10)
                    # Thin light grey line along y-axis
                    ax.yaxis.grid(True,linestyle='-',which='major',color='lightgrey',alpha=0.5)
                    # vertical line over 0-hour
                    #plt.axvline(x=box_dates.index(0)+1,linestyle='-',color='lightgrey')#,alpha=0.75)
                    # Hide these grid behind plot objects
                    ax.set_axisbelow(True)
                    # Change Labels
                    #xmin1, xmax1 = plt.xlim()
                    #plt.xlim(xmin=xmin1-1,xmax=xmax1+1)
                    # Set font size
                    for labeltick in ax.xaxis.get_majorticklabels():
                        labeltick.set_fontsize(11)
                    locs, labels = plt.xticks()
                    plt.xticks(box_index,box_dates)
                    # Axis & Title
                    ax.set_xlabel('Hours Relative to Lifetime Minimum SLP')
                    ax.set_ylabel(age_pair_label[yvar[1]])
                    ax.set_title(age_pair_label['title'])
            
                    fig.savefig(bname,dpi=144)
                    plt.clf()
                    plt.close('all')
                    if verbose: print "\t\t\tMade ",bname

                # For Line plots with error bars
                #------------------------------
                if not make_box_plots:
                    bp = plt.boxplot(box_data, notch=0, sym='', vert=1, whis=1.5)
                    numBoxes = len(bp['medians'])
                cname = pname.replace("histoAge","lineAge")
                fig = plt.figure(figsize=(width,hieght))
                ax = fig.add_subplot(111)
                std_error = []
                for i in xrange(numBoxes):
                    if counts[i]:
                        std_error.append(2.0*(stds[i]/numpy.sqrt(counts[i])))
                    else:
                        std_error.append(numpy.nan)
                plt.errorbar(box_index,averages,yerr=std_error,marker="o",ms=5,color='k',
                    ecolor='k',linestyle="-",elinewidth=1,lw=2,capsize=5)
                # Thin light grey line along y-axis
                #ax.grid()
                #ax.yaxis.grid(True,linestyle='-',which='major',color='lightgrey',alpha=0.5)
                # Hide these grid behind plot objects
                #
                xmin1, xmax1 = plt.xlim()
                plt.xlim(xmax=xmax1+1)
                # vertical line over 0-hour
                plt.axvline(x=box_dates.index(0)+1,linestyle='-',color='k',alpha=0.75)

                # Change Labels
                # Set font size
                for labeltick in ax.xaxis.get_majorticklabels():
                    labeltick.set_fontsize(11)
                # Alter label
                locs, labels = plt.xticks()
                plt.xticks(box_index,box_dates)
                # Axis & Title
                ax.set_xlabel('Hours Relative to Lifetime Minimum SLP',verticalalignment='top')
                ax.set_ylabel(age_pair_label[yvar[1]])
                ax.set_title(age_pair_label['title'])

                fig.savefig(cname,dpi=144)
                plt.clf()
                plt.close('all')
                if verbose:  print "\t\t\tMade ",cname

            # # For 2d histograms
            ##------------------------------
            # #x = numpy.array(x_dat)
            # #y = numpy.array(y_dat)
            # x = numpy.array(xbox)
            # y = numpy.array(ybox)
            # ymin = numpy.nanmin(y)
            # ymax = numpy.nanmax(y)
            # xmin = x.min()
            # xmax = x.max()
            # xmin = xmin - 1
            # xmax = xmax + 1
            # gridsize = int(xmax-xmin)
            # #golden_ratio = 1.61803399*2.0
            # #width = 40.0
            ##hieght = width/golden_ratio
             # box_index = range(min_age_index,max_age_index+1)
             # box_dates = [age_hours[xx] for xx in box_index]
             # for bintype in [None,'log']:
             #     fig = plt.figure(figsize=(width,hieght))
             #     ax = fig.add_subplot(111)
             #     plt.plot(med_index,medians,'r.-',linewidth=3)
             #     if bintype == 'log':
             #         pname = pname.replace("histoAge","histoAge_log")
             #         plt.hexbin(x,y,cmap=cm.bone_r,bins='log',gridsize=gridsize)
             #     else:
             #         plt.hexbin(x,y,cmap=cm.bone_r,gridsize=gridsize)
             #     #plt.hexbin(x,y,cmap=cm.bone_r,gridsize=gridsize)
             #     ##plt.hexbin(x_dat,y_dat,cmap=cm.bone_r,gridsize=gridsize,bins='log')
             #     ##plt.hexbin(x_dat,y_dat,cmap=cm.hot_r,marginals=True)
             #     plt.axis([xmin,xmax,ymin,ymax])
             #     plt.xticks(box_index,box_dates)
             #     fig.savefig(pname,dpi=144)
             #     plt.close('all')
             #     print "\t\t\tMade ",pname

    #          #import sys; sys.exit("Stop Here")

    # Trim memory
    for xvar in agedkeys:
        del pastDataHisto[xvar]

    nekey = sorted(pastDataHisto.keys())
    # Process 1d Histograms
    #for var in pastDataHisto:
    for var in pbara(nekey,"\tPlotting 1D Histograms ... ",60):
        parts = var.split("_")
        the_key = parts[0]
        if the_key.find('aged-indices') != -1: continue
        
        if the_key.find('intensity') != -1 and len(intensities) > 1 : 
            continue
        if single_variable:
            if the_key.find(single_variable) == -1: continue

        draw_script = histodo.histo_stuff[the_key]

        # Adjust bins_left_edge latitude based bins and hemisphere states
        if the_key == "lats":
            if parts[2] == "sh":
                ds = [x for x in draw_script.split(";") if x.find("bins_left_edge") != -1][0]
                lmin = ds.split("=")[1]
                lmin = lmin.split("(")[1]
                lmin = lmin.split(",")[0]
                draw_script = draw_script.replace(ds.split("=")[1]," numpy.arange(%s,0.0,bin_width)" % lmin)
                draw_script = draw_script.replace("xupper = 90.000000","xupper = 0.000000")
            if parts[2] == "nh":
                ds = [x for x in draw_script.split(";") if x.find("bins_left_edge") != -1][0]
                lmax = ds.split("=")[1]
                lmax = lmax.split("(")[1]
                lmax = lmax.split(",")[1]
                draw_script = draw_script.replace(ds.split("=")[1]," numpy.arange(0.0,%s,bin_width)" % lmax)
                draw_script = draw_script.replace("xlower = -90.000000","xlower = 0.000000")

        stmp = "%s/%s/" % (parts[2],parts[1])
        if the_key in stat_defs["per_step_group"]:
            tmp = "%sfigs/pdfs/%s%s_histo_per_step_%s"
        else:
            tmp = "%sfigs/pdfs/%s%s_histo_%s"
        pname_base = tmp % (out_path,stmp,model,var)

        if cyclolysis:
            pname_base = pname_base+"_cyclolysis"
        if cyclogenesis:
            pname_base = pname_base+"_cyclogenesis"
        if use_usi_intensity:
            pname_base = pname_base+"_pinten"
        if use_slp_min:
            pname_base = pname_base+"_slpmin"
         #pname_base = pname_base+"_long"

        if the_key == "bearing":
            #rp_psrose:
            continue
        pcd = plot_cmd_1d_histo

        report_save.write(report_fmts[0].format(the_key,parts[1].upper(),
            parts[3].upper(), parts[2].upper()))

        # Skip plots with no values
        if not pastDataHisto[var]: 
            report_save.write(report_fmts[1].format(-999,-999,-999,-999,-999,-999,
                -999,-999,-999,-999,-999))
            continue

        # Trim zero or missing values...
        if the_key not in ["fill","lats"]:
            if the_key in ['asymmetry','amp-asymmetry']:
                tmp_dat = [x for x in pastDataHisto[var] if float(x) >= -1.0]
            else:
                tmp_dat = [x for x in pastDataHisto[var] if float(x) > 0.0]
        else:
            tmp_dat = pastDataHisto[var]

        msg = plot_stats(tmp_dat,[],pname_base,pcd,draw_script,
            fig_format=fig_format,no_stats=no_stats,no_title=no_title)
        if verbose: print "\t\t\t"+msg[0]

        # Test for nothing
        if isinstance(msg,str):
            #report_save.write(msg)
            continue

        report_save.write(report_fmts[1].format(msg[1]["Count"],msg[1]["Min"],
            msg[1]["Max"],msg[1]["Mean"],msg[1]["Median"],msg[1]["Mode"],
            msg[1]["ModeAlt"],msg[1]["STD"],msg[1]["Skew"],msg[1]["Kurt"],
            msg[1]["per5"],msg[1]["per95"]))
        
        # Save data?

 #ADD end

    import sys; sys.exit("Stop Here")

    #then I need a way to read and compile (see below) data for
    #1dhisto, timeseries, matched timeseries (2d scatter, histo) and lifecycle
    #plots and get it all to matplot lib I should just make the ploting
    #part of DefineHistogramStuff and DefineHistogramStuff







#                        * regular large lists for histograms as above
#                        * 2d paired lists for scatter plots or 2d histograms
#                        * track centered plots centered on min_loc (scatter plots)
#                            use mp_age_len to make collection bins and fill based
#                            on aged-indices where the lifetime slp min is at index
#                            mp_min_loc which should be the center of the collection
#                            bins.

    # open saved file.



