from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports

# Third party module imports
import numpy
import copy 

# MCMS module imports
from setup_bins import setup_bins

def setup_track_stats(tf,troubled_centers):

    # insert quantity info:
    # 0 - CisG
    # 1 - StheC
    # 2 - CisB
    # 3 - Dissimilarity Scores
    # 4 - Bearing
    # 5 - Ccount
    # 6 - Dscore used (selected) when dscore used at all
    # 7 - Dscore not used (unselected values)  when dscore used at all
    stat_groups = ["CisG","StheC","CisB","Dscore","Bearing",
                   "Ccount","Dscore_Used","Dscore_Unused"]
    # injested centers
    readin_centers =  numpy.zeros((tf.jm*tf.im),dtype=numpy.float)

    # For frequency plots
    data_index = [len(stat_groups)]
    data_index.append(tf.jm*tf.im)
    bucket_freq_sum = numpy.zeros(data_index,dtype=numpy.float)
    bucket_freq_cnt = numpy.zeros(data_index,dtype=numpy.float)
  
    probs = []
    if troubled_centers:
        polar = [x for x in xrange(0,tf.pbot+1)]
        polar.extend([x for x in xrange(tf.ptop,tf.jm*tf.im+1)])
        # Add polar to extended troubled
        probs = troubled_centers[:]
        #probs.extend(polar)

    big_bins = [x for x in range(len(stat_groups))]
    # Special prob version
    big_bins_no_prob = [x for x in range(len(stat_groups))]
    big_bins_prob = [x for x in range(len(stat_groups))]


    # CisG bins for histograms
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,5.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    CisG_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[0] = CisG_bins
    if probs:
        CisG_bins_no_prob = copy.deepcopy(CisG_bins)
        big_bins_no_prob[0] = CisG_bins_no_prob
        CisG_bins_prob = copy.deepcopy(CisG_bins)
        big_bins_prob[0] = CisG_bins_prob     

    # StheC bins for histograms
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,2.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    StheC_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[1] = StheC_bins
    if probs:
        StheC_bins_no_prob = copy.deepcopy(StheC_bins)
        big_bins_no_prob[1] = StheC_bins_no_prob
        StheC_bins_prob = copy.deepcopy(StheC_bins)
        big_bins_prob[1] = StheC_bins_prob    

    # CisB bins for histograms
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,1.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    CisB_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[2] = CisB_bins
    if probs:
        CisB_bins_no_prob = copy.deepcopy(CisB_bins)
        big_bins_no_prob[2] = CisB_bins_no_prob
        CisB_bins_prob = copy.deepcopy(CisB_bins)
        big_bins_prob[2] = CisB_bins_prob    

    # Dscore bins for histograms
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,5.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    Dscore_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[3] = Dscore_bins
    if probs:
        Dscore_bins_no_prob = copy.deepcopy(Dscore_bins)
        big_bins_no_prob[3] = Dscore_bins_no_prob
        Dscore_bins_prob = copy.deepcopy(Dscore_bins)
        big_bins_prob[3] = Dscore_bins_prob 
        
    # Bearing bins for histograms
    bin_width = 5.0#45.0
    bins_left_edge = numpy.arange(0.0,360.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    bearing_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[4] = bearing_bins
    if probs:
        bearing_bins_no_prob = copy.deepcopy(bearing_bins)
        big_bins_no_prob[4] = bearing_bins_no_prob
        bearing_bins_prob = copy.deepcopy(bearing_bins)
        big_bins_prob[4] = bearing_bins_prob 

    # Ccount bins for histograms (number of potential connections)
    bin_width = 1.0
    bins_left_edge = numpy.arange(0.0,10.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    Ccount_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                 tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[5] = Ccount_bins
    if probs:
        Ccount_bins_no_prob = copy.deepcopy(Ccount_bins)
        big_bins_no_prob[5] = Ccount_bins_no_prob
        Ccount_bins_prob = copy.deepcopy(Ccount_bins)
        big_bins_prob[5] = Ccount_bins_prob 

    # Dscore_Used bins for histograms (these of the dscores used to make choice)
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,5.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    Dscore_Used_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                   tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[6] = Dscore_Used_bins
    if probs:
        Dscore_Used_bins_no_prob = copy.deepcopy(Dscore_Used_bins)
        big_bins_no_prob[6] = Dscore_Used_bins_no_prob
        Dscore_Used_bins_prob = copy.deepcopy(Dscore_Used_bins)
        big_bins_prob[6] = Dscore_Used_bins_prob 
         
    # Dscore_Unused bins for histograms (these are the pool of dscores when a choice is made)
    bin_width = 0.1
    bins_left_edge = numpy.arange(0.0,5.0,bin_width)
    tmp = setup_bins(bins_left_edge)
    Dscore_Unused_bins = [numpy.zeros(len(bins_left_edge),dtype=numpy.integer),
                   tmp[0],tmp[1],tmp[2],tmp[3]]
    big_bins[7] = Dscore_Unused_bins
    if probs:
        Dscore_Unused_bins_no_prob = copy.deepcopy(Dscore_Unused_bins)
        big_bins_no_prob[7] = Dscore_Unused_bins_no_prob
        Dscore_Unused_bins_prob = copy.deepcopy(Dscore_Unused_bins)
        big_bins_prob[7] = Dscore_Unused_bins_prob 

    inputs = ("readin_centers","stat_groups","bucket_freq_sum","bucket_freq_cnt","probs",
            "big_bins","big_bins_no_prob","big_bins_prob","CisG_bins","CisG_bins_no_prob",
            "CisG_bins_prob","StheC_bins","StheC_bins_no_prob","StheC_bins_prob",
            "CisB_bins","CisB_bins_no_prob","CisB_bins_prob","Dscore_bins",
            "Dscore_bins_no_prob","Dscore_bins_prob","bearing_bins",
            "bearing_bins_no_prob","bearing_bins_prob","Ccount_bins",
            "Ccount_bins_no_prob","Ccount_bins_prob","Dscore_Used_bins",
            "Dscore_Used_bins_no_prob","Dscore_Used_bins_prob","Dscore_Unused_bins",
            "Dscore_Unused_bins_no_prob","Dscore_Unused_bins_prob")
    return inputs,(readin_centers,stat_groups,bucket_freq_sum,bucket_freq_cnt,probs,
            big_bins,big_bins_no_prob,big_bins_prob,CisG_bins,CisG_bins_no_prob,
            CisG_bins_prob,StheC_bins,StheC_bins_no_prob,StheC_bins_prob,
            CisB_bins,CisB_bins_no_prob,CisB_bins_prob,Dscore_bins,
            Dscore_bins_no_prob,Dscore_bins_prob,bearing_bins,
            bearing_bins_no_prob,bearing_bins_prob,Ccount_bins,
            Ccount_bins_no_prob,Ccount_bins_prob,Dscore_Used_bins,
            Dscore_Used_bins_no_prob,Dscore_Used_bins_prob,Dscore_Unused_bins,
            Dscore_Unused_bins_no_prob,Dscore_Unused_bins_prob)
