#!/usr/bin/env python -tt
"""This module pre-calculate things for mcms_track_finder.py

Options/Arguments:

Returns/Creates:

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard modules

# Third party modules

# MCMS modules
from set_search_radius import set_search_radius
from fill_search import fill_search
from inflate_gdict import inflate_gdict
#from print_gdict import PrintGdict
#from shift_gridid import shift_gridid

def setup_tracking(defs,ijdict,defs_grid,rf):
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------

    # Pre-bind for speed
    jm = defs_grid["jm"]
    im = defs_grid["im"]
    timestep = defs_grid["timestep"]
    travel_distance = defs.max_cyclone_speed*timestep

    # Find latitudes where all longitudes fit within travel_distance. Not
    # the same as for center_finder as this is for reasonable cyclone movement
    # not reasonable low pressure proximity.
    rname = "%stravel_radius_report.txt" % rf.shared_path
    verbose_here = 0
    fnc = set_search_radius(defs_grid,defs,rf,travel_distance,rname,
        verbose_here)
    use_all_lons,search_radius,regional_nys,sr_extra = fnc

    # Enumerate the model grid (GridID) and then make a dictionary with the 8
    #   grid points around each GridID ... adapting for polar and longitude
    #   wrap around.
    gdict = inflate_gdict(im,jm)

    verbose_here = 0
    # define dictionary of travel_distance grids
    tdict = fill_search(defs_grid,defs,rf,regional_nys,search_radius,
        sr_extra,gdict,ijdict,use_all_lons,verbose_here)
    
    return tdict

