# Standard modules
from random import choice

# Third party module imports
    
# MCMS modules
import util.write_mcms

def sort_rules_reducible(s_data,tc,sr_verbose=0):
    """Apply Sort Rule with WALL members of a case set are equivalent"""
    form_lon = util.write_mcms.form_lon
    unform_lon = util.write_mcms.unform_lon
    form_colat = util.write_mcms.form_colat
    unform_colat = util.write_mcms.unform_colat
    tabs = 5
    retained = []
    set_aside = []
    trouble = []
    if tc:
        # List of centers in s_data that contain troubled centers
        testc = s_data.keys()[0]
        h = (0 if s_data[testc]["CoLat"] > 9000 else 1)
        trouble = [x for x in s_data if x in tc[h]]
        if sr_verbose:
            print "%sTroubles (%d): %s" % ("\t"*tabs,len(trouble),trouble)
        # If all centers troubled then just treat as regular centers as there
        #   is not preference beyond the selection rules used here.
        if len(trouble) == len(s_data):
            trouble = []
    #---------------------------------------------------------------------------
    # Try 1: Judge by lowest central SLP
    #---------------------------------------------------------------------------
    if trouble:
        sort_by_cslp = sorted([s_data[x]['GridSLP'] for x in s_data
            if x not in trouble])
        if sr_verbose:
            print "%sfull sort_by_cslp (%d): %s" % ("\t"*tabs,
                len([s_data[x]['GridSLP'] for x in s_data]),
                sorted([s_data[x]['GridSLP'] for x in s_data]))
            print "%strimmed sort_by_cslp (%d): %s" % ("\t"*tabs,
                len(sort_by_cslp),sort_by_cslp),
    else:
        sort_by_cslp = sorted([s_data[x]['GridSLP'] for x in s_data])
        if sr_verbose:
            print "%ssort_by_cslp (%d): %s" % ("\t"*tabs,len(sort_by_cslp),
                sort_by_cslp),
    if len([x for x in sort_by_cslp if x == sort_by_cslp[0]]) == 1:
        # Single minima
        set_aside = ['GridSLP',sort_by_cslp[0]]
        ties = [x for x in s_data if s_data[x]['GridSLP'] == sort_by_cslp[0]]
        if sr_verbose: print
    else:
        #-----------------------------------------------------------------------
        # Try 2: Judge by highest Laplacian
        #        Among only the centers sharing lowest central SLP, note this
        #        is skipped for centers at high latitude where Laplacian not
        #        calculated.
        #-----------------------------------------------------------------------
        if trouble:
            ties = [x for x in s_data if s_data[x]['GridSLP'] == sort_by_cslp[0] and
            x not in trouble]
            sort_by_lap = sorted([s_data[x]['GridLAP'] for x in ties])
            if sr_verbose:
                tties = [x for x in s_data if s_data[x]['GridSLP'] == sort_by_cslp[0]]
                ssort_by_lap = sorted([s_data[x]['GridLAP'] for x in tties])
                print "Tie!\n%sfull sort_by_lap (%d):%s" % ("\t"*(tabs+1),
                    len(ssort_by_lap),ssort_by_lap)
                print "%strimmed sort_by_lap (%d): %s" % ("\t"*(tabs+1),
                    len(sort_by_lap),sort_by_lap),
        else:
            ties = [x for x in s_data if s_data[x]['GridSLP'] == sort_by_cslp[0]]
            sort_by_lap = sorted([s_data[x]['GridLAP'] for x in ties])
            if sr_verbose:
                print "Tie!\n%ssort_by_lap (%d): %s" % ("\t"*(tabs+1),
                    len(sort_by_lap),sort_by_lap),
        if len([x for x in sort_by_lap if x == sort_by_lap[-1]]) == 1:
            set_aside = ['GridLAP',sort_by_lap[-1]]
            if sr_verbose: print
        else:
            #-------------------------------------------------------------------
            # Try 3: Judge by lowest Average SLP
            #        Among only the centers sharing lowest central SLP and
            #        highest highest Laplacian.
            #-------------------------------------------------------------------
            if trouble:
                ties = [x for x in ties if s_data[x]['GridLAP'] == sort_by_lap[-1] and
                x not in trouble]
                sort_by_aslp = sorted([s_data[x]['RegSLP'] for x in ties])
                if sr_verbose:
                    tties = [x for x in tties if s_data[x]['GridLAP'] == ssort_by_lap[-1]]
                    ssort_by_aslp = sorted([s_data[x]['RegSLP'] for x in tties])
                    print "Tie!\n%sfull sort_by_aslp (%d): %s" % ("\t"*(tabs+2),
                        len(ssort_by_aslp),ssort_by_aslp)
                    print "%strimmed sort_by_aslp (%d): %s" % ("\t"*(tabs+2),
                        len(sort_by_aslp),sort_by_aslp),
            else:
                ties = [x for x in ties if s_data[x]['GridLAP'] == sort_by_lap[-1]]
                sort_by_aslp = sorted([s_data[x]['RegSLP'] for x in ties])
                if sr_verbose: print "Tie!\n%ssort_by_aslp (%d): %s" % (
                    "\t"*(tabs+2),len(sort_by_aslp),sort_by_aslp),
            if len([x for x in sort_by_aslp if x == sort_by_aslp[0]]) == 1:
                set_aside = ['RegSLP',sort_by_aslp[0]]
                if sr_verbose: print
            else:
                #---------------------------------------------------------------
                # Try 4: Release the Fail Safe! Judge by lowest abs latitude.
                #        Among only the centers sharing lowest central SLP,
                #        highest highest Laplacian and lowest Average SLP.
                #---------------------------------------------------------------
                if trouble:
                    ties = [x for x in ties
                        if s_data[x]['RegSLP'] == sort_by_aslp[0] and
                        x not in trouble]
                    sort_by_lat = sorted([abs(9000-s_data[x]['CoLat'])
                        for x in ties])
                    if sr_verbose:
                        tties = [x for x in tties
                            if s_data[x]['RegSLP'] == ssort_by_aslp[0]]
                        ssort_by_lat = sorted([abs(9000-s_data[x]['CoLat'])
                            for x in tties])
                        print "Tie!\n%sfull sort_by_lat (%d): %s" % (
                            "\t"*(tabs+3),len(ssort_by_lat),ssort_by_lat)
                        print "%sfull sort_by_lat (%d): %s" % ("\t"*(tabs+3),
                            len(sort_by_lat),sort_by_lat),
                else:
                    ties = [x for x in ties
                        if s_data[x]['RegSLP'] == sort_by_aslp[0]]
                    sort_by_lat = sorted([abs(9000-s_data[x]['CoLat'])
                        for x in ties])
                    if sr_verbose:
                        print "Tie!\n%ssort_by_lat (%d): %s" % ("\t"*(tabs+3),
                            len(sort_by_lat),sort_by_lat),
                if len([x for x in sort_by_lat if x == sort_by_lat[0]]) == 1:
                    set_aside = ['CoLat',sort_by_lat[0]]
                    if sr_verbose: print
                else:
                    #-----------------------------------------------------------
                    # Try 5: Honestly, what's going on here?
                    #-----------------------------------------------------------
                    if trouble:
                        ties = [x for x in ties
                            if abs(9000-s_data[x]['CoLat']) == sort_by_lat[0]
                            and x not in trouble]
                        sort_by_lon = sorted([s_data[x]['Lon'] for x in ties])
                        if sr_verbose:
                            tties = [x for x in tties
                                if abs(9000-s_data[x]['CoLat']) == ssort_by_lat[0]]
                            ssort_by_lon = sorted([s_data[x]['Lon'] for x in ties])
                            print "Tie!\n%sfull sort_by_lon (%d): %s" % (
                                "\t"*(tabs+4),len(ssort_by_lon),ssort_by_lon)
                            print "%strimmed sort_by_lon (%d): %s" % (
                                "\t"*(tabs+4),len(sort_by_lon),sort_by_lon),
                    else:
                        ties = [x for x in ties
                            if abs(9000-s_data[x]['CoLat']) == sort_by_lat[0]]
                        sort_by_lon = sorted([s_data[x]['Lon'] for x in ties])
                        if sr_verbose:
                            print "Tie!\n%ssort_by_lon (%d): %s" % (
                                "\t"*(tabs+4),len(sort_by_lon),sort_by_lon),
                    # If the number of such overly equal grids is too large
                    # assume this isn't really something to retain and send
                    # out a special flag and dicard the whole caseset.
                    if len(ties) > 10:
                        #if sr_verbose:
                        #    print "\n%sOverly Tied... discard whole thing" % (
                        #        "\t"*(tabs+4))
                        #return 2
                        # Other option to discard is to retain one from the case set
                        #   at random
                        sort_by_lon = [choice(sort_by_lon)]  
                        if sr_verbose:
                            print "\n%sOverly Tied... random member retained" % (
                                "\t"*(tabs+4))
                            print "\n%ssort_by_lon (%d): %s" % (
                                "\t"*(tabs+5),len(sort_by_lon),sort_by_lon)
                    if sort_by_lon[-1] <= 18000:
                        use_lon = sort_by_lon[0]
                    else:
                        use_lon = sort_by_lon[-1]
                    if len([x for x in sort_by_lon if x == use_lon]) == 1:
                        set_aside = ['Lon',use_lon]
                        if sr_verbose: print
                    else:
                        #-----------------------------------------------------------
                        # Try 6: Fail!
                        #-----------------------------------------------------------
                        print "WARNING: sort_rules_reducible retained nothing"
                        return retained
    #---------------------------------------------------------------------------
    # Extract Retained center
    #---------------------------------------------------------------------------
    if set_aside[0] == 'CoLat':
        # Need to fine the matching CoLat given sort_by_lat is ABS(lat) so
        #   possible have to -1x to get correct hemisphere.
        if trouble:
            nhemi_match = [x for x in ties
                if s_data[x][set_aside[0]] == 9000-set_aside[1]
                and x not in trouble]
            shemi_match =[x for x in ties
                if s_data[x][set_aside[0]] == 9000+set_aside[1]
                and x not in trouble]
        else:
            nhemi_match = [x for x in ties
                if s_data[x][set_aside[0]] == 9000-set_aside[1]]
            shemi_match =[x for x in ties
                 if s_data[x][set_aside[0]] == 9000+set_aside[1]]
        if nhemi_match and shemi_match:
            print "WARNING: sort_rules_reducible CoLat problem"
            print "nhemi_match",nhemi_match," shemi_match",shemi_match
            retained = []
        elif nhemi_match:
            retained = nhemi_match
        elif shemi_match:
            retained = shemi_match
        else:
            print "WARNING: sort_rules_reducible empty CoLat problem"
    else:
        if trouble:
            retained = [x for x in ties
                if s_data[x][set_aside[0]] == set_aside[1] and
                x not in trouble]
        else:
            retained = [x for x in ties
                if s_data[x][set_aside[0]] == set_aside[1]]
    if not retained:
        print "WARNING: sort_rules_reducible retained nothing"
    elif len(retained) > 1:
        print "WARNING: sort_rules_reducible retained multiple centers"
        print "%sretained: %s" % ("\t"*(tabs+0),retained)
        print "%sset_aside: %s" % ("\t"*(tabs+0),set_aside)
        print "%ss_data: %s" % ("\t"*(tabs+0),s_data)
        retained = []
    if trouble:
        if set(retained) == set(trouble):
            print "WARNING: sort_rules_reducible retained troubled center"
            print "%sretained: %s" % ("\t"*(tabs+0),retained)
            print "%sset_aside: %s" % ("\t"*(tabs+0),set_aside)
            print "%ss_data: %s" % ("\t"*(tabs+0),s_data)
            print "%strouble: %s" % ("\t"*(tabs+0),trouble)
            retained = []
    #retained = [s_data[x] for x in s_data
    #    if s_data[x][set_aside[0]] == set_aside[1]][0]
    if sr_verbose: print "%sretained: %s" % ("\t"*(tabs+0),retained)
    return retained

