from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

def summary_report(out_path,model,super_years):
    """Compiles a summary report for all years"""
    total_cnt = 0
    candidates = 0
    lap_cnt = 0
    reg_cnt = 0
    rad_cnt = 0
    trb_cnt = 0
    pol_cnt = 0
    verbose = 0
    
    def dumpline(the_line,the_cnt,the_candidates,verbose=0):
        msg = the_line.split(" ")
        # Remove any empty strings....
        msg = [x for x in msg if x]
        a = '%10d\t(' % (the_cnt)
        msg[1] = a
        if the_candidates > 0:
            fraction = 100.0*(the_cnt/the_candidates)
        else:
            fraction = 0.0
        f = "%6.2f%%) Failed" % (fraction)
        msg[2] = f
        smsg =  " ".join(msg)
        return smsg
    
    report_file = "%sstats/mcms_%s_center_final_report_%04d-%04d.txt"
    report_file = report_file % (out_path,model,
        int(super_years[0]),int(super_years[-1]))
    print "\tCreating",report_file
    try:
        report_save = open(report_file,"w")
    except:
        print "\tError opening: %s" % (report_file)
        return 1
    
    report_save.writelines("%04d--%04d\n" % (int(super_years[0]),
        int(super_years[-1])))
    
    big_buffer = {}
    # Read partial reports
    iyear = 0
    fmt = "%sstats/mcms_%s_%04d_centers_report.txt"
    for loop_year in xrange(int(super_years[0]),int(super_years[-1])+1):
        r_buffer = []
        r_file = fmt % (out_path,model,loop_year)
        if verbose:
            print "\tAdding",r_file
        try:
            r_read = open(r_file,"r")
        except:
            print "\tError opening: %s" % (r_file)
            return 1
        for line in r_read:
            r_buffer.append(line)
        r_read.close()
        big_buffer[iyear] = r_buffer
        iyear += 1
    
    # Compile statistics
    iyears = iyear
    for iyear in xrange(iyears):
        main_line = big_buffer[iyear][1]
        args = main_line.split(" ")
        # Remove any empty strings....
        args = [x for x in args if x]
        if verbose:
            print "\n\tYear", big_buffer[iyear][0],
            print "\tmain_line:",main_line,
            print "\targs:",args
        
        #if percentage == '(100.00%)' 8 args if '( 99.99%)' 9
        if len(args) == 8:
            total_cnt += int(args[3])
            candidates += int(args[6])
        else:
            total_cnt += int(args[3])
            candidates += int(args[7])
        if verbose:
            print "\tTweaked:",args
            print "\ttotal_cnt:",total_cnt
            print "\tcandidates:",candidates
        
        if candidates > 0:
            fraction = 100.0*(total_cnt/candidates)
        else:
            fraction = 0.0
        f = "%6.2f%%)" % (fraction)
        if verbose:
            print "\tfraction:",fraction
        
        lap = big_buffer[iyear][2]
        brgs = lap.split(" ")
        # Remove any empty strings....
        brgs = [x for x in brgs if x]
        aa = brgs[1].split("\t")
        lap_cnt += int(aa[0])
        
        reg = big_buffer[iyear][3]
        brgs = reg.split(" ")
        brgs = [x for x in brgs if x]
        aa = brgs[1].split("\t")
        reg_cnt += int(aa[0])
        
        rad = big_buffer[iyear][4]
        brgs = rad.split(" ")
        brgs = [x for x in brgs if x]
        aa = brgs[1].split("\t")
        rad_cnt += int(aa[0])
        
        trb = big_buffer[iyear][5]
        brgs = trb.split(" ")
        brgs = [x for x in brgs if x]
        aa = brgs[1].split("\t")
        trb_cnt += int(aa[0])
        
        if verbose:
            print "\tlap",lap,
            print "\treg",reg,
            print "\trad",rad,
            print "\ttrb",trb,
        
        if iyear == iyears-1:
            if len(args) == 8:
                args[6] = str(candidates)
                args[3] = str(total_cnt)
            else:
                args[3] = str(total_cnt)
                args[7] = str(candidates)
            args[5] = f
            msg = " ".join(args)
            report_save.writelines(msg)
            if verbose:
                print msg

            smsg = dumpline(lap,lap_cnt,candidates,verbose)
            report_save.writelines(smsg)
            if verbose:
                print smsg

            smsg = dumpline(reg,reg_cnt,candidates,verbose)
            report_save.writelines(smsg)
            if verbose:
                print smsg
            
            smsg = dumpline(rad,rad_cnt,candidates,verbose)
            report_save.writelines(smsg)
            if verbose:
                print smsg
            
            smsg = dumpline(trb,trb_cnt,candidates,verbose)
            report_save.writelines(smsg)
            if verbose:
                print smsg
    return total_cnt

