from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
#import math

# Third party module imports

# MCMS module imports

def track_reports_and_plots(tf,fnc_out,readin_centers):

    if tf.save_plots == 1 and tf.save_stats:
        from plot_map import plotmap
        from save_netcdf import Save_NetCDF   
        import numpy 
        from extras import ProgressBarAlt
    elif tf.save_plots == 2 and tf.save_stats:
        from save_netcdf import Save_NetCDF   
        import numpy 
        from extras import ProgressBarAlt

    save_stats = tf.save_stats
    save_plots = tf.save_plots
    might_be_long = tf.might_be_long
    super_total_centers_read = tf.super_total_centers_read
    tracks_file = tf.tracks_file
    clim_tag = tf.clim_tag
    out_path = tf.out_path
    years = tf.years
    lons = tf.lons
    lats = tf.lats
    im = tf.im
    jm = tf.jm
    flag_files = tf.flag_files
    known_flags = tf.known_flags 
    flags_used = tf.flags_used
    fig_format = tf.fig_format
    model = tf.model  
    seasons = tf.seasons
    super_total_tracks,super_total_centers_cnt,touch_concerns,flag_cnt = fnc_out

    #
    # Make Final Report
    #
    print "\tMaking Final Report..."

    report_file = tracks_file.replace("tracks.txt","tracks_report.txt")
    report_file = report_file.replace(out_path,"%sstats/" % (out_path))
    report_save = open(report_file,"w")

    msg1 = "Total Centers Read: %d\nTotal Centers Saved: %d (%6.2f%%)"
    msg1 += "\nTotal Tracks: %d\nDiscards:\n"
    msg = msg1 % (super_total_centers_read,super_total_centers_cnt[0],
                  100.0*(super_total_centers_cnt[0]/super_total_centers_read),
                  super_total_tracks)
    report_save.writelines(msg)

    msg1 = "\t% 6d\t(%6.2f%%)\t%s\n"
    for e in flags_used:
        msg = msg1 % (super_total_centers_cnt[e],
                      100.0*(super_total_centers_cnt[e]/super_total_centers_read),
                      known_flags[e])
        report_save.writelines(msg)

    # Final sanity check that everything is accounted for.
    if super_total_centers_read != sum(super_total_centers_cnt):
        msg = "Final Total Count Error:\n\tsuper_total_centers_read = %d"
        msg += "\n\tsuper_total_centers_cnt = %s sum(%d)"
        sys.exit(msg % (super_total_centers_read,repr(super_total_centers_cnt),
                        sum(super_total_centers_cnt)))
    report_save.close()

    if save_plots:
        # Make frequency plot
        msg = "\tSaving more Stats"
        if save_plots:
            msg += " and more Plots..."
        else:
            msg += " ..."
        pbara = ProgressBarAlt(flag_files,msg,60)
        #pbara = flag_files   
        for flag in pbara:
            for sea in xrange(len(seasons)):
                # Just counts
                # FIX error with numpy/matplot lib and missing so just set to zero now
                #comp_out = numpy.where(flag_cnt[:,sea,flag] < 1.,-10000000000.0,flag_cnt[:,sea,flag])
                #comp_out = numpy.where(flag_cnt[:,sea,flag] < 1.,0.0,flag_cnt[:,sea,flag])
                s = seasons[sea]
                # As Frequency of Canidate Centers
                pname = "%sfigs/maps/global/%s/%s_tf_freq_%s_%s_%s%s"
                pname = pname % (out_path,s,model,flag_files[flag],s,clim_tag,fig_format)
                comp_out = numpy.zeros(im*jm,dtype=numpy.float)
                for i in xrange(im*jm): # loop over each grid
                    if readin_centers[i] > 0:
                        comp_out[i] = numpy.divide(flag_cnt[i,sea,flag],readin_centers[i])
                    else:
                        comp_out[i] = 0.0            
                if save_plots == 1:
                    fplot = plotmap(missing=0.0,color_scheme="jet",lon_0=0.0)
                    fplot.create_fig()
                    fplot.add_field(lons,lats,comp_out,ptype='pcolor',)
                    fplot.finish(pname)
                pname = pname.replace("/figs/maps/global/%s" % s,"/netcdfs")
                pname = pname.replace(fig_format,".nc")
                save_it = Save_NetCDF(comp_out,lons,lats,pname,0,
                    name=flag_files[flag],long_name=flag_files[flag],
                    units='Frequency of Occurrence Kept Among Candidate Centers') 
                #print "\t\tCreated flag %d: %s" % (flag,pname)
                
                # Just Counts
                comp_out = numpy.where(flag_cnt[:,sea,flag] < 1.,0.0,flag_cnt[:,sea,flag])
                pname = "%sfigs/maps/global/%s/%s_tf_%s_%s_%s%s"
                pname = pname % (out_path,s,model,flag_files[flag],s,clim_tag,fig_format)
                if save_plots == 1:
                    fplot = plotmap(missing=0.0,color_scheme="jet",lon_0=0.0)
                    fplot.create_fig()
                    fplot.add_field(lons,lats,comp_out,ptype='pcolor',)
                    fplot.finish(pname)
                pname = pname.replace("/figs/maps/global/%s" % s,"/netcdfs")
                pname = pname.replace(fig_format,".nc")
                save_it = Save_NetCDF(comp_out,lons,lats,pname,0,
                    name=flag_files[flag],long_name=flag_files[flag],
                    units='Counts') 
                #print "\t\tCreated flag %d: %s" % (flag,pname)

            ## Just Counts
            #for sea in xrange(len(seasons)):
                tmp = numpy.copy(touch_concerns[:,sea])
                comp_out = numpy.where(tmp < 1.,0.0,tmp)
                pname = "%sfigs/maps/global/%s/%s_tf_touch_concerns_%s_%s_%s%s"
                pname = pname % (out_path,s,model,seasons[sea],s,clim_tag,fig_format)
                if save_plots == 1:
                    fplot = plotmap(missing=0.0,color_scheme="jet",lon_0=0.0)
                    fplot.create_fig()
                    fplot.add_field(lons,lats,comp_out,ptype='pcolor',)
                    fplot.finish(pname)
                pname = pname.replace("/figs/maps/global/%s" % s,"/netcdfs")
                pname = pname.replace(fig_format,".nc")
                save_it = Save_NetCDF(comp_out,lons,lats,pname,0,
                    name='touch_concerns',long_name='Kept Centers Whose Tracks Pass Troubled Grids',
                    units='Counts')
                #print "\tCreated: %s" % (pname)
