# Standard modules
from itertools import chain

# Third party module imports
    
# MCMS modules

def trim_singletons(o_set,pool,kpool,verbose=0):
    """Check for centers with no overlap. Assume that redundant singletons
    dealt with separately."""
    singletons = set([x for x in o_set if len(o_set[x]) == 1])
    if not singletons: return 0
    # Check for redundant singletons. These only happen because an
    # overlap_set was reduced and an odd single center was left that is
    # not a true singleton (non-overlapping center) because that center can
    # still be found in other overlap_sets. Otherwise, this shouldn't happen
    # due to the actions of correct_overlap_asymmetry.
    for s in list(singletons):
        redundant = [x for x in o_set if s in o_set[x]]
        if len(redundant) > 1:
            if verbose: print "\t\t\tRedundant Singleton %d: %s" % (s,redundant)
            # Just remove these from o_set and singletons
            singletons.remove(s)
            del o_set[s]
    if not singletons: return 0
    # Remaining singletons can be retained as non-overlapping centers.
    if verbose: print "\t\tSingletons (%d): %s" % (len(singletons),singletons)
    # Update pools
    pin = len(pool)
    kin = len(kpool)
    sout = len(singletons)
    pool.difference_update(singletons)
    kpool.update(singletons)
    if pin - sout != len(pool):
        print "Warning Singleton: center_pool(in) not updated correctly"
        print "pin %d, len(pool) %d, sout %d" % (pin,len(pool),sout)
        return 1
    if kin + sout != len(kpool):
        print "Warning Singleton: kept_pool(in) not updated correctly"
        print "kin %d, sout %d, len(kpool) %d" % (kin,sout,len(kpool))
        return 1
    # Trim o_set (because of redundant check these should be clean removes)
    oin = len(o_set)
    membersin = set(chain(*[o_set[x] for x in o_set]))
    for s in singletons: del o_set[s]
    oout = len(o_set)
    membersout = set(chain(*[o_set[x] for x in o_set]))
    if oin - sout != oout:
        print "Warning Singleton: o_set(in) not updated correctly"
        print "oin %d, oout %d, sout %d" % (oin,oout,sout)
        return 1
    if len(membersin) - sout != len(membersout):
        print "Warning Singleton: o_set(in) not updated correctly"
        pmat = "len(membersin) %d, len(membersout) %d, sout %d"
        print pmat % (len(membersin),len(membersout),sout)
        print "\tmembersin",membersin
        print "\tmembersout",membersout
        print "\tsingletons",singletons
        return 1
    return 0

