#!/usr/bin/env python -tt
"""Finds the zonal wavenumber from latitude (degrees,float) and
    wavelength (km). If wavenumber less than 1 (near poles) rounded
    to 1.

    inputs:
    wavelength: find wavenumber for (float) in kilometers.
    latitude: latitude to access wavenumber (float,degrees).

Returns: wavenumber (int)

Examples:

Notes: This should work with any standard installation of python version
       2.4 or greater. I have tested it on Apple OS-X (10.5), Ubuntu (8.04)
       and RedHat Enterprise 4.0 Linux distributions.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2009/1  MB - File created.
"""

import math

def zonal_wavenumber(latitude,wavelength):
    earth_radius = 6371.0002 # Earth's radius (km)
    circumference = 2.0 * math.pi * earth_radius * math.cos(math.radians(float(latitude)))
    wn = int(round(circumference/wavelength))
    if wn < 1:
        wn = 1
    return wn
#---Start of main code block.
if __name__=='__main__':

    latitudes = []
    scale = 1
    inc = (1.0/float(scale))
    lat = -90.0-inc
    for j in range(180*scale + 1):
        lat += inc
        latitudes.append(lat)
    print latitudes

    wavelength = 5000.0
    for lat in latitudes:
        print lat,zonal_wavenumber(lat,wavelength)
