def where_y(source,target,interval,edges=[]):

    """This function locates the y_index (integer) of a target latitude
       by searching the source latitude array.

    Options/Arguments:
        sys -- module
        source -- list of latitudes.
        target -- latitude to find in source.
        interval -- half the grid spacing of source.

    Returns:
        y -- the y_index of source holding target.

    Examples:

    Notes: If the target falls on a boundary between grids, then
           we opt to assign y to the equatorward grid.

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2008/05  MB - File created.
        2008/10  MB - Added input checks, docstring.
        2008/10  MB - Fixed bug in the 'find it' check.
        2011/01  MB - Updated
    """

    import sys
    
    # Given that source are the midpoints of the bin and interval is half
    # the grid/bin spacing, then define the edges as so.
    if not edges:
        edges = dict([(round(i+interval,3),1) for i in source])
        edges.update([(round(i-interval,3),1) for i in source])
        edges = edges.keys()
        edges.sort()
    #print "source"
    #print source
    #print "edges"
    #print edges
    # Find the "bin" that target falls into. If target is on an edge then
    # choose the equatorward grid. If target is a grid midpoint use that.

    if target in source:
        # target in source
        #print "target in source",target,source.index(target)
        y = source.index(target)
        return y,0

    if target in edges:
        # target on an edge
        y = edges.index(target)
        if target > 0.0:
            y -= 1
        #print "target on an edge",target,y
        return y,1
    for j in xrange(len(edges)-1):
        # target between edges
        #print "edges[j] <= target <= edges[j+1]",edges[j],target,edges[j+1],
        if edges[j] <= target <= edges[j+1]:
            y = j
            #FIX/checkf
            #print "*"
            return y,0
        #else:
        #    print 

    # Error check
    sys.exit("Error in where_y %s %s %s %s" % (repr(source),
                                               repr(target),
                                               repr(interval),
                                               repr(hit)))
