#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import textwrap

# Third party module imports

# ----------------------------------------------------------------------------
def WrapHeader(in_string,starter,width=70):
    """
    Breaks a long string to a fixed width (for header info).

    Input:
    ^^^^^
    in_string - String
       Long string to be broken.
    starter - Integer
       Accommodation size for any extra space to be added before in_string.
       That is, for a label such as 'File Description: '. Can be Zero.
    width - Integer
       Length of each new line of text (default 70 characters).
    Output:
    ^^^^^^
    out_string - String
       A reformated version of in_string.

 file_description = ("It was a dark and stormy night; the rain fell in torrents"+
            " -- except at occasional intervals, when it was checked by a" +
            " violent gust of wind which swept up the streets (for it is in" +
            " London that our scene lies), rattling along the housetops, and" +
            " fiercely agitating the scanty flame of the lamps that struggled" +
            " against the darkness.)")


    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/10 MB - File created.
    """
    # ----------------------------------------------------------------------------
    in_string = " "*starter+in_string
    breakup = textwrap.wrap(in_string,width)
    breakup[0] = breakup[0].lstrip()
    breakup = ["#"+x for x in breakup]
    out_string = "\n".join([x for x in breakup])
    out_string = out_string[1:]
    out_string = out_string.replace("#","#\t")

    return out_string

